;#> cliprange.dc1
; Identifier	cliprange
;
; Purpose	determine (clipped) range of a vector
;
; Synopsis	range = cliprange( x, [clip, min=min, max=max, xrange=xrange] )
;
; Arguments	Name	I/O    Type:	 Description
;		-------------------------------------------------------------
;		x	 I	--	 vector of any (sortable) type
;		clip	 I     flt(arr)  2 values: min, max clip fraction
;					 1 value : symmetric clip fraction
;					 default : no clipping
;		xrange	 I     fltarr	 take this portion of the vector
;		min	 I     float	 lo limit is at least this
;		max	 I     float	 hi limit is at most this
;
; Returns	range	 O     fltarr	 2 values [lo,hi] at
;					   (100 * clip) percentiles
;
; Description	The min and max of the vector X is determined where
;		( 100 * clip[0] ) percent of the lowest values and
;		( 100 * ( 1 - clip[1] ) percent of the highest values
;		are ignored.
;
; Comment	--
;
; Example	x = findgen( 100 ) + 10
;		print, cliprange( x, [0.01,0.95], max=100 )
;		   11.0    100.0
;
; Category	UTIL ( module: cliprange )
;
; Filename	cliprange.pro
;
; Author	Do Kester
;		Fred Lahuis
;		Ekki Wieprecht
;
; Version	9.2
;
; History	9.0  2000/01/17 DK, recoded into IDL -> SRE402 cliprange
;		9.1  2000/05/10 FL, proper min/max, check on infinite data
;		9.2  2000/05/25 DK, adjust to pipeline standards
;#<

FUNCTION cliprange, xin0, clip, min=minval, max=maxval, xrange=xrange

compile_opt idl2

if ( n_elements( xrange ) EQ 2 ) then xin = xin0[xrange[0]:xrange[1]] $
else xin = xin0

b = where( finite( xin ), nb )

CASE nb OF
 0 : return, [ 0, 1 ]
 1 : return, [ xin[b] - 1, xin[b] + 1 ]
 ELSE : x = xin[b]
ENDCASE

n = n_elements( x ) - 1

CASE n_elements( clip ) OF
 0 : BEGIN				; don't need sorting here
	x0 = min( x, max=x1 )
	xr = [x0,x1]
	if ( n_elements( minval ) EQ 1 ) then xr[0] = xr[0] > minval
	if ( n_elements( maxval ) EQ 1 ) then xr[1] = xr[1] < maxval
	return, xr			; return range
     END
 1 : BEGIN				; need sorting for next cases
	lo = long( clip * n )
	hi = n - lo
     END
 2 : BEGIN
	lo = long( clip[0] * n )
	hi = long( clip[1] * n )
     END
 ELSE : BEGIN
	lo = 0
	hi = n
	END
ENDCASE

a = sort( x )

xr = [ x[a[lo>0]], x[a[hi<n]] ]
IF ( n_elements( minval ) EQ 1 ) THEN xr[0] = xr[0] > minval
IF ( n_elements( maxval ) EQ 1 ) THEN xr[1] = xr[1] < maxval

RETURN, xr
END
