;#> frin_model.dc3
; Identifier	frin_model
;
; Purpose	calculate the fringe model
;
; Synopsis	result = frin_model( user, par, prior=prior )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		user	I/O    struct	user struct (see frin_search)
;		par	 I     fltarr	list of parameters (fringe cycles)
;		prior	 I     ftlarr	prior info (not used)
;
; Returns	result	 O     float	chi-square
;
; Description	chi-sq of the model fit is returned. Within the user-struct
;		also the amplitude of the cosine and sine is returned for all
;		fringe cycles in par. And in addition the covariance matrix
;		is returned, with parameter order:
;		  par[0], cosamp[0], sinamp[0], par[1], cosamp[1], ... etc
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_model.pro
;
; Author	Do Kester
;
; Version	1.3
;
; History	1.0  15-03-2001 DK, first implementation of frin_model.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  16-05-2001 DK, repaired IDL5.2 bug
;               1.3  03-07-2003 FL, /double in ludc call and math error catch
;#<
;===========================================================================

function frin_model, user, par, prior=prior

; compile_opt idl2, hidden

catch, error_status

if error_status ne 0 then begin
  catch,/Cancel
  print,'Error in frin_model', error_status
  print,'Error message: ', !ERROR_STATE.MSG
  return,-1
endif

np = n_elements( par )

nn = n_elements( user.wn )
onef = dblarr( 2*np ) + 1.0
ones = dblarr( nn ) + 1.0
awt = onef # user.wt

frwn = [par] # ( 2 * !PI * user.wn )

csp = dblarr( 2*np, nn )
k2 = indgen( np ) * 2
csp[k2,*]   = cos( frwn )
csp[k2+1,*] = sin( frwn )

mat = ( awt * csp ) # transpose( csp )
vec = total( ( onef # ( user.wt * user.xx ) ) * csp, 2 )

ludc, mat, index, /double, interchanges=parity
sol = lusol( mat, index, vec )

user.fringe = sol # csp

chisq = total( user.wt * ( user.xx - user.fringe )^2 )

user.ac[0:np-1] = sol[k2]
user.as[0:np-1] = sol[k2+1]

ptr_free, user.mat
csw = dblarr( 3*np, nn )
k3 = indgen( np ) * 3
csw[k3,*] = ( ( sol[k2+1] # ones ) * csp[k2,*] - $
	      ( sol[k2]   # ones ) * csp[k2+1,*] ) * $
	    ( ( dblarr( np ) + 1.0 ) # ( 2 * !PI * user.wn ) )
csw[k3+1,*] = csp[k2,*]
csw[k3+2,*] = csp[k2+1,*]

user.mat = ptr_new( ( ( ( dblarr( 3*np ) + 1.0 ) # user.wt ) * csw ) # $
		    transpose( csw ) )

return, chisq
end


