; $Id: af_finite.pro,v 1.5 2006/01/18 06:57:00 baba Exp $
;
; Copyright (c) 2002-2005, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   af_finite.pro
;
; PURPOSE:
;   Special values, such as NaN, Inf, are converted to float.
;
; CATEGORY:
;   Function. Library for ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = af_finite(indata, outdata, [value=value])
;
; INPUTS:
;   indata      input data array
;   value       value to be converted from special values.
;               -9999. if not specified. (optional.)
;
; OUTPUTS:
;   outdata     output data array (double)
;
; RETURN:
;    0 if function ended successfully.
;   -1 in failure.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;       result = af_finite (vdet, finitevdet)
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   Jun. 2003    Written by A. Kawamura.
;   Jul. 2005    Last modified by Makiuti
;-

FUNCTION af_finite, indata, outdata, value = value
;ON_ERROR 2

IF N_PARAMS(0) EQ 0 THEN BEGIN
    MESSAGE,'FUNCTION', /CONTINUE
    PRINT,'ret = af_finite(indata, outdata[,value=value])'
    PRINT,'            indata = input data array'
    PRINT,'            value = value to be converted from special values.$'
    PRINT,'                    -9999. if not specified. (optional)'
    PRINT,'            outdata = output data array'

    RETURN, -1
ENDIF

;;; Reading input data ;;;
  LMESSAGE, 5, 'AF_FINITE was called...', /CONTINUE

  si = SIZE(indata)
  outdata = indata

;;; Set default value to replace NaN and Inf
  sp_value = -9999.

;;; Check value to be replaced
  IF KEYWORD_SET(value) EQ 0 THEN value = sp_value
  IF ( SIZE(value,/TYPE) LT 2 ) AND $
     ( SIZE(value,/TYPE) GT 4 ) THEN BEGIN
     value = sp_value
     LMESSAGE, 3, 'Warning:Type of value is invalid. -9999. was input instead.', /CONTINUE
  ENDIF

;;; Search for NaN ;;;;
  LMESSAGE, 7, 'Searching for NaN', /CONTINUE
  ind_nan = WHERE (FINITE(indata, /NAN), count_nan)
  msg=STRCOMPRESS('Number of NaN ... '+STRING(count_nan))
  LMESSAGE, 7, msg, /CONTINUE

  LMESSAGE, 7, 'Searching for Infinity', /CONTINUE
  ind_inf = WHERE (FINITE(indata, /INFINITY), count_inf)
  msg=STRCOMPRESS('Number of INFINITE ... '+STRING(count_inf))
  LMESSAGE, 7, msg, /CONTINUE

  IF count_nan NE 0 THEN outdata[ind_nan] = value
  IF count_inf NE 0 THEN outdata[ind_inf] = value

  ;; Display a result
  IF ((count_nan NE 0) OR (count_inf NE 0)) THEN BEGIN
    msg = 'NaN and/or INFINITY is replaced with ' + strcompress(string(value))
    LMESSAGE, 5, msg, /CONTINUE
  ENDIF

  outdata = double(outdata)

  RETURN, 0

END
