; $Id: cnv_aftime.pro,v 1.4 2007/08/28 06:03:27 makiuti Exp $
;
; Copyright (c) 2005-2007, ASTRO-F Data Reduction Team. All rights reserved.
;
;+
; NAME:
;   cnv_aftime.pro
;
; PURPOSE:
;   Convert AFTIME and Date(Time) mutually
;
; CATEGORY:
;   Function. Library procedure for ASTRO-F pipeline (GB).
;
; CALLING SEQUENCE:
;   (1-1) aftime = cnv_aftime(Year, Month, Day, Hour, Minute, Second)
;   (1-2) aftime = cnv_aftime('yyyy/mm/dd hh:mm:ss.sss')
;   (2)   date   = cnv_aftime(AFtime[, /FITS_FMT, JD=jd]) ;; AFTime must be double
;
;     /FITS_FMT  ; Return the string of date in FITS standard form. (YYYY-MM-DDThh:mm:ss.sss)
;     JD=jd      ; Set this keyword to a named variable that will contain the Julian Day Number
;
; INPUTS:
;   AFtime   double value (or array) of AFTIME
;     or
;   Date     string of date and time
;
; RETURN:
;   Result of conversion
;
; OUTPUT
;   AFTIME value (double) or Date (string)
;   Julian Day Number (option)
;
; EXAMPLE:
;    IDL> aftime = cnv_aftime(2006, 4, 15, 20, 12, 45.245)
;    IDL> aftime = cnv_aftime('2006/04/15 20:12:45.245')
;    IDL> date = cnv_aftime(198447165.245d, /FITS_FMT, JD=jd)
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   Jul. 2005    Written by Makiuti
;   Aug. 2007    Changed to consider a leap second
;-

;;; Convert Date string into AFTIME
FUNCTION d2aft, year, month, day, hour, minute, second, $
           JD=jd_afdate  ; Set this keyword to a named variable that will contain the Julian Day Number

  jd_afdate = JULDAY(month, day, year, hour, minute, second)
  jd_org = double(JULDAY(1,1,2000))

  aftime = (jd_afdate -jd_org + 12.0/24.0) * 60. * 60. *24

  ; Adjust a leap second at 2006/01/01 00:00:00 UTC
  IF (aftime GE 189388800.0d) THEN BEGIN
    IF ((aftime LT 189388801.0d) AND (second GE 60.0)) THEN BEGIN
        print, '!! leap second !!'
    ENDIF ELSE BEGIN
      aftime = aftime + 1
    ENDELSE
  ENDIF

  Return, aftime

END

;;; Convert AFTIME into Date string
FUNCTION aft2d, $
           aftime, $             ; AFTIME (double); A numeric value or array
           FITS_FMT=fits_fmt, $  ; Return the string of date in FITS standard form. (YYYY-MM-DDThh:mm:ss.sss)
           JD=jd_aftime          ; Set this keyword to a named variable that will contain the Julian Day Number

  ;;;;;;
  ; Adjust a leap second at 2006/01/01 00:00:00 UTC
  ;;;;;;
  ; make index after 2006/01/01 00:00:00 UTC
  idx_aftl = where(aftime GE 189388801.0d, cnt_aftl)
  ; make index at 2005/12/31 23:59:60.?
  idx_leap = where((aftime GE 189388800.0d) AND (aftime LT 189388801.0d), cnt_leap)

  adj_sec = 0.0d ; offset for a leap second; usually zero

  IF (cnt_aftl GT 0) THEN BEGIN
    aftime[idx_aftl] = aftime[idx_aftl] - 1
  ENDIF
  IF (cnt_leap GT 0) THEN BEGIN
    aftime[idx_leap] = aftime[idx_leap] - 1
    adj_sec = 1.0d ; offset for a leap second
  ENDIF
  ;;;;;;

  jd_aftime = (double(aftime)/60./60./24. + double(JULDAY(1,1,2000)) - 12.0/24.0)
  CALDAT, jd_aftime, Month, Day, Year, Hour, Minute, Second

  IF (cnt_leap GT 0) THEN BEGIN
    Second[idx_leap] = Second[idx_leap] + adj_sec ; for a leap second 2005/12/31 23:59:60.?
  ENDIF

  IF Keyword_Set(FITS_FMT) THEN BEGIN;; Return date in FITS standard form
    afdate = strcompress(string(Year), /REMOVE_ALL)+'-'+string(Month, format='(i2.2)')+'-'+string(Day, format='(i2.2)')+'T'+$
           string(Hour, format='(i2.2)')+':'+string(Minute, format='(i2.2)')+':'+$
           strcompress(string(Second, format='(f06.3)'), /REMOVE_ALL)
  ENDIF ELSE BEGIN ;; Return date in Default format
    afdate = strcompress(string(Year), /REMOVE_ALL)+'/'+string(Month, format='(i2.2)')+'/'+string(Day, format='(i2.2)')+' '+$
           string(Hour, format='(i2.2)')+':'+string(Minute, format='(i2.2)')+':'+$
           strcompress(string(Second, format='(f06.3)'), /REMOVE_ALL)+' (UTC)'
  ENDELSE

  Return, afdate ;; Return string(s) of date and time

END

;;; Main procedure
FUNCTION cnv_aftime, arg1, arg2, arg3, arg4, arg5, arg6, $
             FITS_FMT=fits_fmt, $ ; Return the string of date in FITS standard form. (YYYY-MM-DDThh:mm:ss.sss)
             JD=jd                ; Set this keyword to a named variable that will contain the Julian Day Number

  ON_ERROR, 2

  IF N_Params() EQ 1 THEN BEGIN
    IF Size(arg1, /type) EQ 7 THEN BEGIN
      Reads, arg1, year, month, day, hour, minute, second, $
             format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f6.3)'
      aftime = d2aft(year, month, day, hour, minute, second, JD=jd)
      Return, aftime
    ENDIF
    IF Size(arg1, /type) NE 5 THEN Message, '### The argument (aftime) must be double. ###'
    afdate = aft2d(arg1, FITS_FMT=fits_fmt, JD=jd)
    Return, afdate
  ENDIF ELSE IF N_Params() EQ 6 THEN BEGIN
    aftime = d2aft(arg1, arg2, arg3, arg4, arg5, arg6, JD=jd)
    Return, aftime
  ENDIF ELSE BEGIN
    print, "*** Usage ***"
    print, "  Convert from Date to AFTIME(1): aftime = cnv_aftime(Year, Month, Day, Hour, Minute, Second)"
    print, "  Convert from Date to AFTIME(2): aftime = cnv_aftime('yyyy/mm/dd hh:mm:ss.sss')"
    print, "  Convert from AFTIME to Date   : date   = cnv_aftime(AFtime[, /FITS_FMT, JD=jd]) ;; AFTime must be double"
    print, "  ex.)"
    print, "     IDL> aftime = cnv_aftime(2006, 4, 15, 20, 12, 45.245)"
    print, "     IDL> aftime = cnv_aftime('2006/04/15 20:12:45.245')"
    print, "     IDL> date = cnv_aftime(198447165.245d, /FITS_FMT, JD=jd)"
    return, -1
  ENDELSE

END