; $Id: gb_diff2p.pro,v 1.11 2006/10/16 02:13:46 makiuti Exp $
;
; Copyright (c) 2002-2006, ASTRO-F Data Reduction Team. All rights reserved.
;
;+
; NAME:
;   gb_diff2p.pro
;
; PURPOSE:
;   Input data is differentiated.
;
; CATEGORY:
;   Function. Green Box procedure in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = gb_diff2p(tsd, vdet, diff_vdet
;            [, ferr=ferr, shiftvl=shiftvl])
;
; INPUTS:
;   tsd        object reference of TSD
;   vdet       array of detector value ('FLUX')
;   shiftvl    should be +1(default) or -1
;
; OUTPUTS:
;   diff_vdet  array of differentiated detector value ('FLUX')
;   ferr       error of detector value ('FERR')
;
; RETURN:
;   !SUCCEED if function ended successfully.
;   !FAIL in failure.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;       result = gb_diff2p(tsd, vdet, diff_vdet)
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   May  2003    Written by A. Kawamura.
;   Oct. 2006    Last modified by Makiuti
;-
FUNCTION gb_diff2p, tsd, vdet, diff_vdet, $
                    ferr=ferr, shiftvl=shiftvl
;ON_ERROR 2

IF N_PARAMS(0) EQ 0 THEN BEGIN
    MESSAGE,'FUNCTION gb_diff2p, tsd, vdet, diff_vdet $',/CONTINUE
    MESSAGE,'      [,ferr=ferr, shiftvl = shiftvl]',/CONTINUE
    MESSAGE,'INPUTS:',/CONTINUE
    MESSAGE,'      tsd:         Object reference type of TSD.',/CONTINUE
    MESSAGE,'      vdet:        array of detector value.',/CONTINUE
    MESSAGE,'OUTPUT:',/CONTINUE
    MESSAGE,'      diff_vdet:   array of differentiated detector value.',/CONTINUE
    RETURN, !FAIL
ENDIF

;;; Start Differentiation ;;;
   LMESSAGE, 7, 'GB_Diff2p called for differentiation...', /CONTINUE

;;; number of channels ;;;
   si = SIZE(vdet)
     arr_dim  = si[0]
     num_ch   = si[1]
     num_data = si[2]

;;; Check input parameters ;;;

IF (arr_dim LT 1) OR (arr_dim GT 2) THEN BEGIN

   LMESSAGE, 3, 'Data array is 1-D or 2-D. Not FIS data?', /CONTINUE
   RETURN,!FAIL

ENDIF

    ;;-----------------------
    ;; Check packetid
    ;;-----------------------
    ;;; If CDS only, just return.
    packetid = tsd->Get_Value_TSD('FIS_OBS','PacketID')
    dummy = packetid - shift(packetid, -1)

     size_dummy = SIZE(dummy)
     dummy = reform(dummy[1:size_dummy[1]-1])
     ind_packetid = WHERE(dummy NE 0, cnt_packetid)

    IF cnt_packetid EQ 0 THEN BEGIN

      msg = "PacketID is constant..."
      LMESSAGE, 7, msg, /CONTINUE

      ;; If CDS mode
      ;IF packetid[0] EQ 67 or packetid[0] EQ 83 THEN BEGIN ; at IT#2
      ;IF packetid[0] EQ 65 or packetid[0] EQ 81 THEN BEGIN ; from EFC#5

      aftime = tsd->Get_Value_TSD('FIS_OBS','AFTIME')
      inst = tsd->Get_header_keyword_tsd('PRIMARY','INSTRUME')

      IF smpl_mode(packetid[0], AFTIME=aftime[0], TYPE=inst) EQ 1 THEN BEGIN
        LMESSAGE, 7, "Only data with CDS mode...", /CONTINUE
        diff_vdet = vdet
        return, !SUCCEED
        ;return, !FAIL
      ENDIF

    ENDIF
    ;;; End of checking packetid ;;;


;; Copy FErr
ferr_org = ferr

;;----------------------------------------------------------
;; Differentiate with 2-point
;;----------------------------------------------------------
IF (KEYWORD_SET(shiftvl) EQ 0)  THEN shiftvl = +1

IF (shiftvl EQ +1) THEN BEGIN
   msg = 'Executing differentiation by shifting one sampling to the right...'
   LMESSAGE, 7, msg, /CONTINUE

   IF arr_dim EQ 1 THEN BEGIN

     LMESSAGE, 3, 'Data is one dimentional array', /CONTINUE
     diff_vdet = vdet - shift(vdet, shiftvl)

     IF n_elements(ferr_org) NE 0 THEN BEGIN ; Calc. of propagation of error
       ferr = ferr_org * sqrt(2)
     ENDIF

   ENDIF ELSE BEGIN

     diff_vdet = dblarr(num_ch, num_data)
     diff_vdet = vdet - shift(vdet, 0, shiftvl)

     IF n_elements(ferr_org) NE 0 THEN BEGIN ; Calc. of propagation of error
       ferr = ferr_org * sqrt(2)
     ENDIF

   ENDELSE

ENDIF ELSE BEGIN                       ;;; shiftv1=-1
   msg = 'Executing differentiation by shifting one sampling to the left..(Process is NOT Default)'
   LMESSAGE, 5, msg, /CONTINUE

   IF arr_dim EQ 1 THEN BEGIN

     LMESSAGE, 3, 'Data is one dimentional array', /CONTINUE
     diff_vdet = shift(vdet, shiftvl) - vdet

     IF n_elements(ferr_org) NE 0 THEN BEGIN ; Calc. of propagation of error
       ferr = ferr_org * sqrt(2)
     ENDIF

   ENDIF ELSE BEGIN

     diff_vdet = dblarr(num_ch, num_data)
     diff_vdet = shift(vdet, 0, shiftvl) - vdet

     IF n_elements(ferr_org) NE 0 THEN BEGIN ; Calc. of propagation of error
       ferr = ferr_org * sqrt(2)
     ENDIF

   ENDELSE

ENDELSE
;;; END of differentiation with 2-point  ;;;;

;;----------------------------------------------------------
;; Check CDS modes
;;----------------------------------------------------------
;;; In case of CDS, NO differentiation should be applied. ;;;
    aftime = tsd->Get_Value_TSD('FIS_OBS','AFTIME')
    inst = tsd->Get_Header_Keyword_TSD('PRIMARY','INSTRUME')
    ; Get SEQ (sampling) pattern indicator array
    seq_mode = smpl_mode(packetid, AFTIME=aftime[0], TYPE=inst)
    ;
    ind = WHERE(seq_mode EQ 1, cnt) ; CDS mode
    IF cnt NE 0 THEN BEGIN
      ; scale factor to correct the difference of exposure (integration) time
      ; between CDS and other modes.
      IF (num_ch EQ 100) THEN sf = 39.55/25.63 ELSE sf = 59.32/45.41
      ;
      diff_vdet[*,ind] = vdet[*,ind] * sf
      ferr[*,ind] = ferr_org[*,ind] * sf
    ENDIF
;;;END checking CDS modes ;;;


;;----------------------------------------------------------
;; Set flags in TSD, 'Arith_err', 'No_Diff'
;;----------------------------------------------------------

    ;;-----------------------
    ;; Search for 'arith_err'
    ;;-----------------------
    LMESSAGE, 7, "Setting Arith_Err...", /CONTINUE

    arith_err = tsd->Get_Pix_Flag_TSD('FIS_OBS','Arith_err')

    ind_nan = where(FINITE(diff_vdet, /NAN), count_nan)
    ind_inf = where(FINITE(diff_vdet, /INFINITY), count_inf)

    msg = STRCOMPRESS('Number of NaN ... '+STRING(count_nan))
    LMESSAGE, 7, msg, /CONTINUE
    msg = STRCOMPRESS('Number of INFINITE ... '+STRING(count_inf))
    LMESSAGE, 7, msg, /CONTINUE

    IF (count_nan NE 0) THEN arith_err[ind_nan] = 1
    IF (count_inf NE 0) THEN arith_err[ind_inf] = 1
    ind = where(arith_err NE 0, cnt)
    IF (cnt NE 0) THEN arith_err[ind] = 1
    ret = tsd->Set_Pix_Flag_TSD('FIS_OBS','Arith_err', arith_err)

    ;;-----------------------
    ;; Set 'no_diff' flag
    ;;-----------------------

    pf_no_diff = tsd->Get_Pix_Flag_TSD('FIS_OBS','No_Diff')
    pf_bad = tsd->Get_Pix_Flag_TSD('FIS_OBS','Bad')

    ; frame flag
    frm_f_bad = tsd->Get_Flag_TSD('FIS_OBS','Bad_Frame')
    ; pixel flags
    pf_undef_anom = tsd->Get_Pix_Flag_TSD('FIS_OBS','Undef_Anom')
    pf_satur = tsd->Get_Pix_Flag_TSD('FIS_OBS','Saturate')
    pf_reset = tsd->Get_Pix_Flag_TSD('FIS_OBS','Reset')
    pf_rstanom = tsd->Get_Pix_Flag_TSD('FIS_OBS','RstAnom')
    pf_no_rp_corr = tsd->Get_Pix_Flag_TSD('FIS_OBS','No_RP_Corr')

    ; Set No Good frames for differentiation
    ng_frm_for_diff = pf_undef_anom + pf_satur + pf_rstanom + pf_no_rp_corr + arith_err ; NG (bad) frame
        ; 'bad' flag cannot be used here, because all 'dead' pixels have been set as 'bad'.
        ; The differentiation module should differentiate dead pixels.
    FOR ch = 0, num_ch-1 DO ng_frm_for_diff[ch, *] = ng_frm_for_diff[ch, *] + frm_f_bad

    idx = where(ng_frm_for_diff NE 0, cnt)
    If (cnt NE 0) THEN ng_frm_for_diff[idx] = 1

    f_value1 = bytarr(num_ch, num_data)
    f_value2 = bytarr(num_ch, num_data)

    IF shiftvl EQ +1 THEN BEGIN ;Default
      LMESSAGE, 7, 'Setting no_diff at the left end of the data', /CONTINUE
        f_value1[*, 0] = 1
      LMESSAGE, 7, 'Setting flag, no_diff, with "NG for diff" frame', /CONTINUE
        ; no_diff at NG frame and its right frame
        f_new_no_diff = ng_frm_for_diff + SHIFT(ng_frm_for_diff,0,1)
    ENDIF ELSE BEGIN
      LMESSAGE, 7, 'Setting no_diff at the right end of the data', /CONTINUE
        f_value1[*, num_data-1] = 1
      LMESSAGE, 7, 'Setting flag, no_diff, with "NG for diff" frame', /CONTINUE
        ; no_diff at NG frame and its left frame
        f_new_no_diff = ng_frm_for_diff + SHIFT(ng_frm_for_diff,0,-1)
    ENDELSE

    IF cnt_packetid NE 0 THEN BEGIN
      LMESSAGE, 7, "Setting no_diff where packet ID changes...", /CONTINUE
      f_value2[*, ind_packetid] = 1
    ENDIF

    f_new_no_diff =  f_new_no_diff + f_value1 + f_value2 + pf_reset

    idx = where(f_new_no_diff NE 0, cnt)
    If (cnt NE 0) THEN f_new_no_diff[idx] = 1
    LMESSAGE, 7, 'Setting flag, no_diff...', /CONTINUE
      ret = tsd->Set_Pix_Flag_TSD('FIS_OBS','No_Diff', f_new_no_diff)

;;;; 'bad' will be set in gb_differentiation.pro
;    f_new_bad = pf_bad + f_new_no_diff
;    idx = where(f_new_bad NE 0, cnt)
;    IF cnt NE 0 THEN f_new_bad[idx]=1
;    LMESSAGE, 7, 'Setting flag, bad...', /CONTINUE
;      ret = tsd->Set_Pix_Flag_TSD('FIS_OBS','Bad', f_new_bad)

  RETURN, !SUCCEED

END
