; $Id: getcal_fit.pro,v 1.2 2006/10/06 08:46:24 makiuti Exp $
; Copyright (c) 2006, ASTRO-F Data Reduction Team. All rights reserved.
;
;+
; NAME:
;   GETCAL_FIT
;
; PURPOSE:
;       This function receives differenciated signals of the FIS, and
;       integrate them. Then this make linear fit for 4-points(frames).
;       This returns the peak value of ramp slopes as a responsivity indicator.
;
; CATEGORY:
;   Function. Green Box library in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = getcal_fit(influx, nouse, SLOPE=slope, SIGMA=sigma, /FITERRON, /DISP)
;
; INPUTS:
;    influx : Differenciated signals of one channel of the FIS (one-dimensional array).
;           ; This array is expected to include single Cal-On signal.
;    nouse  : Unusable flag (reset, bad, etc.)
;    SLOPE  : Ramp slopes (Fitting result)
;    SIGMA  : Error of ramp slopes (Fitting result)
;    /FITERRON : Add Error of fitting into a return value (return[1])
;              : Usually the return value doesn't contain fitting error 
;              :  but contains rms of base level (at where Cal Off),
;              :  because fitting error become large due to non-linear ramp curve.
;    /DISP     : Option to display of fitting result (slope and error) for each Cal
;
; OUTPUTS:
;
; RETURN:
;   Two elements array will be returned.
;     return[0] : Peak value of ramp slopes
;     return[1] : Error level
;
; MODIFICATION HISTORY:
;       Sep. 2006    Written by Makiuti.
;       Oct. 2006    Last modified
;-
FUNCTION GetCal_Fit, influx, nouse, SLOPE=slope, SIGMA=sigma, FITERRON=fiterron, DISP=disp

  IF (n_elements(nouse) EQ 0) THEN BEGIN ; If there is no 'nouse' (indicating unusable frame) array
    nouse = fltarr(n_elements(flux))
  ENDIF

  dum = where(nouse EQ 0, cnt_ok)
  IF (cnt_ok LE 0) THEN return, [0,0] ; If all data cannot be used

  IF (n_elements(influx) LT 30) THEN return, [0,0] ; Not enough data

  flux = influx
  idx = where(nouse EQ 1, cnt_nouse)
  IF (cnt_nouse GT 0) THEN  flux[idx] = 0.0
  index = lindgen(n_elements(flux))

  rev_diff = flux
  rev_diff[*] = 0.0

  ; Integrate influx array
  FOR i=1, n_elements(flux)-1 DO BEGIN
    IF (flux[i] NE 0) THEN BEGIN
      rev_diff[i] = rev_diff[i-1] + flux[i]
    ENDIF ELSE BEGIN
      rev_diff[i] = 0.0 ; ZbgOb`Őϕs\ɂȂƂO
    ENDELSE
  ENDFOR

  ;; # for check
  ;plot, rev_diff, psym=-4

  ; Arrays for results of linear fitting
  slope = fltarr(n_elements(index))
  slope[*] = !Values.F_NaN
  sigma = fltarr(n_elements(index))
  sigma[*] = !Values.F_NaN

  ; linear fit with 4-points
  FOR i=0, n_elements(index)-4 DO BEGIN
    x = index[i:i+3]
    y = rev_diff[i:i+3]
    ind = where(y NE 0, cnt)

    IF (cnt EQ 4) THEN BEGIN ; 4t[ׂĎgꍇ
      ret = LINFIT(x[ind], y[ind], SIGMA=sgm)
      slope[i] = ret[1] ; `tBbgʂ̌X
      sigma[i] = sgm[1]
    ENDIF ELSE IF (cnt EQ 3) THEN BEGIN ; 4t[1t[0
      IF (total(ind EQ 0) EQ 0) THEN BEGIN ; 1stt[̒l0
        fnum = [0,1,2,3]
        ret = LINFIT(x[fnum], y[fnum], SIGMA=sgm)
        slope[i] = ret[1]
        sigma[i] = sgm[1]
      ENDIF ELSE IF (total(ind EQ 1) EQ 0) THEN BEGIN ; 2ndt[̒l0
        fnum = [1,2,3]
        ret = LINFIT(x[fnum], y[fnum], SIGMA=sgm)
        slope[i] = ret[1]
        sigma[i] = sgm[1]
      ENDIF ELSE IF (total(ind EQ 3) EQ 0) THEN BEGIN ; 4tht[̒l0
        fnum = [0,1,2]
        ret = LINFIT(x[fnum], y[fnum], SIGMA=sgm)
        slope[i] = ret[1]
        sigma[i] = sgm[1]
      ENDIF ELSE BEGIN ; 3rdt[̒l0
        slope[i] = !Values.F_NaN
        sigma[i] = !Values.F_NaN
      ENDELSE
    ENDIF ELSE BEGIN; 4t[2t[ȏオ0
      slope[i] = !Values.F_NaN
      sigma[i] = !Values.F_NaN
    ENDELSE
  ENDFOR

  ; Sort in descending order
  temp_idx = sort(slope)
  sorted_slope = slope[reverse(temp_idx)]
  sorted_sigma = sigma[reverse(temp_idx)]
  ; remove NaN
  sorted_slope = sorted_slope[where(finite(sorted_slope) EQ 1)]
  sorted_sigma = sorted_sigma[where(finite(sorted_sigma) EQ 1)]


  ;; # for check
  IF keyword_set(disp) THEN BEGIN
    ; plot data
    plot, slope, psym=-4
    errplot, slope+sigma, slope-sigma
    ;; plot sorted data
    ;plot, sorted_slope, psym=-4
    ;errplot, sorted_slope+sorted_sigma, sorted_slope-sorted_sigma
  ENDIF

  ; Estimate Cal lamp level and error
  calval  = sorted_slope[0] ; largest (peak) value
  baseval = median(sorted_slope[20:n_elements(sorted_slope)-5]) ; base (dark) level
  cal_measured = calval - baseval ; Estimated value for Cal-lamp
  base_error = stddev(sorted_slope[20:n_elements(sorted_slope)-5]) ; rms of base level

  IF keyword_set(fiterron) THEN BEGIN
    error = sorted_sigma[1] + base_error
  ENDIF ELSE BEGIN
    error = base_error
    ; (In case of the SW, ) Fitting error is large due to short cal ON.
    ; Then we regards only rms of base signals as error of Cal lamp level.
  ENDELSE

  Return, [cal_measured, error]

END
