;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_check_glitch_tsd
; PURPOSE:
;       remove possible sources from glitch TSD
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
; 
; INPUTS:
;       data          input glitch tsd
;       proj          projected mask for MMT coefficients
;       sigma         noise level 
;       sep_threshold threshold to extract individual signals 
;       sep_limit     limit on separation between individual signals
;       ratio_limit   limit on the width ratio to identify glitch
;                     tails
;       fraction_at_width   the fraction of signal at which the widths
;                           are calculated. 
;       source_size   size in readout for sources
;
; OUTPUTS:
;       data          output (modified)
;       glitch_measures  a structure for measured values
;       flg_dip       flag for identified dippers
;       flg_fad       flag for identified faders
;       flg_src       flag for identified strong sources
;
; RETURN
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
;	2. The after effects are corrected with the transient correction.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Mar. 2004		Written by T.Takagi
;    
;-


PRO mmt_check_glitch_tsd, data, proj, sigma, sep_threshold, sep_limit, $
                            ratio_limit, fraction_at_width, source_size,$
                            flg_dip, flg_fad, flg_src,pspikes,$
                            glitch_measures=glitch_measures

;- remove small signals from glitch 
id = where(abs(data) le sep_threshold,cnt)
if (cnt ge 1) then data[id] = 0

;- mask to identify individual signals in given TSD 
mask = mmt_isolate_signals(data, sep_threshold, sep_limit=sep_limit)

n_signal = max(mask)
glitch_measures = mmt_define_glitch_measures(1,n_signal=n_signal)

;;;- check differentiation 
diff = deriv(data)    
sig_diff = sigma * sqrt(2)
diff = diff/sig_diff

for i=1, n_signal do begin
    fader=0
    dipper=0
    isGlitch = 1 ;; first, assume that this is glitch
    isSource = 0 

    id = where(mask eq i)
    sub_tsd = data[id]
    

    ;; integrate 
    sum=total(sub_tsd)
    if (sum eq 0.) then begin 
        isGlitch = 0
        goto, finish
    endif
    n_data= n_elements(sub_tsd)
    if (n_data le sep_limit) then begin
        isGlitch = 0
        goto, finish
    endif

    if (sum gt 0) then begin
        max_id = mmt_get_peak(sub_tsd)
        max_proj = mmt_get_peak(proj[id])
        fader = 1
    endif else if (sum lt 0) then begin
        max_id = mmt_get_peak(-sub_tsd)
        max_proj = mmt_get_peak(-proj[id])
        dipper = 1
    endif

    ;; reject fader without spike
    if (fader eq 1) then begin 
        sp_mask = mask[pspikes]
        dum = where(sp_mask eq i,cnt)
        if (cnt eq 0) then begin 
            isGlitch = 0
            goto, finish
        endif
    endif

;- maximum differentiation
    max_diff = max(abs(diff[id]))

;;;
;;; Iteration for two positions for possible peak
;;;
    for ite=1,2 do begin 
        if (ite eq 1) then max_id = max_id
        if (ite eq 2) then begin
            if (max_proj eq max_id) then goto, finish
            max_id = max_proj
            isGlitch = 1
        endif
    
        sum1=total(sub_tsd[0:max_id])
        sum2=total(sub_tsd[max_id:*])

        mmt_get_signal_width, sub_tsd, max_id, $
          width1, width2, frac=fraction_at_width
    
        

        if (width1 eq 0) then begin
            w_ratio = width2
        endif else begin 
            w_ratio = 1.*width2/width1
        endelse

        sn = abs(sub_tsd[max_id])/sigma
        duration = width1+width2
        glitch_frac = 1.*duration/n_elements(sub_tsd)        

    ;- remove symmetrical signals
        if (w_ratio lt ratio_limit) then begin 
            isGlitch = 0
        endif

    ;- remove weak signals
        if (sn le 1.) then begin 
            isGlitch = 0
        endif

    ;- remove short positive signals
        if (fader eq 1 and duration le source_size) then begin 
            isGlitch = 0
        endif

    ;- remove short duration signals mainly for dippers
        if (duration le 5) then begin 
            isGlitch = 0
        endif

        ;- remove extended sources 
        if (fader eq 1 and max_diff le 0.2) then begin 
            isGlitch = 0
        endif

        ;; strong source detection to avoid positive spike detection
        sn_limit = 10.
        if (duration le source_size and fader eq 1 and $
            sn ge sn_limit and w_ratio le ratio_limit) then begin 
            isSource = 1
            isGlitch = 0
        endif        

        ;; Remove extended strong source from glitch list
        sn_arr = sub_tsd/sigma
        id_src = where(sn_arr ge sn_limit, cnt)
        if (cnt ge source_size and fader eq 1) then begin 
            isGlitch = 0
            isSource = 1
        endif

;    isGlitch = 1 ; for debug
        
        glitch_measures.isGlitch[i-1] = isGlitch
        glitch_measures.id_start[i-1] = id[0]
        glitch_measures.max_id[i-1] = max_id
        glitch_measures.proj[i-1] = proj[id[max_id]]
        glitch_measures.sigma[i-1] = sigma
        glitch_measures.width1[i-1] = width1
        glitch_measures.w_ratio[i-1] = w_ratio
        glitch_measures.sn[i-1] = sn
        glitch_measures.duration[i-1] = duration
        glitch_measures.size[i-1] = n_elements(sub_tsd)
        glitch_measures.integral[i-1] = sum
        glitch_measures.int1[i-1] = sum1
        glitch_measures.int2[i-1] = sum2
        glitch_measures.glitch_frac[i-1] = glitch_frac
        glitch_measures.max_diff[i-1] = max_diff
        glitch_measures.dipper[i-1] = dipper
        glitch_measures.fader[i-1] = fader


;;-   For debug     
;    if (isGlitch eq 1) then begin 
;        !p.multi=[0,1,2]
;        plot, sub_tsd
;        xyouts, 'iteration = '+strcompress(ite)
;        tmpx = [max_id, max_id]
;        tmpy = [sub_tsd[max_id],sub_tsd[max_id]]
;        oplot, tmpx, tmpy, psym=2
;        tmpx = [max_id-width1, max_id+width2]
;        tmpy = [sub_tsd[max_id-width1],sub_tsd[max_id+width2]]
;        oplot, tmpx, tmpy, psym=2
;        plot, proj[id]
;        wait,1
;    endif


        if (isGlitch eq 1) then goto, finish
        
    endfor
    
    finish:


    if (isGlitch eq 0) then begin 
        data[id] = 0  ;; note that this data is glitch_tsd
    endif
    
    if (isGlitch eq 1) then begin 
        if (fader eq 1) then flg_fad[id]=1
        if (dipper eq 1) then flg_dip[id]=1
    endif    

    if (isSource eq 1) then begin 
        i1 = (max_id - source_size) > 0
        i2 = (max_id + source_size) < (n_elements(id)-1)
        flg_src[id[i1:i2]] = 1
    endif

endfor        

;- 1 readout singal results in n_signal=0. 
id = where(data ne 0, cnt)
if (cnt ge 1 and n_signal eq 0) then data[id] = 0

END
