;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_object_id.pro
; PURPOSE:
;       Identify structures and objects in MMT space
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;       wave         MMT wave [readout, scale]
;       sigma_j      standard deviation at each scale 
;       threshold    threshold for significant MMT coefficients
; OUTPUTS:
;       mask         1 (-1) for positive (negative) significant
;                    coefficients [readout, scale]
;       proj         porjected mask
;       struct       structure id at each [readout, scale]. Objects
;                    have the same structure ID in multi-scale.
;       nstrs        number of structures  
;
; RETURN
;
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
;	2. The after effects are corrected with the transient correction.
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	Aug. 2004		Written by T.Takagi
;    
;-

PRO mmt_object_id, wave, sigma_j, threshold, mask, proj, struct, nstrs

time=dblarr(10)
time[0] = systime(/second)

;- number of readout 
ndata = n_elements(wave[*,0])
nscale = n_elements(wave[0,*])

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Thresholding at each scale
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mask = intarr(ndata,nscale)
for i=1,nscale-1 do begin

;;; positive signals    
    yes=where(wave[*,i] ge threshold*sigma_j[i],count)
      if (count ne 0) then mask[yes,i]=1

;;; negative signals
    yes=where(wave[*,i] le -threshold*sigma_j[i],count)
      if (count ne 0) then mask[yes,i]=-1

endfor

time[1] = systime(/second)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Object detection and labelling 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                  
;; Identification of structure at each scale
;- Array for structure ID. 
struct = intarr(ndata,nscale)
;- Array for projected mask
proj = intarr(ndata)

nstr=0
init_struct_id_at_scale = intarr(nscale)
for i=nscale-1, 1, -1 do begin 
    init_struct_id_at_scale[i] = nstr
;; structure ID increases from the largest scale
    zero=where(wave[*,i] eq 0, cnt)
;    print, 'Scale =',i, 1.*cnt/ndata

    ;; Projected mask
    proj[*] = proj[*] + mask[*,i]
        
    ;; ID of structure
    for j=1,ndata-1 do begin
        if (mask[j-1,i]*mask[j,i] eq 0) then begin
            if (mask[j,i] ne 0) then nstr=nstr+1
        endif
        if (mask[j,i] ne 0) then begin
            struct[j,i]=nstr
        endif
    endfor    
endfor

;proj = total(mask,2) ;; this does not help for speed up

time[2] = systime(/second)

;; Number of structures
nstrs = nstr
new_struct = struct

;; Object identification by interscale relations
;- Start from the largest scale
abs_wave = abs(wave)
for i=nscale-2, 1, -1 do begin
    idNonZero = where(struct[*,i] ne 0, cnt)
    if (cnt ne 0) then begin
        id_start = min(struct[idNonZero,i]) ; first structure ID in scale i
        id_end = max(struct[idNonZero,i])   ; last structure ID in scale i

;        print, i, id_start, id_end

        ;; Loop for structure ID (str_id)
        id1 = 0
        for id_str=id_start, id_end  do begin
;            id = where(struct[*,i] eq id_str) 
            id = where(struct[id1:*,i] eq id_str) 
            id = id + id1
            peaks = where(abs_wave[id,i] eq max(abs_wave[id,i]),cnt)+id[0]
            ;;; sometimes there are multiple peaks
            if (cnt eq 1) then peak_id = peaks
            if (cnt ge 2) then begin 
                peak_id = peaks[0]
            endif

            ;;A) If peak location is included in an object at 
            ;;   the next larger scale, assign the same structure ID.
            connect = where(mask[peaks,i+1] ne 0, cnt)            
            if (cnt ne 0) then begin
                new_struct[id,i]=new_struct[peak_id,i+1]
            endif
            id1 = id[n_elements(id)-1]                        
        endfor
    endif
endfor

struct = new_struct

time[3] = systime(/second)

;;;
;;; Check computation time
;;;
;elapsed_time = time[1:3]-time[0:2]
;print, elapsed_time
;print, 'total time = ',time[3]-time[0]
;stop 

END
