; $Id: gb_differentiation.pro,v 1.13 2006/10/03 10:32:33 makiuti Exp $
;
; Copyright (c) 2002-2006, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   GB_Differentiation.pro
;
; PURPOSE:
;   Input data is differentiated.
;
; CATEGORY:
;   Function. Green Box procedure in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;   Result = GB_Differentiation(tsd
;            [, diff_mod=diff_mod, shiftvl=shiftvl, disp=disp
;             , rdvdet=rdvdet])
;
; INPUTS:
;   tsd     TSD object reference.
;
; OUTPUTS:
;   tsd     TSD object reference. Differentiated TSD.
;
; RETURN:
;   !SUCCEED if function ended successfully.
;   !FAIL in failure.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   1. Extract detector data from TSD.
;   2. Carry out differentiation; method is TBD.
;   3. Set flags for immediately before and/or after reset, "no_diff,"
;      and others if needed (depends on the algorithm.)
;   4. Flag those with unusual behaviour after the differentiation;
;      possible candidates of unusual behaviour are to be considered
;      and given in CCF.
;   5. Write a history in TSD header.
;   6. Output TSD object reference.
;   7. Write a summary of this function.
;      The contents of the summary with its management are TBD.
;
;   Items to be determined;
;     1. Method of differentiation.
;     2. Re-definition of the time may be needed
;     after the differentiation for some methods.
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;   Oct. 2002    Written by A. Kawamura.
;   Nov. 2003    Default parameters use 2pt differentiation
;   Oct. 2006    Last modified by Makiuti
;-

FUNCTION gb_differentiation, tsd, $
           diff_mod=diff_mod, shiftvl=shiftvl, disp=disp,$
           rdvdet=rdvdet
;ON_ERROR 2

;;; Starting GB_Differentiation ;;;
    LMESSAGE, 5, 'Executing GB_Differentiation...', /CONTINUE

;;; Extract values from TSD object reference ;;;
    in_vdet = tsd->Get_Value_TSD('FIS_OBS','Flux')
    err = af_finite(in_vdet, vdet)

    ferr = tsd->Get_Value_TSD('FIS_OBS','FErr')

IF KEYWORD_SET(rdvdet) EQ 1 THEN vdet = tsd->Get_Value_TSD('FIS_OBS','DET')

;;; number of channels ;;;
    si = SIZE(vdet)
      arr_dim  = si[0]
      num_ch   = si[1]
      num_data = si[2]

;;; get reset status ;;;

     rst = tsd->Get_Pix_Flag_TSD('FIS_OBS','Reset')


;;----------------------------------------------------------
;; Differentiation
;;----------------------------------------------------------
  IF KEYWORD_SET(diff_mod) EQ 0 THEN diff_mod='gb_diff2p.pro'

    CASE diff_mod OF

         'gb_diff2p.pro' : BEGIN

                         IF KEYWORD_SET(shiftvl) EQ 1 THEN BEGIN
                            err_code = gb_diff2p(tsd, vdet, diff_vdet, ferr=ferr, shiftvl=shiftvl)
                         ENDIF ELSE BEGIN
                            err_code = gb_diff2p(tsd, vdet, diff_vdet, ferr=ferr)
                         ENDELSE

                           END

         'gb_diff3pl.pro' : BEGIN

                         err_code = gb_diff3pl(vdet, diff_vdet, tsd=tsd)

                            END

          ELSE:             BEGIN
                         LMESSAGE, 1, 'No such a function', /CONTINUE
                         err_code = -1
                            END
          ENDCASE

          IF err_code NE 0 THEN BEGIN
          LMESSAGE, 1, 'Differentiation failed...', /CONTINUE
          Return, !FAIL

  ENDIF

;;----------------------------------------------------------
;; Write out 'Flux'
;; Set 'Bad' flag at frame 'No_Diff' was set
;;----------------------------------------------------------
    LMESSAGE, 7, 'Writing differentiated TSD...', /CONTINUE
    ret = tsd->Set_Value_TSD('FIS_OBS','Flux', diff_vdet)
    ret = tsd->Set_Value_TSD('FIS_OBS','FErr', ferr)

    nodiff = tsd->Get_Pix_Flag_TSD('FIS_OBS','No_Diff')
    bad = tsd->Get_Pix_Flag_TSD('FIS_OBS','Bad')

    new_bad = nodiff + bad
      idx = where(new_bad NE 0, cnt)
      IF cnt NE 0 THEN new_bad[idx] = 1
    LMESSAGE, 7, 'Setting pix. flag, BAD,...', /CONTINUE
    ret = tsd->Set_Pix_Flag_TSD('FIS_OBS','Bad', new_bad)

;;----------------------------------------------------------
;; Set quality indicator
;;----------------------------------------------------------
; Currently this is only for the case of "diff_mod='gb_diff2p.pro'"

    qual_df_eq = tsd->Get_Quality_TSD('FIS_OBS','Qual_df_eq')

    ind = where(nodiff EQ 1, cnt)
    IF (cnt NE 0) THEN qual_df_eq[where(nodiff EQ 1)] = 3

    ret = tsd->Set_Quality_TSD('FIS_OBS','Qual_df_eq', qual_df_eq)


;;----------------------------------------------------------
;; Display option
;;----------------------------------------------------------
    IF KEYWORD_SET(disp) EQ 1 THEN BEGIN
      !p.multi=[0,1,2]
      mindata=1.2*min(vdet)
      maxdata=1.2*max(vdet)
      mindiff=1.2*min(diff_vdet)
      maxdiff=1.2*max(diff_vdet)

      IF arr_dim EQ 1 THEN BEGIN
        erase
        plot,vdet[*],yrange = [mindata,maxdata],xs=1,ys=1
        plot,diff_vdet[*],col=200,yrange = [mindiff,maxdiff],xs=1,ys=1
      ENDIF ELSE BEGIN
        erase
        FOR i = 0, num_ch-1 DO BEGIN
          ti1=strcompress('ch'+string(i+1))
          ti2=strcompress('ch'+string(i+1)+': differentiated')
          plot,vdet[i,*],yrange = [mindata,maxdata],title=ti1,xs=1,ys=1
          plot,diff_vdet[I,*],col=200,yrange = [mindiff,maxdiff],xs=1,ys=1,title=ti2
          wait,0.1
        ENDFOR

      ENDELSE

      !p.multi=0
    ENDIF

;;----------------------------------------------------------
;; Write HISTORY in header
;;----------------------------------------------------------
    LMESSAGE, 7, 'Writing HISTORY', /CONTINUE
    name_ver=gb_get_name_ver(infile='gb_differentiation')
    hist_message = STRCOMPRESS(' '+name_ver+' done. CCF= ')

    dummy = tsd_fxaddhist(tsd, hist_message, /prres)
    IF dummy NE 0 THEN $
      LMESSAGE, 3, 'Unable to write HISTORY', /CONTINUE

    ;;; Write a summary of this function

    RETURN, !SUCCEED

END
