; $Id: gb_set_glitch_status_mmt.pro,v 1.15 2007/06/01 11:45:26 lys Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	GB_Set_Glitch_Status_MMT.pro
; PURPOSE:
;	Detect spikes by cosmic-ray hits. Flag, "MTGL_TYPE*," is set.
; CATEGORY:
;	Function. Green Box procedure in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = GB_Set_Glitch_Status_MMT(tsd
;	[, output_glitch=output_glitch, debug=debug])
; INPUTS:
;	tsd		TSD object reference.
;       output_glitch   Flag to output TSD of glitch and glitch list
;       debug           Flag to use debug mode
; OUTPUTS:
;	tsd		TSD object reference. Deglitched TSD.
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;	
; PROCEDURE:
;	1. This function will be provided by the European team.
;	2. The after effects are corrected with the transient correction.
;	3. A large spike may cause saturation.  
;	4. It is to be noted that the flux data necessary for 
;	the transient 
;	correction is lost until the next reset when a detector is 
;	saturated.
;	5. The data of previous missions, such as IRTS/FILM and ISO/LWS,
;	6. PHOT, are useful example to investigate the real cosmic-ray
;	glitches on the same type of detectors.
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
; 	Dec. 2003		Written by T.Takagi 
;       Nov. 2006		Modified by E.Figueredo
;       Jan. 2007               Modified by E.Figueredo
;       May. 2007               Modified by E.Figueredo
;-

FUNCTION GB_Set_Glitch_Status_MMT, tsd, Calib_Const_File = ccf, output_glitch=output_glitch, debug = debug, channel=channel, segment=segment, $
                                   max_data_length=max_data_length
;ON_ERROR 2

if (n_elements(output_glitch) eq 0) then output_glitch=0
if (n_elements(debug) eq 0) then debug = 0

;;; parameters in CCF
  spike_threshold = 4  ; threshold for iso_despiker
  mmt_threshold = 5    ; threshold for glitch detection      
  remove = 0           ; 1- remove glitches from TSD (works only for despiker)
  if (n_elements(max_data_length) eq 0) then begin 
      max_data_length=2000 ; maximum data length for each deglitch process
  endif
  data_overlap = 300            ; overlap for data separation
  scale=2L^[0,1,2,3,4,5,6,7,8,9] + 1  ; mmt scale


;;; Starting GB_Set_Glitch_Status ;;;
  LMESSAGE, 5, 'Executing GB_Set_Glitch_Status_MMT...',/CONTINUE
  LMESSAGE, 5, 'spike_threshold = '+strcompress(spike_threshold),/CONTINUE
  LMESSAGE, 5, 'mmt_threshold = '+strcompress(mmt_threshold),/CONTINUE

;;; Extract values from TSD object reference ;;;
  in_flux=tsd->get_value_tsd('FIS_OBS', 'FLUX')
  err=af_finite(in_flux, flux)                  ; Convert NaN and Inf to float

;;; number of channels ;;;
  si = SIZE(flux)
  num_ch = si[1]
  num_data = si[2]

  max_data_length =   max_data_length < num_data
  
;- Read CCF files
  if (num_ch eq 75) then begin 
      sigma_j = CCF_Read_CCF('SIGJL', tsd, HEADER=hdr, ERROR=err)
  endif else begin
      sigma_j = CCF_Read_CCF('SIGJS', tsd, HEADER=hdr, ERROR=err)
  endelse
  cal_lamp = CCF_Read_CCF('GLCAL', tsd, HEADER=hdr, ERROR=err)

;;; Apply deglitcher 
  LMESSAGE, 5, "Start Deglitching with MMT deglitcher", /CONTINUE


  ;; Get flags for bad data 
  fbad = tsd->Get_Pix_Flag_TSD('FIS_OBS', 'BAD')
  freset = tsd->get_pix_flag_tsd('FIS_OBS', 'RESET')
  frstanom = tsd->get_pix_flag_tsd('FIS_OBS', 'RSTANOM')
  fnodiff = tsd->get_pix_flag_tsd('FIS_OBS', 'NO_DIFF')
  fshtop = tsd->Get_Status_TSD('FIS_OBS', 'ShtOp')
  fshtcl = 1-fshtop ;; In Lab data, fshtop = 1 for all data points
  framp = tsd->get_pix_flag_tsd('FIS_OBS','NO_RP_CORR')

  ;- Check bias boost 
  ;  LWboostOn should be used for SW as well, since the reset 
  ;  is very frequent during LW boost, even after SWboost ends. 
  fboost = tsd->get_status_TSD('FIS_OBS','LWboostOn')


;;;
;;; Check observation mode
;;;
  obs_mode = tsd->get_value_tsd('FIS_OBS','PacketID')

;;;
;;;- Calibration lamp
;;;
  if (num_ch eq 75) then calon = 'CALALON'
  if (num_ch eq 100) then calon = 'CALASON' ;
  fcal = tsd->get_status_TSD('FIS_OBS',calon)  
  mask = mmt_isolate_signals(fcal, 0, sep_limit=10)
  ncal = max(mask)
  fcal_tail = fcal

  if (num_ch eq 75) then begin 
      ntail = 100 ;; length of tail from Cal lamp
  endif else begin
      ntail = 10 ;; just temporary - it is important to calculate better
  endelse
  for i=1, ncal do begin 
      id = where(mask eq i)
      i1 = id[n_elements(id)-1]
      i2 = (i1+ntail) < (num_data-1)
      fcal_tail[i1:i2] = 1
  endfor

;;; set-up flag array
  flg_pspike=bytarr(num_ch, num_data)
  flg_nspike=bytarr(num_ch, num_data)
  flg_dipper=bytarr(num_ch, num_data)
  flg_fader=bytarr(num_ch, num_data)
  flg_source=bytarr(num_ch, num_data)
  counter = lonarr(num_ch, num_data) ; array for glitch counter
 

;;; array to store glitch tsd 
  glitch_tsd = dblarr(num_ch, num_data)


;;; Table for glitch list 
  if (output_glitch eq 1) then begin 
      infile = tsd->get_filename()
      str_len = strpos(infile, ".fits")
      listfile = strmid(infile,0,str_len)+'_glitch_list.dat'
      openw, lun, listfile, /get_lun
  endif
  if (debug eq 1) then openw, lun_debug, 'glitch_list.txt', /get_lun

;;;
;;; set condition to use deglitcher 
;;;   (e.g. no need for dead pixel, too many bad data)
;;;
;; Check dead pixels
  dead = bytarr(num_ch)
  for i=0, num_ch-1 do begin
      ind_fbad=WHERE(fbad[i,*] NE 0, cnt_fbad)
      if (cnt_fbad eq num_data) then dead[i]=1
  endfor

;; debug for a specific glitch
i_debug =  68735

;; One channel mode
if (n_elements(channel) eq 0) then begin 
    i1 = 0
    i2 = num_ch-1
endif else begin 
    i1=channel
    i2=channel
endelse

  for i=i1,i2 do begin
      
      sig_j = reform(sigma_j[0:(n_elements(scale)-1),i])      

      ;;;
      ;;; data set-up
      ;;;
      timeline=reform(flux[i,*])

;-  Problem in data 
      if (dead[i] eq 1) then begin 
          LMESSAGE, 7, 'Dead pixel:'+strcompress(i),/CONTINUE
          goto, bad_pix
      endif


      ;;;
      ;;; Treat data identified with flags
      ;;;   - Reset 
      ;;;   - Reset anomary       

      bad_fl1 = freset[i,*]+frstanom[i,*]+framp[i,*]
      mmt_bad_handling, timeline, bad_fl1, scale=1
      bad_freset = freset[i,*]     

      ;;; Check obs mode and nodiff flag
      simple = 64
      cds = 65
      coadd = 66
      fcds = obs_mode ne cds
      fdiff = fnodiff[i,*]*fcds
      bad_fl3 = fdiff
      mmt_bad_handling, timeline, bad_fl3, scale=1

;;;
;;; Subtract calibration lamp - the program mmt_subtract_cal_lamp
;;;                             results in a lot of residual noise in
;;;                             the timeline - this step will be
;;;                             removed until to find a
;;;                             better solution
;;;             
;;;  
;      lmessage,7, 'Cal_lamp subtraction START'
;      mmt_subtract_cal_lamp, $
;        num_ch, i,timeline, cal_lamp, fcal, obs_mode, timeline_nocal
;      lmessage,7, 'Cal_lamp subtraction END'


;; mask cal_lamp and boost       
      bad_fl2 = fboost +fcal_tail
      mmt_bad_handling, timeline, bad_fl2, scale=300

      ;; this is used for noise estimation
      bad_fl_cal = fcal_tail
      bad_fl = bad_fl1 + bad_fl2 + bad_fl3 + bad_fl_cal 

      if (debug eq 1) then begin
          LMESSAGE, 7, 'channel = '+strcompress(i),/CONTINUE          
;          writecol, 'pspike.txt',pspikes
;          writecol, 'nspike.txt',nspikes
;          writecol, 'timeline.txt',timeline
      endif
      
      skip_mmt = 0
      if (skip_mmt eq 1) then goto,bad_pix      

      
      ;;;- Divide timeline data for small segments
      ;- mask for shutter close, etc
      fdiv = fshtcl + fcds
;      mask = mmt_isolate_signals(fshtcl, 0, sep_limit=1)
      mask = mmt_isolate_signals(fdiv, 0, sep_limit=1)
      mask_new = bytarr(n_elements(mask)) 
      count = 0
      for imask=1, max(mask) do begin 
          id = where(mask eq imask)      
          if (id[0] ne 0) then mask_new[id] = mask_new[id[0]-1]+1

          if (id[n_elements(id)-1] ne n_elements(mask)-1) then begin 
              id1 = id[n_elements(id)-1]+1
          endif else begin 
              break
          endelse 
          if (imask+1 gt max(mask)) then begin 
              id2 = num_data-1
          endif else begin 
              id = where(mask eq imask+1) 
              id2 = id[0]-1
          endelse 
          mask_new[id1:id2] = mask_new[id1-1]+1
      endfor
      mask_div = mask_new

;; debug
;      !p.multi=[0,1,2]
;      for idiv=0,max(mask_div) do begin 
;          id = where(mask_div eq idiv)
;          data = timeline[id]
;          plot,timeline
;          plot, id, data
;          wait,3
;      endfor


      mmt_data_separator, timeline, n_data=max_data_length, $
             overlap=data_overlap, subArr, subId, nseg

;      mmt_data_separator, timeline_nocal, n_data=max_data_length, $
;             overlap=data_overlap, subArr_nocal, subId_nocal, nseg_nocal



      ;; Debug 
      if (debug eq 1 and n_elements(i_debug) ne 0) then begin 
          for iseg=0,nseg-1 do begin 
              dum = where(subID[iseg,*] eq i_debug,cnt)
              if (cnt eq 1) then begin 
                  iseg_debug = iseg
                  break
              endif
          endfor
      endif
      
      ;- iteration for data segments 
      ;; One segment mode
      if (n_elements(segment) eq 0) then begin 
          s1 = 0
          s2 = nseg-1
      endif else begin 
          s1=segment
          s2=segment
      endelse

      for i_seg = s1, s2 do begin

          if (nseg eq 1) then begin
              data = reform(subArr)
 ;             data_nocal = reform(subArr_nocal)
              data_nocal = data
              id_orig = reform(subId)
              bad_fl_seg = bad_fl
              bad_freset_seg = bad_freset
          endif else begin 
              data = reform(subArr[i_seg, *])
  ;            data_nocal = reform(subArr_nocal[i_seg, *])
              data_nocal = data
              id_orig = reform(subId[i_seg, *])
              bad_fl_seg = reform(bad_fl[id_orig])
              bad_freset_seg = reform(bad_freset[id_orig])
          endelse

;          if (debug eq 1) then begin 
;              writecol, 'data.txt',data
;              gb_iso_despiker, data, threshold=spike_threshold, $
;                remove=0,debug=1,$
;                subid=reform(subid[i_seg,*]) ; for debug           
;          endif

          ;- first estimate of sigma 
          ;;;;;
;          af_sky_stats, data, mean, var
;          sigma = sqrt(var)
          ;;; noise estimate with highpass filter 
          isFFT = 0
          isSavgol = 1
          if (isFFT eq 1) then begin  
              n_data = n_elements(data)
              if ( (n_data mod 2) eq 1) then n_data = n_data-1              
              fcut = n_data/20
              y=[findgen(n_data/2),findgen(n_data/2)-n_data/2]
              filter=1.0/(1+(y/fcut)^10)
              highpass=fft(fft(data[0:n_data-1],1)*(1.0-filter),-1)
              af_sky_stats, highpass, mean, var
              sigma = sqrt(var)
              data_orig = data
              data[0:n_data-1] = data[0:n_data-1] - highpass[0:n_data-1]
          endif
          if (isSavgol eq 1) then begin 
              savgolFilter = SAVGOL(6,6,0,2);$
                                 ;;(6, 6, 0, 2) ;; default (16,16,0,4)
              low_comp = CONVOL(data, savgolFilter, /EDGE_TRUNCATE)
              data_orig = data
              highpass = data - low_comp          
              ;; Exclude replaced data at bad flag              
              jj = where(bad_fl_seg eq 0,cnt)
              if (cnt ne 0) then begin 
                  af_sky_stats, highpass[jj], mean, var
              endif else begin 
                  af_sky_stats, highpass, mean, var
              endelse
              sigma = sqrt(var)
;              mmm, highpass, skymod, sigma_mmm
              data = data - highpass
              !p.multi=[0,1,3]
;              plot, data_orig, charsize=2
;              plot, low_comp, charsize=2
;              plot, highpass, charsize=2,xtitle="Readout"
;;              wait,1
;              stop
          endif

          ;;;
          ;;; despike
          ;;;         

          ;;; debug
;          window,1
;          !p.multi=0
;          device, decompose=0 
;          loadcolors
;          plot, data_orig
;          oplot, data_nocal,color=5
;          window,0
          ;;; 

          sigma_dif = sigma*sqrt(2)
;          print, 'ISO despiker: sigma_dif = ',sigma_dif
          ;- positive spikes
          pspikes = 0
          gb_iso_despiker, $
            data_nocal, sigma_dif, glitches=pspikes, bad_freset = bad_freset_seg, $
            threshold=spike_threshold, remove=remove, debug=debug,$
            subid=id_orig ; for debug           
          ;- negative spikes
          data_nocal = -data_nocal
          nspikes = 0
          gb_iso_despiker, $
            data_nocal, sigma_dif, glitches=nspikes, bad_freset = bad_freset_seg,  $
            threshold=spike_threshold, remove=remove, debug=0,$
            subid=id_orig ; for debug
          data_nocal = -data_nocal
          ;- flag spike
          dim_ps = size(pspikes,/dim)
          dim_ns = size(nspikes,/dim)
          if (dim_ps ge 1) then begin
              flg_pspike[i,id_orig[pspikes]] =1
          endif
          if (dim_ns ge 1) then begin
              flg_nspike[i,id_orig[nspikes]] =1
          endif

          ;; Exclude calibration lamp from mmt_deglitcher
;          mmt_bad_handling, data, bad_fl_cal[id_orig], scale=300

          ;;;
          ;;; sub_mask to divide data 
          ;;;
;          mask_div[*] = 0
          sub_mask = mask_div[id_orig]
          for idiv=min(sub_mask),max(sub_mask) do begin 
              id = where(sub_mask eq idiv,cnt)
              sub_data = data[id]
              if (cnt le scale[n_elements(scale)-1]) then goto, bad_div


          ;- iteration for deglitch 
          n_deglitch = 0
          stop_iteration = 0
          while (stop_iteration ne 1) do begin
              LMESSAGE, 7, 'channel = '+strcompress(i),/CONTINUE
              LMESSAGE, 7, 'segment = '+strcompress(i_seg),/CONTINUE 
              LMESSAGE, 7, 'sigma    = '+strcompress(sigma),/CONTINUE 
;             LMESSAGE, 7, 'sigma_mmm= '+strcompress(sigma_mmm),/CONTINUE 

              n_deglitch = n_deglitch + 1 

              mmt_deglitcher, sub_data, output, sigma, scale=scale,  $
                sigma_j = sig_j, $
                threshold=mmt_threshold, $;inputs
                dippers=dippers,faders=faders,sources=sources, $ ;outputs
                pspikes=pspikes, nspikes=nspikes, $
                debug=debug, $  ; outputs
                glitch_tsd=glitch_data,$
                glitch_measures=gm, $
                ch=i, subid=id_orig[id] ; for debug

              sz = size(gm)
              if (sz[0] ne 0) then begin 
                  for igm=0, n_elements(gm.isGlitch)-1 do begin 
                      if (gm.isGlitch[igm] eq 1) then begin 
                          mid = gm.id_start[igm]+gm.max_id[igm]
                          maxid = id_orig[id[mid]]
                          counter[i,maxid] = 1
                      endif
                  endfor
              endif

              if (output_glitch eq 1) then begin
                  sz = size(gm)
                  if (sz[0] ne 0) then begin 
                      mmt_output_glitch_list, lun, i, id_orig[id], gm
                  endif
              endif
              if (debug eq 1) then begin 
                  sz = size(gm)
                  if (sz[0] ne 0) then begin 
                      mmt_output_glitch_list, lun_debug, i, id_orig[id], gm
                  endif                  
              endif

              ;- save flag info
              dim_f = size(faders,/dim)
              dim_d = size(dippers,/dim)
              dim_s = size(sources,/dim)
              if (dim_f ge 1) then begin
                  flg_fader[i,id_orig[id[faders]]] =1
              endif
              if (dim_d ge 1) then begin
                  flg_dipper[i,id_orig[id[dippers]]] =1
              endif
              if (dim_s ge 1) then begin
                  flg_source[i,id_orig[id[sources]]] =1
              endif


              if (n_elements(glitch_data) ne 0) then begin 
                  dum = where(glitch_data ne 0., cnt)
                  if (cnt ne 0) then begin               
                      tmp = glitch_tsd[i,id_orig[id]]
                      glitch_tsd[i,id_orig[id]] = tmp+glitch_data
                  endif
              endif

;              stop_iteration = 1
              if (dim_f eq 0 and dim_d eq 0) then begin
                  stop_iteration = 1
              endif else begin 
                  sub_data = output
              endelse 
          ;- deglitch iteration ends       
          endwhile          


          ;- sub-data division iteraton ends
          bad_div:
          endfor          

      ;- data segment iteration ends
      endfor

;      plot, sigma_arr
     
      bad_pix:
  ;- pixel iteration ends
  endfor 
  if (output_glitch eq 1) then free_lun, lun
  if (debug eq 1) then free_lun, lun_debug

;;; Key string for TSD2
;;; type0: positive spike
;;; type1: negative spike
;;; type2: dipper
;;; type3: fader
  key = ["MTGL_TYPE1","MTGL_TYPE2","MTGL_TYPE3","MTGL_TYPE4", $
         "MTGL_TAIL"] ; key in tsd interface for MMT transform

;;- remove spike flag for strong sources
;  sp = flg_pspike eq 1 
;  src = flg_source eq 1
;  id = where( (sp*src) eq 1, cnt)
;  if (cnt ge 1) then flg_pspike[id]=0

  
  dummy = tsd->set_pix_flag_tsd('FIS_OBS', key[0], flg_pspike)
  dummy = tsd->set_pix_flag_tsd('FIS_OBS', key[1], flg_nspike)
  dummy = tsd->set_pix_flag_tsd('FIS_OBS', key[2], flg_dipper)
  dummy = tsd->set_pix_flag_tsd('FIS_OBS', key[3], flg_fader)

;- rise bad flag
  bad = fbad + flg_pspike + flg_nspike + flg_dipper + flg_fader
  id = where(bad ne 0,cnt)
  if (cnt ne 0) then bad[id] = 1
  dummy = tsd->set_pix_flag_tsd('FIS_OBS', 'BAD', bad)

;- set glitch counter  
  for i=0, num_ch-1 do begin 
      ; set -1 at the beginning
      id = where(counter[i,*] eq 1,cnt)
      if (cnt ne 0) then begin
          iend = 0 > (id[0]-1)      
          counter[i,0:iend]=-1

          arr = lindgen(num_data)
          for j=0, cnt-1 do begin 
              ist = id[j]
              if (j eq cnt-1) then begin 
                  iend = num_data-1
              endif else begin
                  iend = id[j+1]-1
              endelse                   
              len = iend-ist
              counter[i,ist:iend]=arr[0:len]
          endfor
      endif else begin 
          counter[i,*] = -1
      endelse 

  endfor
  id = where(counter gt (2^15-1), cnt)
  if (cnt ne 0) then counter[id] = 2^15-1
  dum = tsd->set_value_TSD('FIS_OBS','CNT_GLITCH_MT',fix(counter))
  

;;; Output Glitch TSD to a file
  if (output_glitch eq 1) then begin 
      out_tsd=OBJ_NEW('tsd')
      infile = tsd->get_filename()
      str_len = strpos(infile, ".fits")
      outfile = strmid(infile,0,str_len)+'_glitch.sav'
      save, filename=outfile, glitch_tsd

;      outfile = strmid(infile,0,str_len)+'_glitch.fits'
;      detector = ''
;      if (num_ch eq 75) then detector = 'FIS_LW'
;      if (num_ch eq 100) then detector = 'FIS_SW';
;      dummy = out_tsd->create_tsd(detector,num_data)
;      dummy = out_tsd->set_value_tsd('FIS_OBS','FLUX',glitch_tsd)  
;      dummy = out_tsd->write_tsd(outfile)
      
  endif


;;; Write HISTORY in header ;;;
  LMESSAGE,5, 'Writing HISTORY',/CONTINUE
  name_ver=gb_get_name_ver(infile='gb_set_glitch_status_mmt')     
  hist_message = STRCOMPRESS(' '+name_ver+' done. CCF= ')
  IF KEYWORD_SET(ccf) NE 1 THEN BEGIN
      hist_message = hist_message + 'NONE'
  ENDIF ELSE BEGIN
      hist_message = hist_message+ccf
  ENDELSE

  dummy = tsd_fxaddhist(tsd, hist_message, /prres)
  IF dummy NE 0 THEN $
    LMESSAGE, 1, 'Unable to write HISTORY',/CONTINUE

  return, !SUCCEED
END

