; $Id: im_detpos.pro,v 1.5 2006/05/12 07:34:03 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_detpos.pro
; PURPOSE:
;	determine position in one detector scan
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	result = im_detpos( det, spheric, mapinfo, x, y )
;
; INPUTS:
;	det		detector number
;	spheric 	positional information
;	mapinfo 	map information
; OUTPUTS:
;	x, y		projected values in offsetted, floating pixel values
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;	might not be OK at the poles of the coordinate system
;
; PROCEDURE:
;	Get the detector w.r.t boresight. They are gnomonic projected values.
;	(TBC).
;	Rotate them to (l,b) according to the twist angle.
;	Unproject them into Euler triplets.
;	Convert the triplets to delta-l and delta-b.
;	Add these delta's to (l,b).
;	Use Astrolib's projection method (wcssph2xy) to project (l,b)
;	into (x,y).
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;-

FUNCTION im_detpos, det, spheric, mapinfo, xx, yy

COMPILE_OPT IDL2

;   ON_ERROR 2

    torad = !DPI / 180
    offset = detector_offset_ccf( det=det, nominal= mapinfo.nominal )

    dy = offset[0] * torad / 60 	; proj.pos in radian
    dz = offset[1] * torad / 60

    sl = spheric.sinlon
    cl = spheric.coslon
    sb = spheric.sinlat
    cb = spheric.coslat
    st = spheric.sinrol
    ct = spheric.cosrol

;     u =  dy * ct + dz * st			    ; proj. delta RA
;     v = -dy * st + dz * ct			    ; proj. delta DEC
;    u = -dy * ct - dz * st			    ; proj. delta RA
;    v = +dy * st - dz * ct			    ; proj. delta DEC
    u = +dy * ct - dz * st			    ; proj. delta RA
    v = -dy * st - dz * ct			    ; proj. delta DEC

    r = 1.0 / sqrt( 1 + dy * dy + dz * dz )
    x1 = ( cl * cb - u * sl - v * cl * sb ) * r
    x2 = ( sl * cb + u * cl - v * sl * sb ) * r
    x3 = (	sb	    + v      * cb ) * r
    x3 = ( x3 > (-1) ) < 1			    ; keep -1 <= x3 <= 1

;; part for roll angle. TBC. for now not needed
;    rt = sqrt( 1.0 - x3 * x3 )
;    t1 = ( x1 * x3 * ct + x2 * st ) / rt
;    t2 = ( x2 * x3 * ct - x1 * st ) / rt
;    t3 =   ct * rt
;    dl = atan( x2, sqrt( x1 * x1 + x3 * x3 ) ) / torad
;    dl = asin( x2 ) / torad			    ; equivalent to previous
;    db = atan( x1, x3 ) / torad

    l = atan( x2, x1 ) / torad
    b = asin( x3 ) / torad

    header = im_mkheader( mapinfo )

    extast, header, astrometry

    ad2xy, l, b, astrometry, xx, yy




    ; Use Astrolib proc to project (l,b) into (x,y)
;    wcssph2xy, l, b, xx, yy, mapinfo.proj, crval = mapinfo.center

;    pixpdeg = mapinfo.pixels / mapinfo.size	; pixels per degree

;    xx = xx * pixpdeg[0] + mapinfo.pixels[0] / 2.0    ; offset (0,0) to mid
;    yy = yy * pixpdeg[1] + mapinfo.pixels[1] / 2.0    ;  of map

    return, !SUCCEED

END


