; $Id: im_image.pro,v 1.5 2006/05/12 07:34:03 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_image.pro
; PURPOSE:
;	main program for image construction
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	result = im_image( tsdlist, image, $
;	    center=center, cover=cover, stdev=stdev, $
;	    band=band, size=size, pixels=pixels, $
;	    beam=beam, destripe=destripe, tune=tune, $
;	    width=width, box=box, robust=robust, pointdrizzle=pointdrizzle, $
;	    zero=zero, mask=mask, nodistortion=nodistortion, $
;	    file=file, plot=plot, _extra=_extra
;
; INPUTS:
;	tsdlist 	list of TSD object references.
;	aocu2gads	keyword to copy positional infor from AOCU to GADS
;	center		the center of the map [long,lat]
;			default [0,0]
;	band		band to be used. either LW, LN, SW or SN
;			default: take datatype from tsdlist[0]
;	size		size of the map in degree [square] | [lsize,bsize]
;			default 1 degree
;	pixels		number of pixels on each side
;			default 4 pix per arcmin
;	beam		coaddition beam in arcmin. max 1 value per iter
;			default 0.5
;	projtype	projection type (see wcssph2xy.pro in astrolib/astrom)
;			default 2 (gnomonic)
;	detector	list of detectors to be used
;			default: all dets in this band
;	nodistortion	do not use the detector distortion table
;			default no (the distortion table is used)
;	use_errors	use sample errors as weights
;			default: all weights are 1
;	glitchtype	select glitches: 'none', 'mt', 'gp', or 'all'
;			default: 'all'
;	destripe	number of destripe iterations
;			default 0 (only coadd)
;	width		cross pixel comparison length for destriping
;			default: 41 pixels
;	box		smoothing box for image flux
;			default: 61 pixels
;	knotsep 	distance between spline knots for destriping
;			default: 600 samples
;	robust		iterations to discard outliers
;			default 5
;	zero		use a zero map to start the destriping
;			default no
;	mask		use this 1-0 valued map to suppress destriping
;			default none
;	file		write the image to a (fits) file
;			default: none
;	plot		various plot settings
;			0 : nothing
;			1 : destripe plots
;			2 : interactive destripe plots
;			    left click inside plotwindow -> next plot
;			    right click inside plotwindow -> plot all
;			3 : overlap plot
;			name : plot 1 to file called name.
;			default : 0 (nothing)
;	_extra		for plotting
;
; OUTPUTS:
;	image		the coadded map
;	cover		coverage map
;	stdev		the stdev map
;	weight		the weights map
;			when use_error is not set, it is the same as cover
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;
;
;
;
;
;
;
;
;
;
; EXAMPLE: Make a simple image
;	   IDL> r = im_image( tsdlist, image, center=[98.25,4.8] )
;
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;	May 2006  DK   Update
;-

FUNCTION im_image, tsdlist, image, cover=cover, stdev=stdev, weight=weight, $
	    center=center, band=band, file=file, aocu2gads=aocu2gads, $
	    size=size, pixels=pixels, projtype=projtype, detector=detector, $
	    beam=beam, destripe=destripe, tune=tune, width=width, mask=mask, $
	    knotsep=knotsep, glitchtype=glitchtype, flatfield=flatfield, $
	    zero=zero, box=box, robust=robust, nodistortion=nodistortion, $
	    pointdrizzle=pointdrizzle, use_errors=use_errors, $
	    plot=plot, _extra=_extra

COMPILE_OPT IDL2

COMMON plotopts, manual

;   ON_ERROR 2

    IF ( n_elements( center ) EQ 0 ) THEN center = [0.0,0.0]
    IF ( n_elements( center ) NE 2 ) THEN BEGIN
	lmessage, 0, "Need two values for the image center"
	return, !FAILURE
    ENDIF

    IF ( n_elements( band ) EQ 0 ) then begin
	lmessage, 7, 'Taking band info from tsdlist[0]', /print
	type = tsdlist[0]->get_datatype()
	if ( type EQ 'FIS_LW' ) then begin
	    band = 'LW'
	endif else begin
	    band = 'SW'
	endelse
    endif

    ccfn = 'OFST' + strmid( band, 0, 1 )
    dummy = detector_offset_ccf( ccfn, tsdlist[0] )

    IF ( n_elements( size ) EQ 0 ) THEN size = dblarr( 2 ) + 1 $
    ELSE IF ( n_elements( size ) EQ 1 ) THEN size = dblarr( 2 ) + size $
    ELSE IF ( n_elements( size ) EQ 2 ) THEN size = double( size ) $
    ELSE lmessage, 0, "More than 2 values for size"

    IF ( n_elements( pixels ) EQ 0 ) then pixels = long( size * 240 ) $
    ELSE IF ( n_elements( pixels ) EQ 1 ) THEN pixels = lonarr( 2 ) + pixels $
    ELSE IF ( n_elements( pixels ) GT 2 ) THEN $
	    lmessage, 0, "More than 2 values for pixels"

    IF ( n_elements( beam ) EQ 0 ) THEN beam = 0.5
    IF ( n_elements( robust ) EQ 0 ) THEN robust = 5
    IF ( n_elements( width ) EQ 0 ) THEN width = 41
    IF ( n_elements( box ) EQ 0 ) THEN box = 61
    IF ( n_elements( knotsep ) EQ 0 ) THEN knotsep = 600
    IF ( n_elements( destripe ) EQ 0 ) THEN destripe = 0
    IF ( n_elements( projtype ) EQ 0 ) THEN projtype = 2
    IF ( n_elements( glitchtype ) EQ 0 ) THEN gltype = 'all' $
    ELSE gltype = strlowcase( glitchtype )

    mapinfo = { center : center, $	    ; RA and DEC of the center
		size   : size, $	    ; 2 doubles. in degrees
		pixels : pixels, $	    ; 2 ints.
		beam   : beam[0]/60.0, $    ; radius of the coadd beam
		band   : band, $	    ; SW, SN, LW, or LN
		nominal: keyword_set( nodistortion ), $  ; use distortion table
		drizzle: keyword_set( pointdrizzle ), $
		errors : keyword_set( use_errors ), $
		robust : robust, $
		width  : width, $
		box    : box, $
		knotsep: knotsep, $
		proj   : projtype, $
		gltype : gltype, $
		destr  : keyword_set( zero ), $
		flat   : keyword_set( flatfield ) }

    pltype = ( n_elements( plot ) GT 0 ) ? size( plot, /type ) : 0
    if ( pltype GT 0 ) then begin
	do_plot=1
	if ( pltype EQ 7 ) then start_hc, plot
	if ( pltype LE 3 ) then do_plot = plot
	if ( n_elements( do_plot ) EQ 1 ) then begin
	    if ( do_plot EQ 3 ) then $
	      plot, [0,pixels[0],pixels[0],0,0], [0,0,pixels[1],pixels[1],0] $
;		, yr=[0,pixels[0]], xr=[0,pixels[1]]
		, yr=[-pixels[0],2*pixels[0]], xr=[-pixels[1],2*pixels[1]]
	    if ( do_plot EQ 2 ) then manual = 1 else manual = 0
	endif
    endif else do_plot = 0

    IF ( n_elements( mask ) EQ 0 ) THEN mask = 0

    mapset = {	mom0  : dblarr( pixels[0], pixels[1] ), $
		mom1  : dblarr( pixels[0], pixels[1] ), $
		mom2  : dblarr( pixels[0], pixels[1] ), $
		count : dblarr( pixels[0], pixels[1] ), $
		mask  : mask, $
		cover : dblarr( pixels[0], pixels[1] ), $
		weight: dblarr( pixels[0], pixels[1] ), $
		coadd : dblarr( pixels[0], pixels[1] ), $
		stdev : dblarr( pixels[0], pixels[1] ) }

    if ( keyword_set( AOCU2GADS ) ) then begin
	FOR k = 0, n_elements( tsdlist ) - 1 do begin
;	    Copy RA and DEC in AOCU into GADS
	    tsd = tsdlist[k]
	    ra	= tsd->get_value_tsd( 'AOCU', 'RA' )
	    dec = tsd->get_value_tsd( 'AOCU', 'DEC' )
	    rol = tsd->get_value_tsd( 'AOCU', 'rang' )

	    ret = tsd->set_value_tsd( 'GADS', 'RA', ra )
	    ret = tsd->set_value_tsd( 'GADS', 'DEC', dec )
	    ret = tsd->set_value_tsd( 'GADS', 'rang', rol )
	endfor
    endif

    FOR iter = 0, destripe do begin
	FOR k = 0, n_elements( tsdlist ) - 1 do begin

;	    openr, 1, tsdlist[k], error=err
;	    IF err NE 0 THEN BEGIN
;		lmessage, 0 , 'TSD File not found'
;	    ENDIF
;	    close, 1

;	    tsd = Obj_New( 'tds', infile=tsdlist[k] )

	    tsd = tsdlist[k]

	    r = im_scan( tsd, mapset, mapinfo, detector=detector, $
			 plot=do_plot, scannr=k )
	ENDFOR

	mapinfo.destr = 1			; set destripe option
	if ( iter LT n_elements( beam ) ) then $
		mapinfo.beam = beam[iter] / 60.0

	mapset.weight[*,*] = mapset.mom0
	mapset.coadd[*,*]  = mapset.mom1
	mapset.stdev[*,*]  = mapset.mom2
	mapset.cover[*,*]  = mapset.count

	b = where( mapset.mom0 GT 0, nb )
	IF ( nb GT 0 ) THEN BEGIN
	    mapset.coadd[b] = mapset.mom1[b] / mapset.mom0[b]
	    mapset.stdev[b] = sqrt( mapset.mom2[b] / mapset.mom0[b] - $
				    mapset.coadd[b] ^ 2 )
	ENDIF ELSE BEGIN
	    lmessage, 1, "No overlap found. Stopping."
	    retall
	ENDELSE
	mapset.mom0[*,*]  = 0
	mapset.mom1[*,*]  = 0
	mapset.mom2[*,*]  = 0
	mapset.count[*,*] = 0
    ENDFOR

    if ( pltype EQ 7 ) then stop_hc

    image = mapset.coadd
    cover = mapset.cover
    stdev = mapset.stdev
    weight= mapset.weight

    if ( n_elements( file ) GT 0 ) then $
	r = im2fits( image, mapinfo, file )


    return, !SUCCEED

END


