;+
; NAME:
;   badpix_corr
;
; PURPOSE:
;   Mask to the bad pixels and interpolate them
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by M.Shirahata, 2007/06/21, using maskinterp package
;   2007/07/04   M.Shirahata,  changed the default parameter of max_aperture
;   2007/07/05   M.Shirahata,  very very small bug fix
;   2007/08/27   M.Shirahata,  modification according to the change of the input IDL save format
;   2007/08/31   M.Shirahata,  bug fix.
;   2007/09/04   M.Shirahata,  add /cube_fits option.
;   2007/09/11   M.Shirahata,  add some information in IDL save file.
;   2007/09/12   M.Shirahata,  change array name. coad_err --> coad_stderr
;
;-
pro badpix_corr, IN_FILE = in_file, FUNC = func, CUBE_FITS = cube_fits


; Help print
if (( n_elements(func) eq 0 ) || ( n_elements(in_file) eq 0 )) then begin
    print, 'You can choose the interpolating functions'
    print, '   plsfit : plane surface fit (z = ax +by +c)'
    print, '   gausfit : gaussain surface fit (z = (1/(2*pi*thx*thy))*exp(-.5*(((x-mx)/thx)^2+((y-my)/thy)^2 ))-c)'
    print, '   chebyshfit : Chebyshev polynomial interpolation (z = sum(Ti*Ci)  + .5*C0)'
    print, '   csplinterp : bicubic spline interpolation'
    print, '   splinterp : bicubic spline interpolation'
    print, '   twoordfit : second-order polynomial surface interpolation (z = ax^2 + bxy + cy^2 + dx + ey + f)'
endif

if ( n_elements(in_file) eq 0 ) then begin
    return
endif


; Print function
if n_elements(func) eq 0 then begin
    func ='gausfit'
    print, 'Now, try to use Gausian fit ...'
endif else begin
    print, 'Now, you use ', func, ' fit ...'
endelse


; Raw Image File
; Filename
	str = strsplit(in_file,'.',/extract)
	in_file_header = str[0]
        out_file = in_file_header+'_corr.sav'
        out_fits = in_file_header+'_corr'

; Restore IDL data
        restore, filename=in_file
	print, 'Restored coadd image data:', in_file
	help, coad_det, coad_stderr, coad_num, im_lon_coord, im_lat_coord, $
          data_type, map_mean, grid_size, hdr

        x_size = (size(im_lat_coord))[1]
        y_size = (size(im_lat_coord))[2]
	raw = dblarr(x_size, y_size)
	mask = dblarr(x_size, y_size)
	corr = dblarr(x_size, y_size)
        corr_fraction = dblarr(2)
        okok_cnt = 0.0D

; Main Loop
        for j=0, 3 do begin

            ; Wide Image
            if (j eq 0) then begin
                raw[*,*] = coad_det[0,*,*]
                mask[*,*] = 0.0D
                corr[*,*] = 0.0D
                inmean = map_mean[0,0]
            endif
            ; Wide Error
            if (j eq 1) then begin
                raw[*,*] = coad_stderr[0,*,*]
                mask[*,*] = 0.0D
                corr[*,*] = 0.0D
                inmean = map_mean[0,1]
            endif
            ; Narrow Image
            if (j eq 2) then begin
                raw[*,*] = coad_det[1,*,*]
                mask[*,*] = 0.0D
                corr[*,*] = 0.0D
                inmean = map_mean[1,0]
            endif
            ; Narrow Error
            if (j eq 3) then begin
                raw[*,*] = coad_stderr[1,*,*]
                mask[*,*] = 0.0D
                corr[*,*] = 0.0D
                inmean = map_mean[1,1]
            endif

        ; Make Mask Array
            ; Detect badpixels
            for i = 0L, x_size-1 do begin

                ; Wide
                ok_idx = where(raw[i,*] ne 0, ok_cnt)
                if (ok_cnt gt 0) then begin ; OK data is exist

                    ; Min
                    if (ok_idx[0]-1 le 0) then begin
                        a = 0
                    endif else begin
                        a = ok_idx[0]-1
                    endelse

                    ; Max
                    if (ok_idx[ok_cnt-1]+1 ge y_size-1) then begin
                        b =y_size-1
                    endif else begin
                        b = ok_idx[ok_cnt-1]+1
                    endelse

                    ; Input the flug to the mask array
                    mask[i,0:a] = 1
                    mask[i,b:y_size-1] = 1
                    mask[i,ok_idx] = 1

                    ; Input the mean value to the outer region
                    raw[i,0:a] = inmean
                    raw[i,b:y_size-1] = inmean

                endif else begin ; All data is not good
                    mask[i,*] = 1
                    raw[i,*] = inmean
                endelse
                
                okok_cnt = okok_cnt+ok_cnt
                
            endfor

            ; Correct to the outer region
            corr = maskinterp(raw, mask, 1, 5, func)

            bad_idx = where(mask[*,*] eq 0, bad_cnt)
            bad_fraction = double(bad_cnt) / double(okok_cnt+bad_cnt) * 100

            ; Wide Image
            if (j eq 0) then begin
                print, 'Wide Band Corrected Image was completed.'
                corr_fraction[0] = bad_fraction
                print, 'Corrected pixel number was:', $
                  Strcompress(String(bad_cnt)), ' /', Strcompress(String(okok_cnt+bad_cnt)), $
                  ' =', Strcompress(String(bad_fraction))
                coad_det[0,*,*] = corr[*,*]
            endif
            ; Wide Error
            if (j eq 1) then begin
                print, 'Wide Band Corrected Error was completed.'
                coad_stderr[0,*,*] = corr[*,*]
            endif

            ; Narrow Image
            if (j eq 2) then begin
                print, 'Narrow Band Corrected Image was completed.'
                corr_fraction[1] = bad_fraction
                print, 'Corrected pixel number was:', $
                  Strcompress(String(bad_cnt)), ' /', Strcompress(String(okok_cnt+bad_cnt)), $
                  ' =', Strcompress(String(bad_fraction))
                coad_det[1,*,*] = corr[*,*]
            endif
            ; Narrow Error
            if (j eq 3) then begin
                print, 'Narrow Band Corrected Error was completed.'
                coad_stderr[1,*,*] = corr[*,*]
            endif

        ; End of Main Loop
        endfor


; Save IDL data
        save, im_lon_coord, im_lat_coord, coad_det, coad_stderr, coad_num, $
          data_type, map_mean, grid_size, $
          corr_fraction, func, hdr, filename=out_file
        print, 'Output corrected coadd image data in IDL format:', out_file


; Save Fits data

        IF keyword_set(cube_fits) THEN BEGIN

            ; Create 3D data cube FITS
            w_out_fits = [ coad_det[0,*,*], coad_stderr[0,*,*], coad_num[0,*,*] ]
            w_out_fits = transpose(w_out_fits, [1,2,0])
            n_out_fits = [ coad_det[1,*,*], coad_stderr[1,*,*], coad_num[1,*,*] ]
            n_out_fits = transpose(n_out_fits, [1,2,0])

            sxaddpar, hdr, 'NAXIS',  3
            sxaddpar, hdr, 'NAXIS3', 3
            sxaddpar, hdr, 'CTYPE3', 'DataCube', 'Intensity, Error, Integrated Sample#'
            writefits, out_fits+'_cube_w.fits', float(w_out_fits), hdr
            writefits, out_fits+'_cube_n.fits', float(n_out_fits), hdr
            print, 'Output map data in 3D cube FITS:', out_fits+'_cube_[w|n].fits'

        ENDIF ELSE BEGIN

            ; by default
            writefits, out_fits+'_img_w.fits', float(reform(coad_det[0,*,*])), hdr
            writefits, out_fits+'_img_n.fits', float(reform(coad_det[1,*,*])), hdr
            print, 'Output intensity map in FITS format:', out_fits, '_img_[w|n].fits'
   
            writefits, out_fits+'_err_w.fits', float(reform(coad_stderr[0,*,*])), hdr
            writefits, out_fits+'_err_n.fits', float(reform(coad_stderr[1,*,*])), hdr
            print, 'Output error map in FITS format:', out_fits, '_err_[w|n].fits'
    
            writefits, out_fits+'_num_w.fits', float(reform(coad_num[0,*,*])), hdr
            writefits, out_fits+'_num_n.fits', float(reform(coad_num[1,*,*])), hdr
            print, 'Output integrated sample number map in FITS format:', out_fits, '_num_[w|n].fits'

        ENDELSE


; End of this pro
end
