;+
; NAME:
;   color_corr
;
; PURPOSE:
;   output the color correction factor
;
; MODIFICATION HISTORY:
;   written by M.Shirahata, 2007/07/31
;      2007/08/02   M.Shirahata   power up!
;-

PRO color_corr, band, bb=bb, power=power, factor, CHECK=check

; Display the usage
    if (n_params() eq 0) then begin
        print, "# USAGE #"
        print, "color_corr, band, bb=[temp,beta] or power=[alpha] [, /check]"
        print, "   band      : 0=N60, 1=Wide-S, 2=Wide-L, 3=N160"
        print, "   spectrum  : Blackbody or Power Law"
        print, "               Blackbody  : bb=[temp,beta]"
        print, "               Power Law  : power=[alpha]"
        print, "   factor    : color correction factor (output)"
        print, "   /check    : display the responce graph"
        return
    endif


;;; IMPORTANT MEMO ;;;
; N60    :   65.0um, 153.8cm-1, 4.612THz
; Wide-S :   90.0um, 111.1cm-1, 3.331THz
; Wide-L :  140.0um,  71.4cm-1, 2.141THz
; N160   :  160.0um,  62.5cm-1, 1.874THz


; Physical constant
Kousoku = 2.99792458e+8
Plank = 6.62607e-34
Boltzmann = 1.38065e-23


; Make array
    RSRF_table = dblarr(8,100)
    lambda = dblarr(100)
    frequency = dblarr(100)
    spec_resp = dblarr(100)


;  RSRF file Open
    root = GETENV('FISDR_ROOT')
    OpenR, lun_in, root+'/slowscan/lib/FIS_RSRF_070122.txt', /get_lun, Error=oper

    if(oper eq 0) then begin
        print, 'Read FIS_RSRF_070122.txt file'
    endif else begin
        print, 'Cannot read the FIS_RSRF_070122.txt file !!!'
        return
    endelse


; Read Data from input table
    skip = ''
    ReadF, lun_in, skip
    ReadF, lun_in, RSRF_table


; File Close
    Free_lun, lun_in


; Band selection
    if (band eq 0) then begin
        print, '--- N60 band ---'
        band_name = 'N60'
        c_lambda = 65.0
        c_frequency = 4.612e+12
        lambda = RSRF_table[0,*]
        frequency = RSRF_table[1,*]
        spec_resp = RSRF_table[2,*]
    endif else if (band eq 1) then begin
        print, '--- Wide-S band ---'
        band_name = 'Wide-S'
        c_lambda = 90.0
        c_frequency = 3.331e+12
        lambda = RSRF_table[0,*]
        frequency = RSRF_table[1,*]
        spec_resp = RSRF_table[3,*]
    endif else if (band eq 2) then begin
        print, '--- Wide-L band ---'
        band_name = 'Wide-L'
        c_lambda = 140.0
        c_frequency = 2.141e+12
        lambda = RSRF_table[4,*]
        frequency = RSRF_table[5,*]
        spec_resp = RSRF_table[6,*]
    endif else if (band eq 3) then begin
        print, '--- N160 band ---
        band_name = 'N160'
        c_lambda = 160.0
        c_frequency = 1.874e+12
        lambda = RSRF_table[4,*]
        frequency = RSRF_table[5,*]
        spec_resp = RSRF_table[7,*]
    endif else begin
        print, 'Please select the photometric band !'
        print, '   1: N60'
        print, '   2: Wide-S'
        print, '   3: Wide-L'
        print, '   4: N160'
        return
    endelse


; Spectrum
    if ( $
         ((n_elements(bb) eq 0) and (n_elements(power) eq 0)) $
         or $
         ((n_elements(bb) ge 1) and (n_elements(power) ge 1)) $
       ) then begin
        print, 'Please choose the SED type: Blackbody or Power Law ?'
        print, 'bb=[temp,beta] or power=[alpha]'
        return
    endif

    ; Blackbody
    if (n_elements(power) eq 0) then begin
        if (n_elements(bb) eq 2) then begin
            spec_type = 'Blackbody'
            temp = bb[0]
            beta = bb[1]
            if (temp le 0.0) then begin
                print, 'Blackbody temperature must be grater than 0.0(K)'
                return
            endif
        endif else begin
            print, 'Please input the blackbody temperature and beta'
            print, 'bb=[temp,beta]'
            return
        endelse
    endif

    ; Power law
    if (n_elements(bb) eq 0) then begin
        if (n_elements(power) eq 1) then begin
            spec_type = 'PowerLaw'
            alpha = power[0]
        endif else begin
            print, 'Please input the power law index'
            print, 'power=[alpha]'
            return
        endelse
    endif


; Blackbody/PowerLaw spectrum Bandwidth calculation
    if (spec_type eq 'Blackbody') then begin
        SED_spec = spec_resp * ( 2*Plank*frequency^3 / Kousoku^2 ) $
          / ( exp( (Plank*frequency)/(Boltzmann*temp) ) -1 ) $
          * frequency^(beta) $
          / ( 2*Plank*c_frequency^3 / Kousoku^2 ) $
          * ( exp( (Plank*c_frequency)/(Boltzmann*temp) ) -1 ) $
          / c_frequency^(beta)
    endif

    if (spec_type eq 'PowerLaw') then begin
        SED_spec = spec_resp * frequency^(alpha) / c_frequency^(alpha)
    endif


; Flat spectrum Bandwidth calculation
    flat = spec_resp / frequency * c_frequency


; Integration
    d_nu_flat = int_tabulated(frequency, flat, /double)
    d_nu_SED = int_tabulated(frequency, SED_spec, /double)
    k_SED = d_nu_SED / d_nu_flat


; Results
    if (spec_type eq 'Blackbody') then begin
        print, 'Blackbody Spectrum : temperature =' + Strcompress(String(temp)) + '(K), ' $
          +'beta =' + Strcompress(String(beta))
        print, 'd_nu_flat :' + Strcompress(d_nu_flat/1.0e+12) + ' (THz)'
        print, 'd_nu_SED  :' + Strcompress(d_nu_SED/1.0e+12) + ' (THz)'
        print, 'Color correction factor :' + Strcompress(k_SED)
    endif
    if (spec_type eq 'PowerLaw') then begin
        print, 'Plwer law Spectrum : index =' + Strcompress(String(alpha))
        print, 'd_nu_flat :' + Strcompress(d_nu_flat/1.0e+12) + ' (THz)'
        print, 'd_nu_SED  :' + Strcompress(d_nu_SED/1.0e+12) + ' (THz)'
        print, 'Color correction factor :' + Strcompress(k_SED)
    endif


; RSRF check
    if keyword_set(CHECK) then begin
        plot, lambda, spec_resp, $
          xrange=[0, 250], yrange=[0, 2.0], $
          xtitle='Wavelength [um]', ytitle='Relative Response', title='AKARI FIS/'+band_name, $
          charsize=1.5

        if (spec_type eq 'Blackbody') then begin
            memo0 = 'BB('+Strcompress(String(temp))+'K) * nu^('+Strcompress(String(beta))+')'
        endif
        if (spec_type eq 'PowerLaw') then begin
            memo0 = 'Power Law (index='+Strcompress(String(alpha))+')'
        endif

        memo1 = 'Responce curve'
        memo2 = 'SED * Responce'
        memo3 = 'Flat spectrum * Responce'

        xyouts, 0.57, 0.88, memo0, charsize=1.4, /normal, color='7F7FFF'XL
        xyouts, 0.57, 0.83, memo1, charsize=1.2, /normal
        xyouts, 0.57, 0.79, memo2, charsize=1.2, /normal, color='93DB70'XL
        xyouts, 0.57, 0.75, memo3, charsize=1.2, /normal, color='FFA300'XL

        oplot, lambda, SED_spec, color='93DB70'XL
        oplot, lambda, flat, color='FFA300'XL
    endif


end
