;+
; NAME:
;   ss_coad_image
;
; PURPOSE:
;   Make co-add image of the processed data
;
; OUTPUT:
;    Co-added map data, positional data, etc.
;    as IDL save file ('*_img.sav') and FITS files.
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S. Matsuura, 2006/09/05
;   2006/10/04  S.Matsuura,   minor modification
;   2006/10/18  I.Yamamura,   renamed as ss_coad_image
;   2006/11/28  S.Matsuura,   save coad image data in IDL format
;   2006/12/21  I.Yamamura,   exit gently when no GADS is available
;   2006/12/21  I.Yamamura,   window control revised not to use too many windows
;   2006/12/26  S.Matsuura,   include 'AOT' as an input parameter
;   2006/12/27  S.Matsuura,   changed ra-dec grid size, include 'RST' as an input parameter
;   2006/12/29  Makiuti, S.,  Added /NO_DISPLAY options.
;   2006/12/29  S.Matsuura,   included area boundary information as input parameter.
;                             In case of "MAPPING" this boundary determines the grid in entire map.
;   2006/12/30  M.Shirahata,  modify the plot style.
;   2007/01/05  Makiuti, S.,  Improved error handling.
;                             Added ERROR_STATUS option argument to return error status.
;   2007/01/06  I.Yamamura,   Data over R.A.=360deg can be mapped.
;   2007/01/09  S.Matsuura,   Sigma for bad data rejection can be specified.
;   2007/01/10  I.Yamamura,   image plot in right R.A. orientation and square grid.
;   2007/01/10  S.Matsuura,   chenged position of IF-statement for "mapping".
;   2007/01/17  I.Yamamura,   /ISOTROPIC option removed from CONTOUR call.
;   2007/01/18  M.Shirahata,  changed the default value of sigma from 5 to 2.
;   2007/01/18  S.Matsuura,   changed the scan speed limit to 30"/s for special operation.
;   2007/01/20  S.Matsuura,   simple mean -> weighted mean for co-adding.
;   2007/01/23  S.Matsuura,   included bad data flag found in the flat fielding process.
;   2007/01/30  S.Matsuura,   if det=0 then no contribution to coad image.
;   2007/01/30  M.Shirahata,  change how to treat the t_start and t_end options.
;                             add new option (/newmapping method).
;                             abolished making the PS file.
;   2007/02/02  M.Shirahata,  add data_type in output IDL format data.
;   2007/02/05  S.Matsuura,   small modification in area boundary determination process.
;   2007/02/06  S.Matsuura,   grid size is autonatically set according to N_RAMP_DIV(n_int).
;   2007/02/06  M.Shirahata,  add /SCUT option.
;   2007/02/06  S.Matsuura,   bug fix for add /SCUT option.
;   2007/02/06  M.Shirahata,  return to previous version & bug fix for /SCUT option.
;   2007/02/07  M.Shirahata,  bug fix.
;   2007/02/08  S.Matsuura,   can specify grid size as input option.
;                             Auto grid sizing with n_int is disused.
;   2007/02/08  S.Matsuura,   calculation of RMS fluctution of final coadded data.
;   2007/02/09  S.Matsuura,   Added calculation of mean level of coad image and revised RMS fluctution.
;   2007/02/09  S.Matsuura,   Included map_mean, dec_grid and ra_grid in saved IDL-format image data.
;   2007/02/21  Makiuti, S.,  Set a color table explicitly for contour map display.
;   2007/03/27  M.Shirahata,  Small bug fix for IDL plot. No change to data analysis process.
;   2007/06/06  M.Shirahata,  Small bug fix.
;   2007/07/02  Makiuti, S.   Added LON(LAT)_CENTER options to specify map center position.
;                             Added LON(LAT)_SIZE options to specify map size.
;   2007/07/03  M.Shirahata,  Released the automatic setting function for too grid size
;                             when used via ss_multi_scan_map2.
;   2007/07/24  S.Matsuura,   accept AOT OT_FIS051.
;   2007/08/29  Makiuti, S.   Implemented Gnomonic projection
;                             Added /ECLIPTIC and /GALACTIC options
;   2007/08/31  Makiuti, S.   Added /CUBE_FITS option
;   2007/08/31  M.Shirahata,  Small bug fix.
;   2007/09/03  Makiuti, S.   LON_SIZE value is in unit of degree.
;   2007/09/03  M.Shirahata,  remove mapping option.
;                             rename /newmapping --> /mosaic
;   2007/09/04  Makiuti, S.   bug-fix to solve a minor binning problem
;   2007/09/11  Jeong, W-S.    Added 'Pixel Mapping method' with option /Pix_Mapping
;   2007/09/12  Makiuti, S.   Replaced STDDEV with STDERR in error map, and bug-fix
;   2007/09/14  Makiuti, S.   Not to reject one-sample pixels by sigma-clipping
;   2007/09/14  S.Matsuura,   changed criteria for scan-speed descrimination
;
;-
pro ss_coad_image, x, ra, dec, det, err, eg_gl_cnt, $
                   file_header, data_type, AOT, RST, rst_int, n_int, bad_flg, $
                   coad_det, coad_num, coad_err, $
                   ra_min, ra_max, dec_min, dec_max, $
                   NO_DISPLAY = no_display, ERROR_STATUS = error_status, $
                   SIGMA = sigma, T_START = t_start, T_END = t_end, $
                   mosaic = mosaic, SCUT = scut, $
                   GRID_SW = grid_sw, GRID_LW = grid_lw, $
                   LON_CENTER = lon_center, LAT_CENTER = lat_center, $
                   LON_SIZE = lon_size, LAT_SIZE = lat_size, $
                   ECLIPTIC = ecliptic, GALACTIC = galactic, $
                   CUBE_FITS = cube_fits, Pix_Mapping = Pix_Mapping

!p.multi = [0,0,0]
RAD2SEC = 206264.8062470963551572D ;[arcsec/rad]
RAD2DEG = 57.295779513082320877D   ;[deg/rad]


; Check the parameters and initialize
IF (n_elements(no_display) EQ 0) THEN no_display=0

; variables specified for detector
if(data_type eq 'FIS_SW') then begin
    c_pix = 20
    n_pix = 100
    w_pix = 60
    beam_size = 40.0D         ; FWHM in 1-D profile
    smpl_freq = 25.28D        ; [Hz]
endif else begin   ;'FIS_LW'
    c_pix = 15
    n_pix = 75
    w_pix = 45
    beam_size = 60.0D         ; FWHM in 1-D profile
    smpl_freq = 16.86D        ; [Hz]
endelse


if keyword_set(mosaic) then goto, skipmosaic1

; Scan Cut
if (scut eq 0) then goto, skipscut1

; in case of scut=ON
x0=x
ra0=ra
dec0=dec
det0=det
err0=err
eg_gl_cnt0=eg_gl_cnt

CASE AOT OF
    'PV_FIS01':       scan=4
    'PV_FIS02':       scan=2
    'P12_FIS01':      scan=4
    'P12_FIS02':      scan=2
    'P12_FIS01_SAA':  scan=4
    'P12_FIS02_SAA':  scan=2
    'OT_FIS51':       scan=4
ENDCASE

; Scan Cut Loop Start
for sloop=0, scan DO BEGIN

; File name etc
    snum = Strcompress(String(sloop+1), /remove_all)
    x=x0
    ra=ra0
    dec=dec0
    det=det0
    err=err0
    eg_gl_cnt=eg_gl_cnt0

    ; Time range set
    if (AOT eq 'PV_FIS01') then begin
        if (sloop eq 0) then begin
            t_start=750
            t_end=900
        endif
        if (sloop eq 1) then begin
            t_start=926
            t_end=1050
        endif
        if (sloop eq 2) then begin
            t_start=1076
            t_end=1200
        endif
        if (sloop eq 3) then begin
            t_start=1226
            t_end=1350
        endif
        if (sloop eq scan) then begin
            t_start=750
            t_end=1350
        endif
    endif

    if (AOT eq 'PV_FIS02') then begin
        if (sloop eq 0) then begin
            t_start=750
            t_end=1050
        endif
        if (sloop eq 1) then begin
            t_start=1076
            t_end=1350
        endif
        if (sloop eq scan) then begin
            t_start=750
            t_end=1350
        endif
    endif

    if ((AOT eq 'P12_FIS01') || (AOT eq 'P12_FIS01_SAA') || (AOT eq 'OT_FIS51')) then begin
        if (sloop eq 0) then begin
            t_start=630
            t_end=787
        endif
        if (sloop eq 1) then begin
            t_start=818
            t_end=975
        endif
        if (sloop eq 2) then begin
            t_start=1005
            t_end=1162
        endif
        if (sloop eq 3) then begin
            t_start=1193
            t_end=1350
        endif
        if (sloop eq scan) then begin
            t_start=630
            t_end=1350
        endif
    endif

    if ((AOT eq 'P12_FIS02') || (AOT eq 'P12_FIS02_SAA')) then begin
        if (sloop eq 0) then begin
            t_start=630
            t_end=975
        endif
        if (sloop eq 1) then begin
            t_start=1005
            t_end=1350
        endif
        if (sloop eq scan) then begin
            t_start=630
            t_end=1350
        endif
    endif

    print, 'Time range was set from', Strcompress(t_start) +' to', Strcompress(t_end) +'.'

    ; t_start
    t_start_idx = where(x gt t_start*smpl_freq, t_start_cnt)
    x = x[t_start_idx]
    n_line = n_elements(x)
    ra = ra[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
    dec = dec[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
    det = det[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
    err = err[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
    eg_gl_cnt = eg_gl_cnt[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]

    ; t_end
    t_end_idx = where(x lt t_end*smpl_freq, t_end_cnt)
    x = x[t_end_idx]
    n_line = n_elements(x)
    ra = ra[*,0:t_end_cnt-1]
    dec = dec[*,0:t_end_cnt-1]
    det = det[*,0:t_end_cnt-1]
    err = err[*,0:t_end_cnt-1]
    eg_gl_cnt = eg_gl_cnt[*,0:t_end_cnt-1]

skipscut1:

if (scut ne 0) then goto, skipcut2

; Time range set
if (n_elements(t_start) eq 0) then begin
    CASE AOT OF
        'PV_FIS01':       t_start=750
        'P12_FIS01':      t_start=630
        'P12_FIS01_SAA':  t_start=630
        'PV_FIS02':       t_start=750
        'P12_FIS02':      t_start=630
        'P12_FIS02_SAA':  t_start=630
        'OT_FIS51':       t_start=630
    ENDCASE
endif
if (n_elements(t_end) eq 0) then begin
    t_end = 1350
endif
print, 'Time range was set from', Strcompress(t_start) +' to', Strcompress(t_end) +'.'

; t_start
t_start_idx = where(x gt t_start*smpl_freq, t_start_cnt)
x = x[t_start_idx]
n_line = n_elements(x)
ra = ra[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
dec = dec[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
det = det[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
err = err[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]
eg_gl_cnt = eg_gl_cnt[*,t_start_idx[0]:t_start_idx[0]+t_start_cnt-1]

; t_end
t_end_idx = where(x lt t_end*smpl_freq, t_end_cnt)
x = x[t_end_idx]
n_line = n_elements(x)
ra = ra[*,0:t_end_cnt-1]
dec = dec[*,0:t_end_cnt-1]
det = det[*,0:t_end_cnt-1]
err = err[*,0:t_end_cnt-1]
eg_gl_cnt = eg_gl_cnt[*,0:t_end_cnt-1]

skipcut2:
skipmosaic1:

    ; R.A. in relative expression
    ra_med = MEDIAN(ra)
    print, 'RA_med = ', ra_med
    ra -= ra_med
    if ((MAX(ra) - MIN(ra)) GT 300D) then begin
        if (ra_med GT 180D) then begin
            idx = WHERE (ra LT -180D, cnt)
            if (cnt GT 0) then ra[idx] += 360D
        endif else begin
            idx = WHERE (ra GT 180D, cnt)
            if (cnt GT 0) then ra[idx] -= 360D
        endelse
    endif

    ; scan-speed calculation
    n_int_dbl = double(n_int)
    data_int = rst_int/n_int_dbl
    print, 'Data interval [s]:', data_int
    if(data_int gt 0) then begin
        sp_dec = (dec[0,*]-shift(dec[0,*], 1))/data_int*3600.0D
        sp_ra = cos(dec[0,*]/RAD2DEG)*(ra[0,*]-shift(ra[0,*], 1))/data_int*3600.0D
        abs_speed = sqrt(sp_dec^2+sp_ra^2)
        med_sp_dec = median(abs(sp_dec),/even)
        med_sp_ra = median(abs(sp_ra),/even)
        med_speed = median(abs_speed,/even)

        print, 'Median scan speed [arcsec/s]'
        print, '    angular_speed(RA):', med_sp_ra
        print, '    angular_speed(DEC):', med_sp_dec
        print, '    Absolute:', med_speed
    endif else begin
        message, 'maybe CDS mode, but... Check data!', /continue
        error_status = 1 ; set error status
        return
    endelse
    if (med_speed gt 3 and med_speed lt 5) then scan_speed = 4
    if (med_speed gt 6 and med_speed lt 10) then scan_speed = 8
    if (med_speed gt 12 and med_speed lt 18) then scan_speed = 15
    if (med_speed gt 24 and med_speed lt 36) then scan_speed = 30
    print, 'AOT specified scan speed [arcsec/s]: ', scan_speed
    if (med_speed lt 3 and med_speed gt 36) then begin
    MESSAGE, "Unknown scan speed. Process stopped. Check data!", /CONTINUE
    error_status = 1 ; set error status
    return
    endif


    ; define scan area boundary
    scan_t_idx = where(abs_speed lt 31, scan_t_cnt)
        if (scan_t_cnt GT 0) then begin
            ra_sub  = ra[*,scan_t_idx]
            dec_sub = dec[*,scan_t_idx]
        endif else begin
            MESSAGE, "No proper attitude information. Process stopped. Check data!", /CONTINUE
            error_status = 1 ; set error status
            return
        endelse

    ra_max = max(ra_sub)
    ra_min = min(ra_sub)
    dec_max = max(dec_sub)
    dec_min = min(dec_sub)

    print, 'Plot area [deg]'
    print, '    RA_min:', ra_min+ra_med
    print, '    RA_max:', ra_max+ra_med
    print, '    DEC_min:', dec_min
    print, '    DEC_max:', dec_max

    ; define grid size
    IF (AOT eq 'PV_FIS01' or AOT eq 'P12_FIS01' or AOT eq 'P12_FIS01_SAA' $
      or AOT eq 'OT_FIS51') THEN BEGIN
      if(data_type eq 'FIS_SW') then begin
          grid_size = 15D/3600D
          if (n_elements(grid_sw) eq 1) then grid_size = grid_sw/3600D
      endif else begin
          grid_size = 30D/3600D
          if (n_elements(grid_lw) eq 1) then grid_size = grid_lw/3600D
      endelse
      grid_limit = beam_size/3600D/n_int*(scan_speed/15D)/4D
    ENDIF ELSE BEGIN
      if(data_type eq 'FIS_SW') then begin
          grid_size = 30D/3600D
          if (n_elements(grid_sw) eq 1) then grid_size = grid_sw/3600D
      endif else begin
          grid_size = 60D/3600D
          if (n_elements(grid_lw) eq 1) then grid_size = grid_lw/3600D
      endelse
      grid_limit = beam_size/3600D/n_int*(scan_speed/15D)/2D
    ENDELSE

if keyword_set(mosaic) then goto, skipmosaic3

    ; auto limit of grid size
    if(grid_size lt grid_limit) then begin
        print, '% Input grid size is too small! It may cause a lot of empty grid cells in the map.'
        grid_size = grid_limit
        print, 'Grid size is automatically set to [arcsec]: ', grid_size*3600D
    endif

skipmosaic3:

    ; RA, DEC -> lon, lat (in arbitrary coordinate)
    lon      = ra + ra_med
    lat      = dec

    ; Coordinate conversion
    IF keyword_set(Ecliptic) THEN BEGIN
       euler, lon, lat, lon, lat, 3 ; RA-DEC (J2000) to Ecliptic
    ENDIF ELSE IF keyword_set(Galactic) THEN BEGIN
       euler, lon, lat, lon, lat, 1 ; RA-DEC (J2000) to Galactic
    ENDIF

    ; Selected by scan-speed
    lon_sub = lon[*, scan_t_idx]
    lat_sub = lat[*, scan_t_idx]

    ; Max & min of LON
    lon_max = max(lon_sub)
    lon_min = min(lon_sub)
    ; When the area stride across 360deg
    IF ((lon_max-lon_min) GT 300) THEN BEGIN
      idx = where(lon GT 180, cnt)
      IF (cnt GT 0) THEN lon[idx] -= 360.0
      lon_sub = lon[*, scan_t_idx]
      lat_sub = lat[*, scan_t_idx]
      lon_max = max(lon_sub)
      lon_min = min(lon_sub)
    ENDIF

    ; Max & min of LAT
    lat_max = max(lat_sub)
    lat_min = min(lat_sub)

    ; Define map area center
    lon_mean = mean(clipdt(lon_sub, clipnum=1))
    lat_mean = mean(clipdt(lat_sub, clipnum=1))

    ; If center position were NOT explicitly set, set them here
    IF (n_elements(lon_center) EQ 0) THEN lon_mapcenter = lon_mean $
                                     ELSE lon_mapcenter = lon_center
    IF (n_elements(lat_center) EQ 0) THEN lat_mapcenter = lat_mean $
                                     ELSE lat_mapcenter = lat_center

    ; If map sizes were NOT explicitly set, set them here
    IF (n_elements(lon_size) EQ 0) THEN BEGIN
      lon_size_tmp = lon_max - lon_min ; in unit of any coordinate
      lon_mapsize  = lon_size_tmp*cos(lat_mapcenter/RAD2DEG) ; in unit of degree
    ENDIF ELSE BEGIN
      lon_mapsize = lon_size ; in unit of degree
    ENDELSE
    IF (n_elements(lat_size) EQ 0) THEN BEGIN
      lat_mapsize = lat_max - lat_min
    ENDIF ELSE BEGIN
      lat_mapsize = lat_size ; in unit of degree
    ENDELSE

    ; Define map region
    lon_min = lon_mapcenter - lon_mapsize/cos(lat_mapcenter/RAD2DEG)/2.0
    lon_max = lon_mapcenter + lon_mapsize/cos(lat_mapcenter/RAD2DEG)/2.0
    lat_min = lat_mapcenter - lat_mapsize/2.0
    lat_max = lat_mapcenter + lat_mapsize/2.0

    ; Display map info.
    IF keyword_set(Ecliptic) THEN coordinate = 'Ecliptic' $
      ELSE IF keyword_set(Galactic) THEN coordinate = 'Galactic' $
      ELSE coordinate = 'Equatorial'
    print, ''
    print, 'Map area (' + coordinate + ')'
    print, '    LON_min:', lon_min
    print, '    LON_max:', lon_max
    print, '    LAT_min:', lat_min
    print, '    LAT_max:', lat_max
    print, 'Map center (LON, LAT):', lon_mapcenter, lat_mapcenter
    print, 'Map size (LON, LAT)[degree] :'  , lon_mapsize, lat_mapsize

    ; Data size
    n_data = n_elements(det)/n_pix

    ;;;;;
    ; Make Image FITS header
    ;;;;;

    ;; Map pixel size
    lon_pix_num = fix(lon_mapsize/grid_size) + 1
    lat_pix_num = fix(lat_mapsize/grid_size) + 1
    print, 'Map pixel size: ', lon_pix_num, lat_pix_num
    ;; Map center pixel
    lon_cent_pix = lon_pix_num/2.0
    lat_cent_pix = lat_pix_num/2.0
    print, 'Map center pixel: ', lon_cent_pix, lat_cent_pix
    ;; Map grid size
    print, 'Map grid size [arcsec]; ', grid_size*3600.0D
    print, ''

    ; Find Time Stamp
    CALDAT, SYSTIME(/JULIAN, /UTC), mm, dd, yy, hh, ms, ss
    cdate = STRING(FORMAT='(I4.4,"-",I2.2,"-",I2.2)', yy, mm, dd, /PRINT)

    ; Define FITS header
    sxaddpar, hdr, 'SIMPLE', 'T'
    sxaddpar, hdr, 'NAXIS',   2
    sxaddpar, hdr, 'NAXIS1',  lon_pix_num
    sxaddpar, hdr, 'NAXIS2',  lat_pix_num
    IF keyword_set(Ecliptic) THEN BEGIN
      sxaddpar, hdr, 'CTYPE1',  'ELON-TAN',  ' Ecliptic Lon (Gnomonic projection)'
      sxaddpar, hdr, 'CTYPE2',  'ELAT-TAN',  ' Ecliptic Lat (Gnomonic projection)'
    ENDIF ELSE IF keyword_set(Galactic) THEN BEGIN
      sxaddpar, hdr, 'CTYPE1',  'GLON-TAN',  ' Galactic Lon (Gnomonic projection)'
      sxaddpar, hdr, 'CTYPE2',  'GLAT-TAN',  ' Galactic Lat (Gnomonic projection)'
    ENDIF ELSE BEGIN
      sxaddpar, hdr, 'CTYPE1',  'RA---TAN',  ' RA (Gnomonic projection)'
      sxaddpar, hdr, 'CTYPE2',  'DEC--TAN',  ' DEC (Gnomonic projection)'
    ENDELSE
    sxaddpar, hdr, 'CRVAL1',  lon_mapcenter,  ' Reference pixel value'
    sxaddpar, hdr, 'CRVAL2',  lat_mapcenter,  ' Reference pixel value'
    sxaddpar, hdr, 'CRPIX1',  lon_cent_pix,' Reference pixel'
    sxaddpar, hdr, 'CRPIX2',  lat_cent_pix,' Reference pixel'
    sxaddpar, hdr, 'CDELT1',  -grid_size,  ' Degrees/pixel'
    sxaddpar, hdr, 'CDELT2',  grid_size,   ' Degrees/pixel'
    IF ~keyword_set(Galactic) THEN BEGIN
      sxaddpar, hdr, 'EQUINOX', 2000.00,     ' Equinox of coordinates'
    ENDIF
    sxaddpar, hdr, 'BUNIT',   'MJY/SR',    ' Intensity'
    sxaddpar, hdr, 'DATE',    cdate,       ' Creation date'
    sxaddpar, hdr, 'TELESCOP','AKARI',     ''
    sxaddpar, hdr, 'INSTRUME','FIS',     ''
    ;print, hdr

    ; Projection
    extast, hdr, astr
    AD2XY, lon, lat, astr, im_x, im_y ; Gnomonic projection


;;;;;
; initial co-add calculation
;;;;;
    print, 'making initial co-add image ... '

    coad_num0    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_det0    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_err0    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_weight0 = dblarr(2, lon_pix_num, lat_pix_num)
    coad_mean0   = dblarr(2, lon_pix_num, lat_pix_num)
    coad_det_sq0 = dblarr(2, lon_pix_num, lat_pix_num)
    coad_stdev0  = dblarr(2, lon_pix_num, lat_pix_num)

    ; Wide band
    FOR pix = 0, w_pix-1 DO BEGIN
      IF (bad_flg[pix] EQ 0) THEN BEGIN
        for j = 0L, n_data-1 do begin

          i_x = fix(im_x[pix,j])
          i_y = fix(im_y[pix,j])

          if ( (i_x GE 0 AND i_y GE 0) $
              and ((i_x LT lon_pix_num) AND (i_y LT lat_pix_num)) $
              and (eg_gl_cnt[pix,j] EQ 0) $
              and (det[pix,j] NE 0D) $
              and (err[pix,j] GT 0D) ) then begin

                coad_num0[0,i_x,i_y]    = coad_num0[0,i_x,i_y] + 1D
                coad_det0[0,i_x,i_y]    = coad_det0[0,i_x,i_y] + det[pix,j]/err[pix,j]^2
                coad_weight0[0,i_x,i_y] = coad_weight0[0,i_x,i_y] + 1D/err[pix,j]^2
                coad_mean0[0,i_x,i_y]   = coad_mean0[0,i_x,i_y] + det[pix,j]
                coad_det_sq0[0,i_x,i_y] = coad_det_sq0[0,i_x,i_y] + det[pix,j]^2
          endif

        endfor
      ENDIF
    ENDFOR

    ; Narrow band
    FOR pix = w_pix, n_pix-1 DO BEGIN
      IF (bad_flg[pix] EQ 0) THEN BEGIN
        for j = 0L, n_data-1 do begin

          i_x = fix(im_x[pix,j])
          i_y = fix(im_y[pix,j])

          if( (i_x GE 0 AND i_y GE 0) $
              and ((i_x LT lon_pix_num) AND (i_y LT lat_pix_num)) $
              and (eg_gl_cnt[pix,j] EQ 0) $
              and (det[pix,j] NE 0D) $
              and (err[pix,j] GT 0D) ) then begin
                coad_num0[1,i_x,i_y]    = coad_num0[1,i_x,i_y] + 1D
                coad_det0[1,i_x,i_y]    = coad_det0[1,i_x,i_y] + det[pix,j]/err[pix,j]^2
                coad_weight0[1,i_x,i_y] = coad_weight0[1,i_x,i_y] + 1D/err[pix,j]^2
                coad_mean0[1,i_x,i_y]   = coad_mean0[1,i_x,i_y] + det[pix,j]
                coad_det_sq0[1,i_x,i_y] = coad_det_sq0[1,i_x,i_y] + det[pix,j]^2
            endif

        endfor
      ENDIF
    ENDFOR

; calculate mean and std_dev

    nz_idx = where((coad_num0 GT 0) AND (coad_weight0 GT 0), nz_cnt)
    print, 'Non-zero co-add number: ', nz_cnt
    IF (nz_cnt ne 0) THEN BEGIN
        coad_det0[nz_idx]    = coad_det0[nz_idx]/coad_weight0[nz_idx]
        coad_err0[nz_idx]    = sqrt(1D/coad_weight0[nz_idx])
        coad_mean0[nz_idx]   = coad_mean0[nz_idx]/coad_num0[nz_idx]
        coad_det_sq0[nz_idx] = coad_det_sq0[nz_idx]/coad_num0[nz_idx]
        coad_stdev0[nz_idx]  = sqrt(coad_det_sq0[nz_idx]-coad_mean0[nz_idx]^2)
    ENDIF

    ; To avoid rejection of one-sample pixels by sigma-clipping, 
    ;   set dummy stddev
    idx_1smp_pix = where(coad_num0 EQ 1, cnt_1smp_pix)
    IF (cnt_1smp_pix GT 0) THEN coad_stdev0[idx_1smp_pix] = median(coad_stdev0[nz_idx])

;;;;;
; remake co-add image with sigma rejection
;;;;;
    coad_num    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_det    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_err    = dblarr(2, lon_pix_num, lat_pix_num)
    coad_weight = dblarr(2, lon_pix_num, lat_pix_num)
    coad_mean   = dblarr(2, lon_pix_num, lat_pix_num)
    coad_det_sq = dblarr(2, lon_pix_num, lat_pix_num)
    coad_stdev  = dblarr(2, lon_pix_num, lat_pix_num)

    IF n_elements(sigma) EQ 0 THEN sigma=2D
    print, 're-producing image with sigma rejection ...'
    print, 'Sigma : ', sigma

    ; Wide band
    FOR pix = 0, w_pix-1 DO BEGIN
      IF (bad_flg[pix] EQ 0) THEN BEGIN
        for j = 0L, n_data-1 do begin

          i_x = fix(im_x[pix,j])
          i_y = fix(im_y[pix,j])

          if ( (i_x GE 0 AND i_y GE 0) $
              and ((i_x LT lon_pix_num) AND (i_y LT lat_pix_num)) $
              and (eg_gl_cnt[pix,j] EQ 0) $
              and (det[pix,j] NE 0D) $
              and (err[pix,j] GT 0D) ) then begin

                if (abs(det[pix,j]-coad_det0[0,i_x,i_y]) lt sigma*coad_stdev0[0,i_x,i_y]) then begin $
                coad_num[0,i_x,i_y]    = coad_num[0,i_x,i_y] + 1D
                coad_det[0,i_x,i_y]    = coad_det[0,i_x,i_y] + det[pix,j]/err[pix,j]^2
                coad_weight[0,i_x,i_y] = coad_weight[0,i_x,i_y] + 1D/err[pix,j]^2
                coad_mean[0,i_x,i_y]   = coad_mean[0,i_x,i_y] + det[pix,j]
                coad_det_sq[0,i_x,i_y] = coad_det_sq[0,i_x,i_y] + det[pix,j]^2
                endif

          endif

        endfor
      ENDIF
    ENDFOR

    ; Narrow band
    FOR pix = w_pix, n_pix-1 DO BEGIN
      IF (bad_flg[pix] EQ 0) THEN BEGIN
        for j = 0L, n_data-1 do begin

          i_x = fix(im_x[pix,j])
          i_y = fix(im_y[pix,j])

          if( (i_x GE 0 AND i_y GE 0) $
              and ((i_x LT lon_pix_num) AND (i_y LT lat_pix_num)) $
              and (eg_gl_cnt[pix,j] EQ 0) $
              and (det[pix,j] NE 0D) $
              and (err[pix,j] GT 0D) ) then begin
                if(abs(det[pix,j]-coad_det0[1,i_x,i_y]) lt sigma*coad_stdev0[1,i_x,i_y]) then begin $
                coad_num[1,i_x,i_y]    = coad_num[1,i_x,i_y] + 1D
                coad_det[1,i_x,i_y]    = coad_det[1,i_x,i_y] + det[pix,j]/err[pix,j]^2
                coad_weight[1,i_x,i_y] = coad_weight[1,i_x,i_y] + 1D/err[pix,j]^2
                coad_mean[1,i_x,i_y]   = coad_mean[1,i_x,i_y] + det[pix,j]
                coad_det_sq[1,i_x,i_y] = coad_det_sq[1,i_x,i_y] + det[pix,j]^2
                endif
            endif

        endfor
      ENDIF
    ENDFOR

; re-calculate mean and std_dev

    nz_idx = where(coad_num gt 0 and coad_weight gt 0, nz_cnt)
    print, 'Non-zero co-add number: ', nz_cnt
    if(nz_cnt ne 0) then begin
        coad_det[nz_idx] = coad_det[nz_idx]/coad_weight[nz_idx]
        coad_err[nz_idx] = sqrt(1D/coad_weight[nz_idx])
        coad_mean[nz_idx] = coad_mean[nz_idx]/coad_num[nz_idx]
        coad_det_sq[nz_idx] = coad_det_sq[nz_idx]/coad_num[nz_idx]
        coad_stdev[nz_idx] = sqrt(coad_det_sq[nz_idx]-coad_mean[nz_idx]^2)
    endif

;;;;;
; cell-to-cell rms noise in the coadd map
;;;;;
    map_mean = dblarr(2,4)
    for i = 0, 1 do begin
    p_coad_det = dblarr(lon_pix_num, lat_pix_num)
    p_coad_det[*,*] = coad_det[i,*,*]
    ;; Image Reconstruction using Pixel Mapping Method
    if keyword_set(Pix_Mapping) then begin
       print, '%% Constructing an image by Pixel Mapping method ....'
       pix_map = shift(dist(11,11), 5, 5)
       pix_map(where(pix_map le ceil(beam_size / (grid_size*7200d)))) = 1.0
       pix_map(where(pix_map gt ceil(beam_size / (grid_size*7200d)))) = 0.0
       det_cov = dblarr(lon_pix_num, lat_pix_num)
       det_cov[where(p_coad_det ne 0)] = 1d
       p_coad_det = convolve(p_coad_det, pix_map)
       det_cov = convolve(det_cov, pix_map)
       p_coad_det[where(det_cov lt 1)] = 0
       p_coad_det[where(det_cov ge 1)] = p_coad_det[where(det_cov ge 1)] / det_cov[where(det_cov ge 1)]
       det_cov = 0 & pix_map = 0
       coad_det[i, *,*] = p_coad_det
    endif
    p_nz_idx = where(p_coad_det ne 0, p_nz_cnt)
    delta = 1
    cnt = 0
    if (p_nz_cnt gt 0) then begin
        p_mean = mean(p_coad_det[p_nz_idx], /double)
        p_rms  = stddev(p_coad_det[p_nz_idx], /double)
        while(delta gt 0.05 or cnt lt 5) do begin
        p_mean0  = p_mean
        p_rms0   = p_rms
        nz_idx   = where(abs(p_coad_det[p_nz_idx]-p_mean0) lt 3*p_rms0, nz_cnt)
        p_nz_idx = p_nz_idx[nz_idx]
        if (nz_cnt gt 0) then begin
            p_mean = mean(p_coad_det[p_nz_idx], /double)
            p_rms  = stddev(p_coad_det[p_nz_idx], /double)
        endif
        delta = abs(p_rms-p_rms0)/p_rms0
        cnt += 1
        endwhile
    endif
    map_mean[i,0] = p_mean
    map_mean[i,1] = p_rms
    map_mean[i,2] = p_nz_cnt
    map_mean[i,3] = p_nz_cnt-nz_cnt
    endfor

    print, 'Mean level of coad image and RMS fluctuation after sigma rejection [MJy/sr]:'
    print, '    WIDE:   ', map_mean[0,0], map_mean[0,1]
    print, '    NARROW: ', map_mean[1,0], map_mean[1,1]
    print, 'Number of cells with non-zero values and sigma rejected cells:'
    print, '    WIDE:   ', map_mean[0,2], map_mean[0,3]
    print, '    NARROW: ', map_mean[1,2], map_mean[1,3]

;;;;;
; Calculate coordinate values of each pixels of the map
;;;;;
    lon_pixnum = findgen(lon_pix_num)+0.5
    lat_pixnum = findgen(lat_pix_num)+0.5
    map_xyarr = fltarr(2, lon_pix_num, lat_pix_num)
    FOR i = 0, lon_pix_num-1 DO BEGIN
      FOR j = 0, lat_pix_num-1 DO BEGIN
        map_xyarr[0,i,j] = lon_pixnum[i]
        map_xyarr[1,i,j] = lat_pixnum[j]
      ENDFOR
    ENDFOR

    XY2AD, reform(map_xyarr[0,*,*]), reform(map_xyarr[1,*,*]), astr, im_lon_coord, im_lat_coord

;;;;;
; Calculate max and min values
;;;;;
    coad_wide_max = max(coad_det[0,*,*], wide_max_sub, min=coad_wide_min)
    print, 'max_value in wide coad-map [MJy/sr] and max_idx:', coad_wide_max, wide_max_sub
    print, 'min_value in wide coad-map [MJy/sr]:', coad_wide_min
    wide_max_str = string(coad_wide_max)
    wide_max_str = strtrim(wide_max_str, 2)
    wide_min_str = string(coad_wide_min)
    wide_min_str = strtrim(wide_min_str, 2)

    coad_narrow_max = max(coad_det[1,*,*], narrow_max_sub, min=coad_narrow_min)
    print, 'max_value in narrow coad-map [MJy/sr] and max_idx:', coad_narrow_max, narrow_max_sub
    print, 'min_value in narrow coad-map [MJy/sr]:', coad_narrow_min
    narrow_max_str = string(coad_narrow_max)
    narrow_max_str = strtrim(narrow_max_str, 2)
    narrow_min_str = string(coad_narrow_min)
    narrow_min_str = strtrim(narrow_min_str, 2)

    grid_str = string(grid_size*3600.0)
    grid_str = strtrim(grid_str, 2)

;;;;;
; Calculate standard error of the mean for each pixel
;;;;;
    coad_stderr = coad_stdev
    idx_gt1 = where(coad_num GT 1, cnt_gt1)
    IF (cnt_gt1 GT 0) THEN BEGIN
      coad_stderr[idx_gt1] = coad_stdev[idx_gt1]/sqrt(coad_num[idx_gt1]-1)
      
      ; for pixel in which only one sample
      idx_eq1 = where(coad_num EQ 1, cnt_eq1)
      IF (cnt_eq1 GT 0) THEN coad_stderr[idx_eq1] = median(coad_stdev[idx_gt1])
    ENDIF

;;;;;
; Plot contour
;;;;;
    IF (no_display NE 1) THEN BEGIN ; If /NO_DISPLAY option was not set

    WINDOW, 4, XSIZE=1000, YSIZE=500

    device, get_decomposed=c_decomp ; Get current state of the decomposed flag
    device, decomposed=0
    loadct, 3 ; Load CT of 'Red Temperature'

    if(data_type eq 'FIS_SW') then begin

            ttitle = 'AKARI/FIS Image   '+file_header
            xyouts, 0.5, 0.95, ttitle, charsize=2.2, /normal, align=0.5

            leftmemo = 'grid:'+grid_str+'"' $
              +',  max:'+narrow_max_str+' [MJy/sr]' $
              +',  min:'+narrow_min_str+' [MJy/sr]'
            xyouts, 0.25, 0.03, leftmemo, charsize=1.2, /normal, align=0.5

            rightmemo = 'grid:'+grid_str+'"' $
              +',  max:'+wide_max_str+' [MJy/sr]' $
              +',  min:'+wide_min_str+' [MJy/sr]'
            xyouts, 0.75, 0.03, rightmemo, charsize=1.2, /normal, align=0.5

            contour, coad_det[1,*,*], nlevels=50, /fill, $
              max_value=coad_narrow_max, min_value=coad_narrow_min, $
              title='N60', charsize=1.5, $
              xtitle='LON', ytitle='LAT', $
              position=[0.1,0.18,0.45,0.88], $
              /noerase

            contour, coad_det[0,*,*], nlevels=50, /fill, $
              max_value=coad_wide_max, min_value=coad_wide_min, $
              title='Wide-S', charsize=1.5, $
              xtitle='LON', ytitle='LAT', $
              position=[0.6,0.18,0.95,0.88], $
              /noerase

        endif else begin

            ttitle = 'AKARI/FIS Image   '+file_header
            xyouts, 0.5, 0.95, ttitle, charsize=2.2, /normal, align=0.5

            leftmemo = 'grid:'+grid_str+'"' $
              +',  max:'+wide_max_str+' [MJy/sr]' $
              +',  min:'+wide_min_str+' [MJy/sr]'
            xyouts, 0.25, 0.03, leftmemo, charsize=1.2, /normal, align=0.5

            rightmemo = 'grid:'+grid_str+'"' $
              +',  max:'+narrow_max_str+' [MJy/sr]' $
              +',  min:'+narrow_min_str+' [MJy/sr]'
            xyouts, 0.75, 0.03, rightmemo, charsize=1.2, /normal, align=0.5

            contour, coad_det[0,*,*], nlevels=50, /fill, $
              max_value=coad_wide_max, min_value=coad_wide_min, $
              title='Wide-L', charsize=1.5, $
              xtitle='LON', ytitle='LAT', $
              position=[0.1,0.18,0.45,0.88], $
              /noerase

            contour, coad_det[1,*,*], nlevels=50, /fill, $
              max_value=coad_narrow_max, min_value=coad_narrow_min, $
              title='N160', charsize=1.5, $
              xtitle='LON', ytitle='LAT', $
              position=[0.6,0.18,0.95,0.88], $
              /noerase

        endelse

; Plot coadded number of samples

    !P.MULTI = [0, 2, 2]
    WSET, 0

    plot, coad_num0[0,*,*], yrange=[-1,21], PSYM=1, $
        title='initial coadded # (wide)', $
        xtitle='index', ytitle='COAD #'

    plot, coad_num0[1,*,*], yrange=[-1,21], PSYM=1, $
        title='initial coadded # (narrow)', $
        xtitle='index', ytitle='COAD #'

    sigmamemow = Strcompress(float(double(sigma)))+'-sigma regected sample # (wide)'
    sigmamemon = Strcompress(float(double(sigma)))+'-sigma regected sample # (narrow)'

    plot, coad_num0[0,*,*]-coad_num[0,*,*], yrange=[-1,11], PSYM=1, $
        title=sigmamemow, xtitle='index', ytitle='COAD #'

    plot, coad_num0[1,*,*]-coad_num[1,*,*], yrange=[-1,11], PSYM=1, $
        title=sigmamemon, xtitle='index', ytitle='COAD #'

    !P.MULTI = 0

    device, decomposed=c_decomp ; Restore decomposed flag

  ENDIF ; End of contour image and plot windows display


;;;;;
; Save IDL data file
;;;;;
  if keyword_set(mosaic) then begin
      coad_file = file_header+'_img.sav'
  endif else begin
      if (scut ne 0) then begin ; in case of scut=ON
          if (sloop ne scan) then begin
              coad_file = file_header+'_img_'+snum+'.sav'
          endif
          if (sloop eq scan) then begin
              coad_file = file_header+'_img.sav'
          endif
      endif
      if (scut eq 0) then begin ; in case of scut=OFF
          coad_file = file_header+'_img.sav'
      endif
  endelse

  save, im_lon_coord, im_lat_coord, coad_det, coad_stderr, coad_num, data_type, $
    map_mean, grid_size, hdr, filename=coad_file
  print, 'Output coadd image data in IDL format:', coad_file

;;;;;
; Save FITS files
;;;;;
  if keyword_set(mosaic) then begin
      coad_fits = file_header+'_img'
      err__fits = file_header+'_err'
      inum_fits = file_header+'_num'
  endif else begin
      if (scut ne 0) then begin ; in case of scut=ON
          if (sloop ne scan) then begin
              coad_fits = file_header+'_img_'+snum
              err__fits = file_header+'_err_'+snum
              inum_fits = file_header+'_num_'+snum
          endif
          if (sloop eq scan) then begin
              coad_fits = file_header+'_img'
              err__fits = file_header+'_err'
              inum_fits = file_header+'_num'
          endif
      endif
      if (scut eq 0) then begin ; in case of scut=OFF
          coad_fits = file_header+'_img'
          err__fits = file_header+'_err'
          inum_fits = file_header+'_num'
      endif
  endelse

  IF keyword_set(cube_fits) THEN BEGIN
  ; Create 3D data cube FITS
    w_out_fits = [ coad_det[0,*,*], coad_stderr[0,*,*], coad_num[0,*,*] ]
    w_out_fits = transpose(w_out_fits, [1,2,0])
    n_out_fits = [ coad_det[1,*,*], coad_stderr[1,*,*], coad_num[1,*,*] ]
    n_out_fits = transpose(n_out_fits, [1,2,0])

    sxaddpar, hdr, 'NAXIS',  3
    sxaddpar, hdr, 'NAXIS3', 3
    sxaddpar, hdr, 'CTYPE3', 'DataCube', 'Intensity, Error, Integrated Sample#'
    writefits, file_header+'_cube_w.fits', float(w_out_fits), hdr
    writefits, file_header+'_cube_n.fits', float(n_out_fits), hdr
    print, 'Output map data in 3D cube FITS:', file_header+'_cube_[w|n].fits'
  ENDIF ELSE BEGIN
  ; by default
    writefits, coad_fits+'_w.fits', float(reform(coad_det[0,*,*])), hdr
    writefits, coad_fits+'_n.fits', float(reform(coad_det[1,*,*])), hdr
    print, 'Output intensity map in FITS format:', coad_fits, '_[w|n].fits'

    writefits, err__fits+'_w.fits', float(reform(coad_stderr[0,*,*])), hdr
    writefits, err__fits+'_n.fits', float(reform(coad_stderr[1,*,*])), hdr
    print, 'Output error map in FITS format:', err__fits, '_[w|n].fits'

    writefits, inum_fits+'_w.fits', float(reform(coad_num[0,*,*])), hdr
    writefits, inum_fits+'_n.fits', float(reform(coad_num[1,*,*])), hdr
    print, 'Output integrated sample number map in FITS format:', inum_fits, '_[w|n].fits'
  ENDELSE


; Scan Cut Loop End
if keyword_set(mosaic) then goto, skipmosaic2
if (scut eq 0) then goto, skipscut3     ; in case of scut=OFF
endfor
skipscut3:
skipmosaic2:


error_status = 0                ; set error status

; End of this pro
end
