;+
; NAME:
;   ss_eg_slrmv
;
; PURPOSE:
;   extract slow component and stray light from time-series data 
;   by an easy way with median/boxcar smooth filter
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S.Matsuura, 2007/08/24
;   2007/08/30 S.Matsuura, Included earth avoidance angle information.
;   2007/09/03 S.Matsuura, sigma-clipped boxcar smoothing is used instead of median.
;   2007/09/06 S.Matsuura, output smooth components to be removed for all pixels.
;   2007/09/11 S.Matsuura, added loop count limit.
;   2007/09/14 S.Matsuura, NO_DISPLAY option is now available.
;-
pro ss_eg_slrmv, x, aa_mean, det, err, width_sm, n_line, n_pix, w_pix, bad_flg, $
		det_sm, err_sm, det_sm_ave, det_sm_std, sm_stat, sm_min, $
		NO_DISPLAY = no_display


    medorbxcr = 1	; by default, boxcar smoothing & sigma clipping
    if (medorbxcr eq 0) then begin


    ; extract smooth component of the time series data by median smoothing

    print,'Stray light removal: boxcar smoothing and sigma clipping method is selected.'

    det_sm = dblarr(n_pix,n_line+width_sm)
    err_sm = dblarr(n_pix,n_line+width_sm)
    det_p = dblarr(n_line+width_sm)
    err_p = dblarr(n_line+width_sm)
    hw_sm = fix(width_sm/2)

    for pix = 0, n_pix-1 do begin
    if (bad_flg[pix] eq 0) then begin
        det_p[hw_sm:n_line+hw_sm-1] = det[pix,0:n_line-1]
        det_p[0:hw_sm-1] = det[pix,0]
        det_p[n_line+hw_sm:n_line+2*hw_sm-1] = det[pix,n_line-1]
        err_p[hw_sm:n_line+hw_sm-1] = err[pix,0:n_line-1]
        err_p[0:hw_sm-1] = err[pix,0]
        err_p[n_line+hw_sm:n_line+2*hw_sm-1] = err[pix,n_line-1]
        nz_idx = where(det_p ne 0 and err_p ne 0, nz_idx_cnt)
        if (nz_idx_cnt gt 0) then begin
        det_sm[pix,nz_idx] = median(det_p[nz_idx],width_sm,/double)
        err_sm[pix,nz_idx] = 0
        endif
    endif
    det_p[*] = 0D
    err_p[*] = 0D
    endfor

    det_sm = det_sm[*,hw_sm:n_line+hw_sm-1]
    err_sm = err_sm[*,hw_sm:n_line+hw_sm-1]
    det_sm = det_sm[*,0:n_line-1]
    err_sm = err_sm[*,0:n_line-1]


    endif else begin


    ; extract smooth component of the time series data by boxcar smoothing and sigma clipping

    print,'Stray light removal: boxcar smoothing and sigma clipping method is selected.'

    det_sm = dblarr(n_pix,n_line)
    err_sm = dblarr(n_pix,n_line)
    det_p = dblarr(n_line)
    err_p = dblarr(n_line)
    det_res = dblarr(n_line)

    for pix = 0, n_pix-1 do begin
    if (bad_flg[pix] eq 0) then begin
        det_p = det[pix,*]
        err_p = err[pix,*]

	clip_cnt = 1
	loop_cnt = 0
	while (clip_cnt gt 0) do begin
        nz_idx = where(det_p ne 0 and err_p ne 0, nz_idx_cnt)
        if (nz_idx_cnt gt 0) then begin

	    det_sm[pix,nz_idx] = smooth(det_p[nz_idx],width_sm,/edge_truncate,/NaN)
            err_sm[pix,nz_idx] = 0

	    ; 3-sigma clipping
	    det_res = det_p - det_sm[pix,*]
	    det_res_std = stddev(det_res,/NaN)
	    clip_idx = where(abs(det_res) gt 3*det_res_std, clip_cnt)
	    if (clip_cnt gt 0) then begin
		det_p[clip_idx] = det_sm[pix,clip_idx]
	    endif

        endif
	loop_cnt = loop_cnt + 1
	if (loop_cnt gt 50) then clip_cnt = 0
	endwhile
	; print, 'Loop counts: ', loop_cnt

    endif
    det_p[*] = 0D
    err_p[*] = 0D
    endfor

    endelse


    ; calculate the minimum brightness

    ; define variables
    det_sm_ave0 = dblarr(2,n_line)
    det_sm_std0 = dblarr(2,n_line)
    det_sm_ave = dblarr(2,n_line)
    det_sm_std = dblarr(2,n_line)
    sm_min = dblarr(2)
    sm_stat = dblarr(2,2)
    fine_pix = where(bad_flg eq 0, fine_pix_cnt)
    w_idx = where(fine_pix lt w_pix, w_cnt)
    n_idx = where(fine_pix ge w_pix, n_cnt)

    ; pixel average
    for j = 0, n_line-1 do begin
        det_sm_ave0[0,j] = mean(det_sm[fine_pix[w_idx],j],/double,/NaN)
        det_sm_std0[0,j] = stddev(det_sm[fine_pix[w_idx],j],/double,/NaN)
        det_sm_ave0[1,j] = mean(det_sm[fine_pix[n_idx],j],/double,/NaN)
        det_sm_std0[1,j] = stddev(det_sm[fine_pix[n_idx],j],/double,/NaN)
    endfor

    ; 2-sigma clipping
    for j = 0, n_line-1 do begin
	clip_w_idx = where( abs(det_sm[fine_pix[w_idx],j]-det_sm_ave0[0,j]) $
				lt 2*det_sm_std0[0,j], clip_w_cnt)
	if (clip_w_cnt gt 0) then begin
            det_sm_ave[0,j] = mean(det_sm[fine_pix[w_idx[clip_w_idx]],j],/double,/NaN)
            det_sm_std[0,j] = stddev(det_sm[fine_pix[w_idx[clip_w_idx]],j],/double,/NaN)
	endif
	clip_n_idx = where( abs(det_sm[fine_pix[n_idx],j]-det_sm_ave0[1,j]) $
				lt 2*det_sm_std0[1,j], clip_n_cnt)
	if (clip_n_cnt gt 0) then begin
            det_sm_ave[1,j] = mean(det_sm[fine_pix[n_idx[clip_n_idx]],j],/double,/NaN)
            det_sm_std[1,j] = stddev(det_sm[fine_pix[n_idx[clip_n_idx]],j],/double,/NaN)
	endif
    endfor

    ; smoothing
    det_sm_ave[0,*] = smooth(det_sm_ave[0,*],width_sm,/edge_truncate)
    det_sm_ave[1,*] = smooth(det_sm_ave[1,*],width_sm,/edge_truncate)

    ; minimum and maximum EAA (earth avoidance angle)
    aa_min = min(aa_mean, aa_min_idx, max=aa_max, subscript_max=aa_max_idx)
    
    if (aa_min gt 0 and aa_max gt 0) then begin

	print, 'Minimum Earth AA & idx: ', aa_min, aa_min_idx
	print, 'Maximum Earth AA & idx: ', aa_max, aa_max_idx

	; a minimal point near the largest EAA
	r_idx = where(aa_mean gt aa_max-(aa_max-aa_min)/10, r_cnt)
    	if (r_cnt gt 0) then begin
	    sm_min[0] = min(det_sm_ave[0,r_idx], min_idx_w)
	    sm_min[1] = min(det_sm_ave[1,r_idx], min_idx_n)
	    result = moment(det_sm_ave[0,r_idx],SDEV=stdev,/double,/NaN)
	    sm_stat[0,0] = result[0]
	    sm_stat[0,1] = stdev
	    result = moment(det_sm_ave[1,r_idx],SDEV=stdev,/double,/NaN)
	    sm_stat[1,0] = result[0]
	    sm_stat[1,1] = stdev
	    if (min_idx_w lt 10 or min_idx_w gt r_cnt-11) then begin
		sm_min[0] = sm_stat[0,0]
		print, 'DC drift exist! The minimal value is replaced by the mean (Wide).'
	    endif
	    if (min_idx_n lt 10 or min_idx_n gt r_cnt-11) then begin
		sm_min[1] = sm_stat[1,0]
		print, 'DC drift exist! The minimal value is replaced by the mean (Narrow).'
	    endif
	endif

    endif else begin

	print, 'Minimum/Maximum Earth AA is unknown.'

	; a minimul point near the mid time
	n_x = n_elements(x)
	r_idx = where(x gt x[0]+(x[n_x-1]-x[0])*0.4 and x lt x[0]+(x[n_x-1]-x[0])*0.6, r_cnt)
    	if (r_cnt gt 0) then begin
	    sm_min[0] = min(det_sm_ave[0,r_idx], min_idx_w)
	    sm_min[1] = min(det_sm_ave[1,r_idx], min_idx_n)
	    result = moment(det_sm_ave[0,r_idx],SDEV=stdev,/double,/NaN)
	    sm_stat[0,0] = result[0]
	    sm_stat[0,1] = stdev
	    result = moment(det_sm_ave[1,r_idx],SDEV=stdev,/double,/NaN)
	    sm_stat[1,0] = result[0]
	    sm_stat[1,1] = stdev
	    if (min_idx_w lt 10 or min_idx_w gt r_cnt-11) then begin
		;sm_min[0] = mean(det_sm_ave[0,r_idx], /double, /NaN)
		sm_min[0] = sm_stat[0,0]
		print, 'DC drift exist! The minimal value is replaced by the mean (Wide).'
	    endif
	    if (min_idx_n lt 10 or min_idx_n gt r_cnt-11) then begin
		;sm_min[1] = mean(det_sm_ave[1,r_idx], /double, /NaN)
		sm_min[1] = sm_stat[1,0]
		print, 'DC drift exist! The minimal value is replaced by the mean (Narrow).'
	    endif
	endif

    endelse

    aa_mean_min_w = aa_mean[r_idx[min_idx_w]]
    aa_mean_min_n = aa_mean[r_idx[min_idx_n]]
    print, 'Earth AA at the brightness minimum (Wide) & idx: ', aa_mean_min_w, r_idx[min_idx_w]
    print, 'Earth AA at the brightness minimum (Narrow) & idx: ', aa_mean_min_n, r_idx[min_idx_n]

    ; smooth components to be subtracted
    det_sm[fine_pix[w_idx],*] = det_sm[fine_pix[w_idx],*] - sm_min[0]
    det_sm[fine_pix[n_idx],*] = det_sm[fine_pix[n_idx],*] - sm_min[1]

IF ~keyword_set(no_display) then begin
    ; plot pix-averaged slow component and pix-variance
    ; for WIDE
    plot, x, det_sm_ave[0,*]
    oplot, x, det_sm_std0[0,*]
    oplot, x, det_sm_std[0,*]
    ; for i=0, n_elements(w_idx)-1 do begin
    ;    oplot, x, det_sm[fine_pix[w_idx[i]],*],psym=1
    ; endfor

    ; for NARROW
    plot, x, det_sm_ave[1,*]
    oplot, x, det_sm_std0[1,*]
    oplot, x, det_sm_std[1,*]
    ; for i=0, n_elements(n_idx)-1 do begin
    ;    oplot, x, det_sm[fine_pix[n_idx[i]],*],psym=1
    ; endfor

ENDIF


end
