; $Id: ss_run_ss.pro,v 1.42 2007/09/13 10:48:53 makiuti Exp $
;+
; NAME:
;   SS_RUN_SS
;
; PURPOSE:
;   This is a script to run Slow-Scan tools and create JPEG image files.
;
; CALLING SEQUENCE:
;
;   ss_run_ss, target_dir [, /FILE_INPUT, (/SW or /LW), (/INIT or /MAP), $
;              /NO_DISPLAY, TAG_NAME=string, $
;              /AOCU_ADS, /MK_TXT_FILE, PLOT_PIXEL=value, $
;              N_RAMP_DIV=n_ramp_div, /RAMP_COR_OFF, /GLITCH_RMV_OFF, $
;              /LOCAL_FLAT, T_FLAT_START=value, T_FLAT_END=value, $
;              /MEDIAN_FILTER, /SMOOTH_FILTER, WIDTH_FILTER=value, $
;              /MASK_OFF, /RESP_COR_OFF, /TRANS_COR, /SL_RMV, $
;              SIGMA=value, T_START=value, T_END=value, /SCUT, $
;              /USE_BAD, BAD_THRSHLD=value, $
;              GRID_SW(LW)=value, LON(LAT)_CENTER=value, LON(LAT)_SIZE=value, $
;              (/ECLIPTIC or /GALACTIC), /CUBE_FITS, $
;              /OUTLINE, CONTOUR_NUM=value, /COORD_GRID_ON, $
;              /DEL_FILES, /VERSION_COPY]
;
;     'target_dir' is a string of target directory name.
;     It can include wildcards (ex. '*') to pick multiple directories up.
;     The target directories should contain TSD files of SW and LW of which 
;     names are FIS_(SW|LW)*.fits.gz.
;
;     ('target_dir' is construed as a string of target file name with /FILE_INPUT option.)
;
;     /FILE_INPUT    : Use to input by file not directory.
;     /SW(/LW)       : Process only SW(LW) data.
;     /INIT(/MAP)    : Do only ss_init_proc.pro(ss_make_map.pro)
;     /NO_DISPLAY    : Suppress plot display during processes.
;     PLOT_PIXEL     : Pixel number to be plotted (default = 5).
;     TAG_NAME       : String of tag to stick to all the created files.
;     /AOCU_ADS      : Use AOCU positional information instead of GADS info.
;     /MK_TXT_FILE   : Make intermediate text files.
;     N_RAMP_DIV     : Divide a ramp by this value for fitting (default = 1).
;     /RAMP_COR_OFF  : No ramp curve correction applied.
;     /GLITCH_RMV_OFF: No glitch filter applied.
;     /LOCAL_FLAT    : Use blank sky in the current data itself for flat-fielding.
;     T_FLAT_START   : Beginning time of the data to be used for /LOCAL_FLAT
;     T_FLAT_END     : End time of the data to be used for /LOCAL_FLAT
;     /MEDIAN_FILTER : Apply a median filter to the data.
;     /SMOOTH_FILTER : Similar to MEDIAN_FILTER, but uses boxcar smoothing.
;     WIDTH_FILTER   : Width of Median/Smooth filter
;     /MASK_OFF      : Do not mask Calibration sequences (shutter & Cal. lamp)
;     /RESP_COR_OFF  : No responsivity correction applied.
;     /TRANS_COR     : Transient correction for data after shutter close/open
;     /SL_RMV        : Subtract stray light component caused by the Earth
;     SIGMA          : Sigma clipping threshold value to make a map.
;     T_START        : Beginning time of the data to be used for map making.
;     T_END          : End time of the data to be used for map making.
;     /SCUT          : Generate individual images per one way scan.
;     /USE_BAD       : Use bad-pixel-flags in the TSD to mask the data.
;     BAD_THRSHLD    : Set threshold value to reject pixels with unusual responsivity.
;     GRID_SW(LW)    : Grid size of image map [arcsec]
;     LON(LAT)_CENTER: Center position of image map.
;     LON(LAT)_SIZE  : Size of image map [degree]
;     /ECLIPTIC(/GALACTIC) : Make image map in Ecliptic(GALACTIC) coordinate.
;     /PIX_MAPPING   : Use Pixel Mapping method.
;     /CUBE_FITS     : Output data as a Data Cube FITS file.
;     /OUTLINE       : Display outline on a JPEG image.
;     CONTOUR_NUM    : Number of contours which are drawn on a JPEG image.
;     /COORD_GRID_ON : Display coordinate grid on a JPEG image.
;     /DEL_FILES     : Delete output files (*.txt, *.sav, *.fits) except for JPEG files.
;     /VERSION_COPY  : Copy a file indicating tool's version into each target directory.
;
; EXAMPLE
;
;   ss_run_ss, 'AKARI_FIS*'
;
; MODIFICATION HISTORY:
;   2006/12/22 Written by Makiuti, S.
;   2006/12/28 Added KEEP and DISP_ON options
;   2006/12/29 debugged by Mai Shirahata
;   2006/12/30 Makiuti, S., Added /SW(/LW), /OUTLINE, CONTOUR_NUM options.
;                Write a string of version info on a image. A few improvements.
;   2006/12/30 Mai Shirahata, Added /LOCAL_FLAT, /MEDIAN_FILTER options.
;   2007/01/05 Makiuti, S., Improved error handling. When an error was returned 
;                from the Slow-Scan pipelines, stop processes of the current file 
;                and go to the next file.
;   2007/01/10 Mai Shirahata, Added SIGMA, T_START, T_END options.
;   2007/01/10 Mai Shirahata, Added /INIT, /MAP options.
;   2007/01/17 Makiuti, S., Added /VERSION_COPY option.
;   2007/01/18 Mai Shirahata, Added /SKY_SUB option.
;   2007/01/18 Makiuti, S., Cancel /VERSION_COPY option when data are FTS.
;   2007/01/19 Makiuti, S., A bug-fix and improved message display.
;   2007/01/19 Makiuti, S., Removed /DISP_ON and /KEEP options.
;                           Added /NO_DISPLAY adn /DEL_FILES options. 
;   2007/01/23 M.Shirahata, change output file names (.dat --> .txt)
;   2007/01/24 Makiuti, S., Changed the name of output JPEG files. (*.jpg -> *_img.jpg)
;                           Tightened condition judgements of file names to be removed.
;   2007/01/28 S.Matsuura, included N_RAMP_DIV option
;   2007/01/29 Makiuti, S. Added /USE_BAD option for ss_init_proc.pro
;   2007/01/30 Makiuti, S. Bug-fix (enabled /USE_BAD option)
;   2007/01/30 S.Matsuura, Added T_FLT_ST and T_FLT_EN options to specify time range 
;               of data taken for flat fielding.
;   2007/02/01 M.Shirahata, change option name (median_flt --> median_filter)
;   2007/02/03 S.Matsuura, added new parameter of bad data rejection threshold.
;   2007/02/05 S.Matsuura, added "width_median" option as window size of median filter.
;   2007/02/06 M.Shirahata, add scut option.
;   2007/02/06 M.Shirahata, add /SCUT option.
;   2007/02/07 M.Shirahata, change option names (t_flt_st -> t_flat_start, t_flt_en -> t_flat_end)
;   2007/02/08 S.Matsuura, added grid size input option.
;   2007/02/11 I.Yamamura, added /RAMP_COR_OFF, /GLITCH_RMV_OFF, PLOT_PIXEL,
;                          /SL_RMV, /MASK_OFF, /RESP_COR_OFF,
;                          AOCU -> AOCU_ADS for consistency with
;                          ss_init_proc.pro and ss_make_map.pro
;   2007/02/19 Makiuti, S. Fixed a problem of formatting in the colorbar in JPEG.
;   2007/03/16 I. Yamamura /AOCU_ADS -> AOCU_ADS=aocu_ads in the calling sequence of ss_init_proc.
;   2007/06/26 Makiuti, S. Not create text files by default. Added /MK_TXT_FILE and TAG_NAME options
;   2007/07/02 Makiuti, S. Added LON(LAT)_CENTER, LON(LAT)_SIZE options to ss_make_map.pro
;   2007/08/17 Makiuti, S. Fixed a bug of /DEL_FILES option.
;   2007/08/23 S.Matsuura, TRNS_COR_OFF option: By default, transient correction for data after shutter-open is done.
;   2007/08/31 Makiuti, S. Update accompanying revision of ss_coad_image.pro
;                          Added /Coord_Grid_On, /CUBE_FITS options
;   2007/09/04 Makiuti, S. Changed to accept input by file (Added /FILE_INPUT option)
;   2007/09/06 S.Matsuura, New option: /SMOOTH for high-pass filtering
;   2007/09/06 S.Matsuura, Transient correction is OFF by default
;   2007/09/11 Makiuti, S. Added /Pix_Mapping option which was implemented by Woong-Seob for ss_coad_image
;   2007/09/13 Makiuti, S. Removed /SKY_SUB option, and slightly revised the usage
;
;-
PRO SS_RUN_SS, $
      targetdir, $
      FILE_INPUT=file_input, $
      SW=sw, LW=lw, $
      INIT=init, MAP=map, $
      NO_DISPLAY=nodisp, TAG_NAME=tag_name, $
      AOCU_ADS=aocu_ads, $
      MK_TXT_FILE = mk_txt_file, $
      PLOT_PIXEL = plot_pixel, $
      N_RAMP_DIV=n_int, $
      RAMP_COR_OFF = ramp_cor_off, $
      GLITCH_RMV_OFF = glitch_rmv_off, $
      LOCAL_FLAT=local_flat, $
      T_FLAT_START=t_flat_start, T_FLAT_END=t_flat_end, $
      MEDIAN_FILTER=median_filter, SMOOTH_FILTER=smooth_filter, WIDTH_FILTER=width_filter, $
      MASK_OFF=mask_off, RESP_COR_OFF=resp_cor_off, $
      TRANS_COR=trans_cor, SL_RMV=sl_rmv, $
      SIGMA=sigma, T_START=t_start, T_END=t_end, SCUT=scut, $
      USE_BAD=use_bad, BAD_THRSHLD=bad_thrshld, $
      GRID_SW=grid_sw, GRID_LW=grid_lw, $
      LON_CENTER=lon_center, LAT_CENTER=lat_center, $
      LON_SIZE=lon_size, LAT_SIZE=lat_size, $
      ECLIPTIC = ecliptic, GALACTIC = galactic, $
      Pix_Mapping = Pix_Mapping, CUBE_FITS = cube_fits, $
      OUTLINE=outline, CONTOUR_NUM=cont_num, COORD_GRID_ON=coord_grid_on, $
      DEL_FILES=del_files, VERSION_COPY=version_copy


  ; Display the usage
  IF (n_params() EQ 0) THEN BEGIN
    print, "# USAGE #"
    print, "  ss_run_ss, target_dir [, /FILE_INPUT, (/SW or /LW), (/INIT or /MAP), $"
    print, "             /NO_DISPLAY, TAG_NAME=string, $"
    print, "             /AOCU_ADS, /MK_TXT_FILE, PLOT_PIXEL=value, $"
    print, "             N_RAMP_DIV=n_ramp_div, /RAMP_COR_OFF, /GLITCH_RMV_OFF, $"
    print, "             /LOCAL_FLAT, T_FLAT_START=value, T_FLAT_END=value, $"
    print, "             /MEDIAN_FILTER, /SMOOTH_FILTER, WIDTH_FILTER=value, $"
    print, "             /MASK_OFF, /RESP_COR_OFF, /TRANS_COR, /SL_RMV, $"
    print, "             SIGMA=value, T_START=value, T_END=value, /SCUT, $""
    print, "             /USE_BAD, BAD_THRSHLD=value, $"
    print, "             GRID_SW(LW)=value, LON(LAT)_CENTER=value, LON(LAT)_SIZE=value, $"
    print, "             (/ECLIPTIC or /GALACTIC), /PIX_MAPPING, /CUBE_FITS, $"
    print, "             /OUTLINE, CONTOUR_NUM=value, /COORD_GRID_ON, $"
    print, "             /DEL_FILES, /VERSION_COPY]"
    print, ""
    print, "    'target_dir' is a string of target directory name."
    print, "    It can include wildcards (ex. '*') to pick multiple directories up."
    print, "    The target directories should contain TSD files of SW and LW of which "
    print, "    names are FIS_(SW|LW)*.fits.gz."
    print, ""
    print, "    ('target_dir' is construed as a string of target file name with /FILE_INPUT option.)"
    print, ""
    print, "    /FILE_INPUT    : Use to input by file not directory."
    print, "    /SW(/LW)       : Process only SW(LW) data."
    print, "    /INIT(/MAP)    : Do only ss_init_proc.pro(ss_make_map.pro)"
    print, "    /NO_DISPLAY    : Suppress plot display during processes."
    print, "    PLOT_PIXEL     : Pixel number to be plotted (default = 5)."
    print, "    TAG_NAME       : String of tag to stick to all the created files."
    print, "    /AOCU_ADS      : Use AOCU positional information instead of GADS info."
    print, "    /MK_TXT_FILE   : Make intermediate text files."
    print, "    N_RAMP_DIV     : Divide a ramp by this value for fitting (default = 1)."
    print, "    /RAMP_COR_OFF  : No ramp curve correction applied."
    print, "    /GLITCH_RMV_OFF: No glitch filter applied."
    print, "    /LOCAL_FLAT    : Use blank sky in the current data itself for flat-fielding."
    print, "    T_FLAT_START   : Beginning time of the data to be used for /LOCAL_FLAT"
    print, "    T_FLAT_END     : End time of the data to be used for /LOCAL_FLAT"
    print, "    /MEDIAN_FILTER : Apply a median filter to the data."
    print, "    /SMOOTH_FILTER : Similar to MEDIAN_FILTER, but uses boxcar smoothing."
    print, "    WIDTH_FILTER   : Width of Median/Smooth filter"
    print, "    /MASK_OFF      : Do not mask Calibration sequences (shutter & Cal. lamp)"
    print, "    /RESP_COR_OFF  : No responsivity correction applied."
    print, "    /TRANS_COR     : Transient correction for data after shutter close/open"
    print, "    /SL_RMV        : Subtract stray light component caused by the Earth"
    print, "    SIGMA          : Sigma clipping threshold value to make a map."
    print, "    T_START        : Beginning time of the data to be used for map making."
    print, "    T_END          : End time of the data to be used for map making."
    print, "    /SCUT          : Generate individual images per one way scan."
    print, "    /USE_BAD       : Use bad-pixel-flags in the TSD to mask the data."
    print, "    BAD_THRSHLD    : Set threshold value to reject pixels with unusual responsivity."
    print, "    GRID_SW(LW)    : Grid size of image map [arcsec]"
    print, "    LON(LAT)_CENTER: Center position of image map."
    print, "    LON(LAT)_SIZE  : Size of image map [degree]"
    print, "    /ECLIPTIC(/GALACTIC) : Make image map in Ecliptic(GALACTIC) coordinate."
    print, "    /PIX_MAPPING   : Use Pixel Mapping method."
    print, "    /CUBE_FITS     : Output data as a Data Cube FITS file."
    print, "    /OUTLINE       : Display outline on a JPEG image."
    print, "    CONTOUR_NUM    : Number of contours which are drawn on a JPEG image."
    print, "    /COORD_GRID_ON : Display coordinate grid on a JPEG image."
    print, "    /DEL_FILES     : Delete output files (*.txt, *.sav, *.fits) except for JPEG files."
    print, "    /VERSION_COPY  : Copy a file indicating tool's version into each target directory."
    print, ""
    print, "  ex1) IDL> ss_run_ss, 'AKARI_FIS*'"
    print, "  ex2) IDL> ss_run_ss, '.' ; target_dir is the current directory"
    return
  ENDIF

  ; Initialize
  IF ~keyword_set(AOCU_ADS) THEN aocu_ads = 0
  IF ~keyword_set(RAMP_COR_OFF) THEN ramp_cor_off = 0
  IF ~keyword_set(GLITCH_RMV_OFF) THEN glitch_rmv_off = 0
  IF (n_elements(plot_pixel) EQ 0) THEN BEGIN
    plot_pixel = 5
    print, 'PLOT_PIXEL (pixel number to be plotted) is set to 5'
  ENDIF
  error = 0 ; error status which is returned from slow-scan tools


  IF ~keyword_set(FILE_INPUT) THEN BEGIN ; Search directories by default

    ; Make an string array of target directory names
    dirname_array = FILE_SEARCH(targetdir, /FOLD_CASE, /TEST_DIRECTORY)

    ; Show the total number of directories which were picked up
    num_dir = N_Elements(dirname_array)
    print, 'Search for directories "'+targetdir+'"'
    IF (num_dir EQ 1) THEN BEGIN
      IF (dirname_array EQ '') THEN BEGIN
        print, 'No directory was found.'
        return
      ENDIF
    ENDIF
    print, Strcompress(String(num_dir)) + ' directories were found.'

    IF keyword_set(SW) THEN BEGIN
      j_str = 0
      j_end = 0
    ENDIF ELSE IF keyword_set(LW) THEN BEGIN
      j_str = 1
      j_end = 1
    ENDIF ELSE BEGIN
      j_str = 0
      j_end = 1
    ENDELSE

  ENDIF ELSE BEGIN ; Search files instead of directories
    tsdfile = FILE_SEARCH(targetdir, /FOLD_CASE) ; In this case, targetdir is file name
    num_files = N_Elements(tsdfile)
    print, 'Search for files "'+targetdir+'"'
    IF (num_files EQ 1) THEN BEGIN
      IF (tsdfile EQ '') THEN BEGIN
        print, 'No file was found.'
        return
      ENDIF
    ENDIF
    print, Strcompress(String(num_files)) + ' files were found.'

    dirname_array = '.'
    num_dir = 1
    j_str = 0
    j_end = 0
  ENDELSE

  ;;;;;
  ; Loop for directories
  ;;;;;
  FOR i = 0, num_dir-1 DO BEGIN

    FOR j = j_str, j_end DO BEGIN ; Loop for SW/LW

      IF ~keyword_set(FILE_INPUT) THEN BEGIN ; by default (input by directory)

        ; Find target TSD files
        IF (j EQ 0) THEN BEGIN
          tsdfile = FILE_SEARCH(dirname_array[i]+'/FIS_SW*.fits.gz', /FOLD_CASE)
          IF (n_elements(tsdfile) EQ 1) THEN BEGIN
            IF (tsdfile EQ '') THEN BEGIN
              print, ' No SW file is found in the "'+dirname_array[i]+'"'
              continue
            ENDIF
          ENDIF
          print, Strcompress(String(n_elements(tsdfile))) + $
                 ' SW files are found in the "'+dirname_array[i]+'"'
          title_w = 'Wide-S'
          title_n = 'N60'
        ENDIF ELSE BEGIN
          tsdfile = FILE_SEARCH(dirname_array[i]+'/FIS_LW*.fits.gz', /FOLD_CASE)
          IF (n_elements(tsdfile) EQ 1) THEN BEGIN
            IF (tsdfile EQ '') THEN BEGIN
              print, ' No LW file is found in the "'+dirname_array[i]+'"'
              continue
            ENDIF
          ENDIF
          print, Strcompress(String(n_elements(tsdfile))) + $
                 ' LW files are found in the "'+dirname_array[i]+'"'
          title_w = 'Wide-L'
          title_n = 'N160'
        ENDELSE
      
      ENDIF
      
      FOR k = 0, n_elements(tsdfile)-1 DO BEGIN ; Loop for each file of SW or LW

        IF (strlen(tsdfile[k]) EQ 0) THEN BEGIN
          continue
        ENDIF

        ; Basename of a TSD file without the suffix ; ex. 'FIS_SW_20061201013000'
        tsd_basename = FILE_BASENAME(tsdfile[k], '.fits.gz')
        ; If TAG_NAME was set,
        IF ((n_elements(tag_name) GT 0) AND (size(tag_name, /type) EQ 7)) THEN BEGIN
          tsd_basename = tsd_basename+tag_name
        ENDIF

        ;;;
        ; Do Slow-Scan pipelines
        ;;;

        ; Do ss_init_proc
        if keyword_set(map) then goto, init_skip

        ss_init_proc, tsdfile[k], $
            NO_DISPLAY=nodisp, ERROR_STATUS=error, $
            AOCU_ADS=aocu_ads, RAMP_COR_OFF = ramp_cor_off, $
            GLITCH_RMV_OFF = glitch_rmv_off, USE_BAD=use_bad, $
            PLOT_PIXEL=plot_pixel, MK_TXT_FILE = mk_txt_file, $
            TAG_NAME = tag_name, $
            N_RAMP_DIV=n_int, T_FLAT_START=t_flat_start, T_FLAT_END=t_flat_end

        ; Error handling
        IF (error EQ 1) THEN BEGIN
          message, 'SS_INIT_PROC stopped with an error!!!', /continue
          msg = '**** Skipped '+tsdfile[k]+' ****'
          message, msg, /continue
          print, ''
          continue
        ENDIF

        init_skip:

        ; Do ss_make_map
        if keyword_set(init) then goto, map_skip

        IF ~keyword_set(local_flat) THEN local_flat = 0
        IF ~keyword_set(median_filter) THEN median_filter = 0
        IF ~keyword_set(smooth_filter) THEN smooth_filter = 0
        ;IF ~keyword_set(sky_sub) THEN sky_sub = 0
        IF ~keyword_set(sl_rmv) THEN sl_rmv = 0
        IF ~keyword_set(mask_off) THEN mask_off = 0
        IF ~keyword_set(resp_cor_off) THEN resp_cor_off = 0
        IF ~keyword_set(trans_cor) THEN trans_cor = 0
        IF ~keyword_set(scut) THEN scut = 0

        ss_make_map, dirname_array[i]+'/'+tsd_basename+'_pr.sav', $
            T_START=t_start, T_END=t_end, $
            NO_DISPLAY=nodisp, MK_TXT_FILE = mk_txt_file, $
            LOCAL_FLAT=local_flat, $
            MEDIAN_FILTER=median_filter, SMOOTH_FILTER=smooth_filter, WIDTH_FILTER=width_filter, $
            SKY_SUB=sky_sub, SL_RMV=sl_rmv, $
            MASK_OFF=mask_off, RESP_COR_OFF=resp_cor_off, TRANS_COR=trans_cor, $
            SIGMA=sigma, SCUT=scut, $
            BAD_THRSHLD=bad_thrshld, ERROR_STATUS=error, $
            GRID_SW=grid_sw, GRID_LW=grid_lw, $
            LON_CENTER = lon_center, LAT_CENTER = lat_center, $
            LON_SIZE = lon_size, LAT_SIZE = lat_size, $
            ECLIPTIC = ecliptic, GALACTIC = galactic, $
            CUBE_FITS = cube_fits, Pix_Mapping = Pix_Mapping

        ; Error handling
        IF (error EQ 1) THEN BEGIN
          message, 'SS_MAKE_MAP stopped with an error!!!', /continue
          msg = '**** Skipped '+tsdfile[k]+' ****'
          message, msg, /continue
          print, ''
          continue
        ENDIF

        ;;;
        ; Make JPEG image file
        ;;;

        ; Settings
        !p.multi = [0,2,1]
        LOADCT, 3

        ; Change device to the Z graphics
        currentDevice = !D.Name
        Set_Plot, 'Z', /COPY
        ; Configure the Z graphics buffer
        Device, Set_Resolution=[800,400], Z_Buffer=0
        Erase

        ; Draw image maps (Wide and Narrow)
        restore, dirname_array[i]+'/'+tsd_basename+'_img.sav'

        map_size = size(coad_det)
        lon_haba = float(map_size[2])
        lat_haba = float(map_size[3])
        aspect = lat_haba/lon_haba ; aspect ratio

        max_x_haba = 0.35 ; max width in the normalized coordinate system
        max_y_haba = 0.69 ; max height in the normalized coordinate system

        ; difine width and height to be drawn
        IF (aspect GE 1) THEN BEGIN
          y_haba = max_y_haba
          x_haba = max_x_haba/aspect
        ENDIF ELSE IF (aspect LT 1) THEN BEGIN
          y_haba = max_y_haba * aspect
          x_haba = max_x_haba
        ENDIF

        ; difine position for Wide-band image
        x_center0 = 0.275
        y_center0 = 0.595
        
        pos0_x0 = x_center0 - x_haba/2
        pos0_x1 = x_center0 + x_haba/2
        pos0_y0 = y_center0 - y_haba/2
        pos0_y1 = y_center0 + y_haba/2
        pos0 = [pos0_x0, pos0_y0, pos0_x1, pos0_y1]

        ; difine position for Narrow-band image
        x_center1 = 0.775
        y_center1 = 0.595

        pos1_x0 = x_center1 - x_haba/2
        pos1_x1 = x_center1 + x_haba/2
        pos1_y0 = y_center1 - y_haba/2
        pos1_y1 = y_center1 + y_haba/2
        pos1 = [pos1_x0, pos1_y0, pos1_x1, pos1_y1]

        ;;;; Make contour images
        contour, coad_det[0,*,*], nlevels=50, /fill, $
          xstyle=3, ystyle=3, $
          xtickint=round(lon_haba*10/4)/10.0, title=title_w, xtit='LON', ytit='LAT', $
          position = pos0
        colorbar, range=[min(coad_det[0,*,*]), max(coad_det[0,*,*])], title='[MJy/sr]', $
          position = [0.1, 0.08, 0.45, 0.10], charsize=0.7, format='(f7.1)'

        contour, coad_det[1,*,*], nlevels=50, /fill, $
          xstyle=3, ystyle=3, $
          xtickint=round(lon_haba*10/4)/10.0, title=title_n, xtit='LON', ytit='LAT', $
          position = pos1, /NoErase
        colorbar, range=[min(coad_det[1,*,*]), max(coad_det[1,*,*])], title='[MJy/sr]', $
          position = [0.6, 0.08, 0.95, 0.10], chars=0.7, format='(f7.1)'

        ; Display outline
        IF keyword_set(outline) THEN BEGIN
          contour, coad_det[1,*,*], levels=[0.001], $
            color=FSC_Color('olive', !D.Table_Size-255), $
            xstyle=7, ystyle=7, $
            position = pos1, /NoErase
          contour, coad_det[0,*,*], levels=[0.001], $
            color=FSC_Color('olive', !D.Table_Size-255), $
            xstyle=7, ystyle=7, $
            position = pos0, /NoErase
        ENDIF

        ; Display contours on a image
        IF (n_elements(cont_num) GT 0) THEN BEGIN
          contour, coad_det[0,*,*], nlevels=cont_num, $
            color=FSC_Color('olive', !D.Table_Size-255), $
            xstyle=7, ystyle=7, position = pos0, /NoErase
          contour, coad_det[1,*,*], nlevels=cont_num, $
            color=FSC_Color('olive', !D.Table_Size-255), $
            xstyle=7, ystyle=7, position = pos1, /NoErase
        ENDIF

        ; Display coordinate grid
        IF (n_elements(coord_grid_on) GT 0) THEN BEGIN
          im_size = size(im_lat_coord)
          lon_grid = rebin(indgen(im_size[1]), im_size[1], im_size[2])
          lat_grid = transpose(rebin(indgen(im_size[2]), im_size[2], im_size[1]))

          c_labels = make_array(10, value=1, /integer)

          intv = (max(im_lon_coord)-min(im_lon_coord))/3.0
          levels = (findgen(3)+0.5)*intv + min(im_lon_coord)
          contour, im_lon_coord, lon_grid, lat_grid, levels=levels, c_labels=c_labels, $
            color=FSC_Color('SkyBlue', !D.Table_Size-254), $
            xstyle=7, ystyle=7, position = pos0, /NoErase

          intv = (max(im_lat_coord)-min(im_lat_coord))/3.0
          levels = (findgen(3)+0.5)*intv + min(im_lat_coord)
          contour, im_lat_coord, lon_grid, lat_grid, levels=levels, c_labels=c_labels, $
            color=FSC_Color('SkyBlue', !D.Table_Size-254), $
            xstyle=7, ystyle=7, position = pos0, /NoErase

          intv = (max(im_lon_coord)-min(im_lon_coord))/3.0
          levels = (findgen(3)+0.5)*intv + min(im_lon_coord)
          contour, im_lon_coord, lon_grid, lat_grid, levels=levels, c_labels=c_labels, $
            color=FSC_Color('SkyBlue', !D.Table_Size-254), $
            xstyle=7, ystyle=7, position = pos1, /NoErase

          intv = (max(im_lat_coord)-min(im_lat_coord))/3.0
          levels = (findgen(3)+0.5)*intv + min(im_lat_coord)
          contour, im_lat_coord, lon_grid, lat_grid, levels=levels, c_labels=c_labels, $
            color=FSC_Color('SkyBlue', !D.Table_Size-254), $
            xstyle=7, ystyle=7, position = pos1, /NoErase
        ENDIF

        ; Write version info. on a image
        xyouts, 0, 0, 'Map created at '+systime()+' with Slow-Scan Tools', charsize=0.8, /Normal

        ; Take a content of the current device
        snapshot = TVRD()
        TVLCT, r, g, b, /Get
        ; Finish with the Z graphics buffer
        Device, Z_Buffer=1
        Set_Plot, currentDevice
        ; Construct the 24-bit image
        image24 = BytArr(3, 800, 400)
        image24[0,*,*] = r[snapshot]
        image24[1,*,*] = g[snapshot]
        image24[2,*,*] = b[snapshot]

        ; Create JPEG file
        out_imfile = dirname_array[i]+'/'+tsd_basename+'_img.jpg'
        Write_JPEG, out_imfile, image24, True=1

        ; Post-process
        !p.multi = 0

        map_skip:

        ;;;
        ; Delete temporal files
        ;;;
        IF keyword_set(del_files) THEN BEGIN
          delfiles = FILE_SEARCH(dirname_array[i]+'/'+'FIS_?W*.txt', /FOLD_CASE)
          IF (size(delfiles, /DIM) NE 0) THEN FILE_DELETE, delfiles
          delfiles = FILE_SEARCH(dirname_array[i]+'/'+'FIS_?W*.sav', /FOLD_CASE)
          IF (size(delfiles, /DIM) NE 0) THEN FILE_DELETE, delfiles
          delfiles = FILE_SEARCH(dirname_array[i]+'/'+'FIS_?W*.fits', /FOLD_CASE)
          IF (size(delfiles, /DIM) NE 0) THEN FILE_DELETE, delfiles
          print, 'Deleted output files (*.txt, *.sav, *.fits) except for JPEG files.'
        ENDIF

      ENDFOR

    ENDFOR

    ; Copy the file which indicates the version of the current Slow-Scan tools 
    ;   to each target directory.
    IF (keyword_set(version_copy) AND (error NE 1)) THEN BEGIN
      fisdr_root = GETENV('FISDR_ROOT')
      ret = file_test(fisdr_root+'/slowscan/pro/VERSION')
      IF (ret EQ 1) THEN file_copy, fisdr_root+'/slowscan/pro/VERSION', dirname_array[i], /OVERWRITE
    ENDIF

  ENDFOR

END
