;+
; NAME:
;   ss_trans_corr
;
; PURPOSE:
;   correct transient effect after shutter close
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S. Matsuura, 2007/08/22
;
;   2007/08/28 S.Matsuura, changed time range for the linear fitting.
;   2007/09/05 S.Matsuura, smooth filtering instead of median filtering 
;			is used to extract slow component.
;   2007/09/06 S.Matsuura, changed name: ss_trns_corr -> ss_trans_corr 
;   2007/09/06 S.Matsuura, iteration for linear fitting to caluculate base line. 
;			time range for the correction is shortened.
;   2007/09/07 S.Matsuura, bug fix; bad_pix handling. 
;   2007/09/09 S.Matsuura, median filter is initially used for the loop to obtain smooth curve. 
;   2007/09/11 S.Matsuura, changed position of loop count limit. 
;
;-
pro ss_trans_corr, x_ave, coeff, error, sht_flg_tot, n_int, rst_int, bad_flg, $
            n_pix, coeff_corr, NO_DISP=no_disp

IF (n_elements(no_disp) EQ 0) THEN no_disp=0

; print input parameter
    print, 'Correcting the data affected by shutter close/open ...'

; Check the parameters and initialize
    ; help, x_ave, coeff, error, sht_flg_tot, n_int, rst_int, $
    ;        n_pix, coeff_corr

; main procedure

	sht_op_idx = where(sht_flg_tot eq 1, sht_op_idx_cnt)
	; print, 'Number of shutter open: ', sht_op_idx_cnt

	sht_cls_idx = where(sht_flg_tot eq -1, sht_cls_idx_cnt)
	; print, 'Number of shutter close: ', sht_cls_idx_cnt

	if (sht_op_idx_cnt gt 0 and sht_cls_idx_cnt gt 0) then begin

	for i_pix = 0, n_pix-1 do begin
	if (bad_flg[i_pix] eq 0) then begin

	    ; for the initial shutter-open

		; define variables
		cor_num1 = fix(120/rst_int*n_int)	; 120s data after sht_opn
		cor_num2 = fix(30/rst_int*n_int)	; 30s time window to be corrected
		w_sm = fix(15/rst_int*n_int)		; 15s time window for smoothing

		; index definition
		cor_start1_idx = sht_op_idx[0]
		cor_end1_idx = sht_op_idx[0] + cor_num1 - 1
		t_idx = dblarr(cor_num1)
		t_idx = x_ave[cor_start1_idx:cor_end1_idx]

		; raw data input
		coeff1 = dblarr(cor_num1)
		coeff1p = dblarr(cor_num1)
		coeff1[0:cor_num1-1] = coeff[i_pix,cor_start1_idx:cor_end1_idx,1]
		coeff1p = coeff1


		; generate average baseline to be obtained by correction
		base1 = dblarr(cor_num1)
		base1_s = dblarr(cor_num1)
		base1[0:cor_num1-1] = alog(coeff[i_pix,cor_start1_idx:cor_end1_idx,1])
		base1[0:cor_num2-1] = 0

		clip_cnt = 1
		loop_cnt = 0
		while (clip_cnt gt 0) do begin
		fit_idx = where(base1 ne 0, fit_idx_cnt)
		if (fit_idx_cnt gt 0) then begin

		   base_fit = linfit(t_idx[fit_idx], base1[fit_idx], /double, SIGMA=sigma)
		   base1_s[cor_num2:cor_num1-1] = base_fit[0] $
				+ base_fit[1]*t_idx[cor_num2:cor_num1-1]

		   ; 3-sigma clipping
		   delta = base1 - base1_s
		   delta_std = stddev(delta,/NaN)
		   clip_idx = where(abs(delta) gt 3*delta_std, clip_cnt)
		   if (clip_cnt gt 0) then begin
			base1[clip_idx] = base1_s[clip_idx]
		   endif

		endif else begin

		   base_fit = dblarr(2)
		   base_fit[*] = 0
		   clip_cnt = 0

		endelse
		loop_cnt = loop_cnt + 1
		if (loop_cnt gt 50) then clip_cnt = 0
		endwhile
		; print, 'Loop counts: ', loop_cnt

		base1[0:cor_num1-1] = exp(base_fit[0] + base_fit[1]*t_idx[0:cor_num1-1])


		; smooth filtering to extract slow component
		smooth1 = dblarr(cor_num1)
		clip_cnt = 1
		loop_cnt = 0
		while (clip_cnt gt 0) do begin
		nz_idx = where(coeff1p ne 0, nz_idx_cnt)
		if (nz_idx_cnt gt 0) then begin

		   if (loop_cnt eq 0) then begin
			; median filter to get initial smooth curve
			smooth1[nz_idx] = median(coeff1p[nz_idx],w_sm, /double)
		   endif else begin
			; smooth filter for the next loop
			smooth1[nz_idx] = smooth(coeff1p[nz_idx],w_sm,/edge_truncate,/NaN)
		   endelse

		   ; 3-sigma clipping
		   delta = coeff1p - smooth1
		   delta_std = stddev(delta,/NaN)
		   clip_idx = where(abs(delta) gt 3*delta_std, clip_cnt)
		   if (clip_cnt gt 0) then begin
			coeff1p[clip_idx] = smooth1[clip_idx]
		   endif

		endif else begin

		   smooth1[*] = 0
		   clip_cnt = 0

		endelse
		loop_cnt = loop_cnt + 1
		if (loop_cnt gt 50) then clip_cnt = 0
		endwhile
		; print, 'Loop counts: ', loop_cnt

		; generate corrected data
		coeff1_corr = dblarr(cor_num1)
		coeff1_corr = coeff1 + base1 - smooth1
		coeff_corr[i_pix,cor_start1_idx:cor_end1_idx,1] = coeff1_corr[0:cor_num1-1]


	   ; for the second and subsequent shutter-open events

	   for i = 1, sht_op_idx_cnt-1 do begin

		; define variables
		cor_num0 = fix(30/rst_int*n_int)	; 30s data before sht_cls
		cor_num1 = fix(40/rst_int*n_int)	; 40s data after sht_opn
		cor_num_t = cor_num0 + cor_num1		; 70s data in total
		cor_num2 = fix(30/rst_int*n_int)	; 30s time window to be corrected
		w_sm = fix(15/rst_int*n_int)		; 15s time window for smoothing

		; index definition
		cor_start0_idx = sht_cls_idx[i-1] - cor_num0
		cor_end0_idx = sht_cls_idx[i-1] - 1
		cor_start1_idx = sht_op_idx[i]
		cor_end1_idx = sht_op_idx[i] + cor_num1 - 1
		t_idx = dblarr(cor_num_t)
		t_idx[0:cor_num0-1] = x_ave[cor_start0_idx:cor_end0_idx]
		t_idx[cor_num0:cor_num_t-1] = x_ave[cor_start1_idx:cor_end1_idx]

		; raw data input
		coeff1 = dblarr(cor_num1)
		coeff1p = dblarr(cor_num1+w_sm)	; with index mergin
		coeff1[0:cor_num1-1] = coeff[i_pix,cor_start1_idx:cor_end1_idx,1]
		hw_sm = fix(w_sm/2)
		coeff1p[hw_sm:cor_num1+hw_sm-1] = coeff1[0:cor_num1-1]
		coeff1p[0:hw_sm-1] = coeff1[0:hw_sm-1]
		coeff1p[cor_num1+w_sm-hw_sm:cor_num1+w_sm-1] = coeff1[cor_num1-hw_sm:cor_num1-1]


		; generate average baseline to be obtained by correction
		base = dblarr(cor_num_t)
		base_s = dblarr(cor_num_t)
		base1 = dblarr(cor_num1)
		base[0:cor_num0-1] = coeff[i_pix,cor_start0_idx:cor_end0_idx,1]
		base[cor_num0:cor_num_t-1] = coeff[i_pix,cor_start1_idx:cor_end1_idx,1]
		base[cor_num0:cor_num0+cor_num2-1] = 0

		clip_cnt = 1
		loop_cnt = 0
		while (clip_cnt gt 0) do begin
		fit_idx = where(base ne 0, fit_idx_cnt)
		if (fit_idx_cnt gt 0) then begin

		   base_fit = linfit(t_idx[fit_idx], base[fit_idx], /double, SIGMA=sigma)
		   base_s[0:cor_num0-1] = base_fit[0] $
				+ base_fit[1]*t_idx[0:cor_num0-1]
		   base_s[cor_num0+cor_num2:cor_num_t-1] = base_fit[0] $
				+ base_fit[1]*t_idx[cor_num0+cor_num2:cor_num_t-1]

		   ; 3-sigma clipping
		   delta = base - base_s
		   delta_std = stddev(delta,/NaN)
		   clip_idx = where(abs(delta) gt 3*delta_std, clip_cnt)
		   if (clip_cnt gt 0) then begin
			base[clip_idx] = base_s[clip_idx]
		   endif

		endif else begin

		   base_fit = dblarr(2)
		   base_fit[*] = 0
		   clip_cnt = 0

		endelse
		loop_cnt = loop_cnt + 1
		if (loop_cnt gt 50) then clip_cnt = 0
		endwhile
		; print, 'Loop counts: ', loop_cnt

		base1[0:cor_num1-1] = base_fit[0] $
				+ base_fit[1]*t_idx[cor_num0:cor_num_t-1]


		; smooth filtering to extract slow component
		smooth1 = dblarr(cor_num1)
		smooth1r = dblarr(cor_num1+hw_sm)	; with index mergin
		clip_cnt = 1
		loop_cnt = 0
		while (clip_cnt gt 0) do begin
		nz_idx = where(coeff1p ne 0, nz_idx_cnt)
		if (nz_idx_cnt gt 0) then begin

		   if (loop_cnt eq 0) then begin
			; median filter to get initial smooth curve
			smooth1r[nz_idx] = median(coeff1p[nz_idx],w_sm, /double)
		   endif else begin
			; smooth filter for the next loop
			smooth1r[nz_idx] = smooth(coeff1p[nz_idx],w_sm,/edge_truncate,/NaN)
		   endelse

		   ; 3-sigma clipping
		   delta = coeff1p - smooth1r
		   delta_std = stddev(delta,/NaN)
		   clip_idx = where(abs(delta) gt 3*delta_std, clip_cnt)
		   if (clip_cnt gt 0) then begin
			coeff1p[clip_idx] = smooth1r[clip_idx]
		   endif

		endif else begin

		   smooth1r[*] = 0
		   clip_cnt = 0

		endelse
		loop_cnt = loop_cnt + 1
		if (loop_cnt gt 50) then clip_cnt = 0
		endwhile
		; print, 'Loop counts: ', loop_cnt

		smooth1[0:cor_num1-1] = smooth1r[hw_sm:cor_num1+hw_sm-1]

		; generate corrected data
		coeff1_corr = dblarr(cor_num1)
		coeff1_corr = coeff1 + base1 - smooth1
		coeff_corr[i_pix,cor_start1_idx:cor_end1_idx,1] = coeff1_corr[0:cor_num1-1]

	   endfor

	endif
	endfor

	endif


; plot data

IF (no_disp NE 1) THEN BEGIN ; If /NO_DISP option was not set

    window, 0
    plot_pix = 30
    plot, x_ave, coeff[plot_pix-1,*,1], title='transient correction', $
	yrange = [0,0.003], $
        xtitle='# of samples', ytitle='output [V/s]', $
        ticklen=1.0, xgridstyle=1, ygridstyle=1, $
        position=[0.1,0.1,0.9,0.9]

    oplot, x_ave, coeff_corr[plot_pix-1,*,1],psym=1,color=128

ENDIF

end
