; $Id: base64__define.pro,v 1.3 2006/03/23 06:23:22 baba Exp $
;
; Copyright(C) 2004, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   Base64
;
; PURPOSE:
;   Base64 - Encoding and decoding of base64 strings
;
; CATEGORY:
;   Data conversion
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, November 2004.
;
;-

;----------------------------------------------------------------------------
; Methods to be public
;----------------------------------------------------------------------------

;+
; CALLING SEQUENCE:
;   encoded = encode_base64(original)
;
; INPUTS:
;   original	Original BYTARR
;
; OUTPUTS:
;   encoded	Base64 encoded BYTARR
;
; DESCRIPTION:
;   Encode data by calling the encode_base64() function.
;   This function takes a single argument which is the bytarr to
;   decode and returns the decoded bytarr data.
;   The returned encoded string is broken into lines of no more than
;   76 characters each and it will end with $eol unless it is empty.
;   Pass an empty string as second argument if you do not want the
;   encoded string to be broken into lines.
;
; EXAMPLES:
;   If you want to encode a large file, you should encode it in chunks
;   that are a multiple of 57 bytes.  This ensures that the base64 lines
;   line up and that you do not end up with padding in the middle. 57
;   bytes of data fills one complete base64 line (76 == 57*4/3):
;
;      base64 = OBJ_NEW('base64')
;      OPENR, lun, '/var/log/wtmp', /Get_LUN
;      WHILE NOT EOF(lun) DO BEGIN
;          buf = BYTARR(60*57, /NOZERO)
;          READU, lun, buf
;          PRINT, base64->encode_base64(buf)
;      ENDWHILE
;      FREE_LUN, lun
;      OBJ_DESTROY, base64
;
;   or if you know you have enough memory
;
;      base64 = OBJ_NEW('base64')
;      OPENR, lun, '/var/log/wtmp', /Get_LUN
;      res = FSTAT(lun)
;      buf = BYTARR(res.SIZE, /NOZERO)
;      READU, lun, buf
;      PRINT, base64->encode_base64(buf)
;      FREE_LUN, lun
;      OBJ_DESTROY, base64
;
;-

FUNCTION base64::encode_base64, orig
    MESSAGE, 'Called.', /INFORMATIONAL

    ; base64 alphabet
    b64c="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
    v2b = BYTE(b64c)

    ; make padding
    npad = 0
    IF N_ELEMENTS(orig) MOD 3 NE 0 THEN BEGIN
        npad = 3 - (N_ELEMENTS(orig) MOD 3)
        padding = REPLICATE(0B, npad) ; NULL byte padding
        orig = [orig, padding]
    ENDIF

    ; Reform original array
    orig = REFORM(orig, 3, N_ELEMENTS(orig)/3, /OVERWRITE)

    ; Encode chars to value (0-63)
    value = BYTARR(4, N_ELEMENTS(orig)/3)
    value[0,*] = BYTE(                                 ISHFT(orig[0,*],-2) )
    value[1,*] = BYTE( ISHFT(ISHFT(orig[0,*],6),-2) OR ISHFT(orig[1,*],-4) )
    value[2,*] = BYTE( ISHFT(ISHFT(orig[1,*],4),-2) OR ISHFT(orig[2,*],-6) )
    value[3,*] = BYTE( ISHFT(ISHFT(orig[2,*],2),-2) )
    value = REFORM(value, N_ELEMENTS(value), /OVERWRITE)
    self->undefine, orig ; clear memory

    ; Convert array from value (0-63, and 64) to base64 alphabet
    encoded = v2b[value]
    self->undefine, value ; clear memory

    ; Make padding with '='
    IF npad EQ 1 OR npad EQ 2 THEN BEGIN
        encoded[N_ELEMENTS(encoded)-1] = BYTE('=')
        IF npad EQ 2 THEN encoded[N_ELEMENTS(encoded)-2] = BYTE('=')
    ENDIF

    RETURN, encoded
END

;+
; CALLING SEQUENCE:
;   decoded = decode_base64(encoded)
;
; INPUTS:
;   encoded	Base64 encoded BYTARR
;
; OUTPUTS:
;   decoded	Decoded BYTARR
;
; DESCRIPTION:
;   Decode a base64 string by calling the decode_base64() function.
;   This function takes a single argument which is the bytarr to
;   decode and returns the decoded bytarr data.
;
;   Any character not part of the 65-character base64 subset is
;   silently ignored.  Characters occurring after a '=' padding
;   character are never decoded.
;
;   If the length of the string to decode, after ignoring non-base64
;   chars, is not a multiple of 4 or if padding occurs too early, then
;   a warning is generated if IDL is running under !QUIET = 0.
;
;-

FUNCTION base64::decode_base64, encoded
    MESSAGE, 'Called.', /INFORMATIONAL

    ; A 65-character subset of US-ASCII is used, enabling 6 bits to be
    ; represented per printable character. (The extra 65th character, "=",
    ; is used to signify a special processing function.)
    ;
    ; b64c="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
    ;   ASCII  VALUE  BYTE(c)
    ;    A-Z    0-25  65-90
    ;    a-z   26-51  97-122
    ;    0-9   52-61  48-57  
    ;    +/=   62-64  43,47,61
    ;
    ;               Array for convertion from BYTE(c) to VALUE
    ;        0     2     4     6     8    10    12    14    16    18
    b2v = BYTE([ $
            -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, $
            -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, $
            -1,-1,-1,62,-1,-1,-1,63,52,53,54,55,56,57,58,59,60,61,-1,-1, $
            -1,64,-1,-1,-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14, $
            15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,-1,26,27,28, $
            29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48, $
            49,50,51,-1,-1,-1,-1,-1 ])

    ; Convert array from base64 alphabet to value (-1, 0-63, and 64)
    value = b2v[encoded]
    self->undefine, encoded ; clear memory

    ; Any character not part of the 65-character base64 subset is
    ; silently ignored.
;    ind = WHERE(value EQ -1, count, COMPLEMENT=c_ind)
;    IF count NE 0 THEN value = value[c_ind]
;    self->undefine, c_ind ; clear memory

    ; The number of characters to decode is not a multiple of 4.  Legal
    ; base64 data should be padded with one or two "=" characters to make
    ; its length a multiple of 4.  The decoded result will anyway be as if
    ; the padding was there.
    IF N_ELEMENTS(value) MOD 4 NE 0 THEN BEGIN
        MESSAGE, 'Premature end of base64 data', /INFORMATIONAL
        ; make padding...
        npad = 4 - (N_ELEMENTS(value) MOD 4)
        padding = REPLICATE(64B, npad) ; Introduced: original
        ; padding = MAKE_ARRAY(npad, /BYTE, VALUE=64B) ; Introduced: 4.0
        ; padding = BYTARR(npad)
        ; REPLICATE_INPLACE, padding, 64B ; Introduced: 5.1
        value = [value, padding]
    END

    ; Reform as base64 quartet
    value = REFORM(value, 4, N_ELEMENTS(value)/4, /OVERWRITE)

    ; The '=' padding character occurs as the first or second character
    ; in a base64 quartet.
    IF WHERE(value[0:1,*] EQ 64B) NE -1 THEN BEGIN
        MESSAGE, 'Premature padding of base64 data', /INFORMATIONAL
    ENDIF

    ; Decode decoded chars
    dims = SIZE(value, /DIMENSIONS)
    decoded = BYTARR(3, dims[1])
    decoded[0,*] = BYTE( ISHFT(value[0,*], 2) OR ISHFT(value[1,*], -4) )
    decoded[1,*] = BYTE( ISHFT(value[1,*], 4) OR ISHFT(value[2,*], -2) )
    decoded[2,*] = BYTE( ISHFT(value[2,*], 6) OR       value[3,*]      )
    decoded = REFORM(decoded, N_ELEMENTS(decoded), /OVERWRITE)

    ; check padding and trim
    n = N_ELEMENTS(value)
    n2 = N_ELEMENTS(decoded)
    IF value[n-1] EQ 64 AND value[n-2] EQ 64 THEN BEGIN
        IF value[n-3] EQ 64 THEN decoded = decoded[0:n2-3] $
        ELSE decoded = decoded[0:n2-2]
    ENDIF

    self->undefine, value ; clear memory

    ; Return decoded value (BYTARR or STRING)
    RETURN, decoded
END

; Making an IDL Variable Undefined
; http://www.dfanning.com/tips/variable_undefine.html
; http://www.dfanning.com/programs/undefine.pro
PRO base64::undefine, varname
    IF (N_Elements(varname) NE 0) THEN tempvar = Temporary(varname)
END



;----------------------------------------------------------------------------
; Methods for test
;----------------------------------------------------------------------------

FUNCTION base64::test
    COMPILE_OPT hidden
    MESSAGE, 'Called.', /INFORMATIONAL

;     str = 'VGhpcyBpcyBhIHNlY3JldCAhCg=='
;     print, str, ' : ', STRLEN(str)
;     print, BYTE(str)
;     dstr = self->decode_base64(BYTE(str))
;     help, dstr
;     print, BYTE(dstr)
;     print, STRING(dstr)

; ;    RETURN, 0

;     file = 'encode.txt'
;     OPENR, lun, file, /Get_LUN
;     res = FSTAT(lun)
;     encoded = BYTARR(res.SIZE, /NOZERO)
;     READU, lun, encoded
;     FREE_LUN, lun
    
;     decoded = self->decode_base64(encoded)
;     help, decoded
    
;     print, STRING(decoded)
;     print, decoded
; ;    RETURN, 0

;     str = "Kore ha HIMITSU!!!"
; ;    print, "new test"
; ;    str = "This is a secret !" + STRING(10B)
;     help, strlen(str), str
;     encoded = self->encode_base64(BYTE(str))
;     help, encoded
;     print, encoded, STRING(encoded)

    OPENR, lun, '/var/log/wtmp', /Get_LUN

;     WHILE NOT EOF(lun) DO BEGIN
;         buf = BYTARR(60*57, /NOZERO)
;         READU, lun, buf, TRANSFER_COUNT=count
;         encoded = self->encode_base64(buf)
;         print, encoded
;     ENDWHILE

    res = FSTAT(lun)
    buf = BYTARR(res.SIZE, /NOZERO)
    READU, lun, buf
    encoded = self->encode_base64(buf)
    help, encoded
    decoded = self->encode_base64(encoded)
    help, decoded

    FREE_LUN, lun

END

;----------------------------------------------------------------------------
; Methods to define the class structure
;----------------------------------------------------------------------------

FUNCTION base64::INIT
    COMPILE_OPT hidden
    MESSAGE, 'Called.', /INFORMATIONAL
    RETURN, 1 ; Do not return 0
END

PRO base64::CLEANUP
    COMPILE_OPT hidden
    MESSAGE, 'Called.', /INFORMATIONAL
END

PRO base64__define
    !QUIET = 1 ; Set 0 when you need verbose messages for debug.

    MESSAGE, 'Called.', /INFORMATIONAL
    ; Not intended to be called directly by regular users
    COMPILE_OPT hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { base64, $
                   dummy: '' }
END

;----------------------------------------------------------------------------
; End of file
;----------------------------------------------------------------------------
